@@ -108,7 +108,7 @@ void switch_vidmode (Display *display, int screen, int res_x, int res_y)
   int best_dist = -1;
   int best_index = -1;
 
-  DPRINTF ("vidmode %i %i\n", res_x, res_y);
+  RAISE (MSG_DEBUG, "vidmode %i %i", res_x, res_y);
   if (!has_vidmode (display)) return;
   modecount = vidmodelines (display, screen, &modesinfo);
   for (i = 0; i < modecount; i++) {
@@ -124,7 +124,7 @@ void switch_vidmode (Display *display, int screen, int res_x, int res_y)
     }
   }
   if (best_dist >= 0) {
-    DPRINTF ("vidmode switch %i\n", best_index);
+    RAISE (MSG_DEBUG, "vidmode switch %i", best_index);
     XF86VidModeSwitchToMode (display, screen, modesinfo [best_index]);
   }
 }
@@ -190,7 +190,7 @@ void center_window (Display *display, int screen, Window window,
   int junkint;
   XF86VidModeModeLine modeline;
 
-  DPRINTF ("center 0x%lx %i,%i\n", window, res_x, res_y);
+  RAISE (MSG_DEBUG, "center 0x%lx %i,%i", window, res_x, res_y);
   if (window == None || !has_vidmode (display)) return;
   XGetGeometry (display, window, &root, &x, &y, &width, &height, 
 		&borderw, &depth);
