@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: back_client.c,v 1.10 2002/03/24 17:42:48 dthierbach Exp $
+ * $Id: back_client.c,v 1.22 2003/10/10 16:16:57 dthierbach Exp $
  *
  * Contents:
  *
@@ -63,7 +63,7 @@ char* pool_addFree (char *s)
 {
   int l, r, i;
 
-  DPRINTF ("pool %s ", s);
+  RAISE (MSG_DEBUG, "pool %s ", s);
   if (!s) return NULL;
   l = 0; r = pool_size-1;
   while (l <= r) {
@@ -74,17 +74,20 @@ char* pool_addFree (char *s)
     if (c < 0) r = i-1; else 
     if (c > 0) l = i+1; else
     {
-      DPRINTF ("found\n"); 
+      RAISE (MSG_DEBUG, "found"); 
       xfree (s); 
       return pool_list [i];
     }
   }
   /* Must insert between r and l */
-  if (r < 0) r = 0; /* sanitize for DPRINTF */
-  if (pool_list && r >= 0) DPRINTF (" %i=%s ", r, pool_list[r]);
-  DPRINTF ("*");
-  if (pool_list && l < pool_size) DPRINTF (" %i=%s ", l, pool_list[l]);
-  DPRINTF ("\n");
+#if 0
+  if (r < 0) r = 0; /* sanitize for debug messages */
+  if (pool_list && r >= 0) 
+    RAISE (MSG_DEBUG, " %i=%s ", r, pool_list[r]);
+  RAISE (MSG_DEBUG, "*");
+  if (pool_list && l < pool_size) 
+    RAISE (MSG_DEBUG, " %i=%s ", l, pool_list[l]);
+#endif
   pool_size++;
   pool_list = xrealloc (pool_list, pool_size * sizeof (char *));
   i = l+1;
@@ -115,7 +118,7 @@ void bcl_openCard (CardPtr card)
   CardPtr c;
   int i, index;
 
-  DPRINTF ("bcl_open\n");
+  RAISE (MSG_DEBUG, "bcl_open");
   bcl_card = card;
   bcl_openPipes ();
   /* convert card to index */
@@ -132,7 +135,7 @@ void bcl_openCard (CardPtr card)
 
 void bcl_closeCard (void)
 {
-  DPRINTF ("bcl_close\n");
+  RAISE (MSG_DEBUG, "bcl_close");
   pipeWriteCmd (pipe_out, PCmd_CloseCard);
   pipeWriteArgs (pipe_out, 0);
   bcl_closePipes ();
@@ -140,7 +143,7 @@ void bcl_closeCard (void)
 
 void bcl_setHeads (int main, int tv, int video)
 {
-  DPRINTF ("bcl_setHeads %i %i %i\n", main, tv, video);
+  RAISE (MSG_DEBUG, "bcl_setHeads %i %i %i", main, tv, video);
   pipeWriteCmd (pipe_out, PCmd_SetHeads);
   pipeWriteArgs (pipe_out, 3, sizeof(int), &main, sizeof(int), &tv, 
 		 sizeof(int), &video);
@@ -148,7 +151,7 @@ void bcl_setHeads (int main, int tv, int video)
 
 void bcl_getHeads (int *main, int *tv, int *video, int *max) 
 {
-  DPRINTF ("bcl_getHeads\n");
+  RAISE (MSG_DEBUG, "bcl_getHeads");
   pipeWriteCmd (pipe_out, PCmd_GetHeads);
   pipeWriteArgs (pipe_out, 0);
   pipeReadCmd (pipe_in);
@@ -158,7 +161,7 @@ void bcl_getHeads (int *main, int *tv, int *video, int *max)
 
 void bcl_getHeadDev (int head, int *devFlags) 
 {
-  DPRINTF ("bcl_getHeadDev\n");
+  RAISE (MSG_DEBUG, "bcl_getHeadDev");
   pipeWriteCmd (pipe_out, PCmd_GetHeadDev);
   pipeWriteArgs (pipe_out, 1, sizeof (head), &head);
   pipeReadCmd (pipe_in);
@@ -169,7 +172,7 @@ void bcl_probeChips (void)
 {
   ChipPtr chip, del;
 
-  DPRINTF ("bcl_probe\n");
+  RAISE (MSG_DEBUG, "bcl_probe");
   chip = bcl_card->chips; 
   while (chip) {
     del = chip;
@@ -188,7 +191,7 @@ void bcl_setChip (ChipPtr chip, Bool init)
   ChipPtr c;
   int i, index;
 
-  DPRINTF ("bcl_setChip %s %i\n", chip->name, init);
+  RAISE (MSG_DEBUG, "bcl_setChip %s %i", chip->name, init);
   /* convert chip to index */
   i = index = 0;
   for (c = bcl_card->chips; c; c = c->next) {
@@ -201,14 +204,14 @@ void bcl_setChip (ChipPtr chip, Bool init)
 
 void bcl_setSettings (TVSettings *set)
 {
-  DPRINTF ("bcl_setSettings\n");
+  RAISE (MSG_DEBUG, "bcl_setSettings");
   pipeWriteCmd (pipe_out, PCmd_SetSettings);
   pipeWriteArgs (pipe_out, 1, sizeof(TVSettings), set);
 }
 
 void bcl_getSettings (TVSettings *set)
 {
-  DPRINTF ("bcl_getSettings\n");
+  RAISE (MSG_DEBUG, "bcl_getSettings");
   pipeWriteCmd (pipe_out, PCmd_GetSettings);
   pipeWriteArgs (pipe_out, 0);
   pipeReadCmd (pipe_in);
@@ -217,14 +220,14 @@ void bcl_getSettings (TVSettings *set)
 
 void bcl_setMode (TVRegs *regs)
 {
-  DPRINTF ("bcl_setMode\n");
+  RAISE (MSG_DEBUG, "bcl_setMode");
   pipeWriteCmd (pipe_out, PCmd_SetMode);
   pipeWriteArgs (pipe_out, 1, sizeof(TVRegs), regs);
 }
 
 void bcl_getMode (TVRegs *regs)
 {
-  DPRINTF ("bcl_getMode\n");
+  RAISE (MSG_DEBUG, "bcl_getMode");
   pipeWriteCmd (pipe_out, PCmd_GetMode);
   pipeWriteArgs (pipe_out, 0);
   pipeReadCmd (pipe_in);
@@ -233,14 +236,14 @@ void bcl_getMode (TVRegs *regs)
 
 void bcl_setModeSettings (TVRegs *regs, TVSettings *set)
 {
-  DPRINTF ("bcl_setModeSettings\n");
+  RAISE (MSG_DEBUG, "bcl_setModeSettings");
   pipeWriteCmd (pipe_out, PCmd_SetModeSettings);
   pipeWriteArgs (pipe_out, 2, sizeof(TVRegs), regs, sizeof(TVSettings), set);
 }
 
 void bcl_setTestImage (TVEncoderRegs *tv, TVSettings *set)
 {
-  DPRINTF ("bcl_setTestImage\n");
+  RAISE (MSG_DEBUG, "bcl_setTestImage");
   pipeWriteCmd (pipe_out, PCmd_SetTestImage);
   pipeWriteArgs (pipe_out, 2, sizeof(TVEncoderRegs), tv, 
 		 sizeof(TVSettings), set);
@@ -250,7 +253,7 @@ long bcl_getStatus (int index)
 {
   long l;
 
-  DPRINTF ("bcl_getStatus\n");
+  RAISE (MSG_DEBUG, "bcl_getStatus");
   pipeWriteCmd (pipe_out, PCmd_GetStatus);
   pipeWriteArgs (pipe_out, 1, sizeof(index), &index);
   pipeReadCmd (pipe_in);
@@ -262,12 +265,12 @@ TVConnect bcl_getConnection (void)
 {
   TVConnect c;
 
-  DPRINTF ("bcl_getConnection\n");
+  RAISE (MSG_DEBUG, "bcl_getConnection");
   pipeWriteCmd (pipe_out, PCmd_GetConnection);
   pipeWriteArgs (pipe_out, 0);
   pipeReadCmd (pipe_in);
   pipeReadArgs (pipe_in, 1, sizeof(c), &c);
-  DPRINTF ("bcl_getConnection got %i\n", c);
+  RAISE (MSG_DEBUG, "bcl_getConnection got %i", c);
   return c;
 }
 
@@ -276,7 +279,7 @@ int bcl_listModes (TVSystem system, TVMode *(list[]))
   TVMode *m, *r;
   int n, i, c;
 
-  DPRINTF ("bcl_listModes %i\n", system);
+  RAISE (MSG_DEBUG, "bcl_listModes %i", system);
   pipeWriteCmd (pipe_out, PCmd_ListModes);
   pipeWriteArgs (pipe_out, 1, sizeof(system), &system);
   pipeReadCmd (pipe_in);
@@ -298,7 +301,7 @@ Bool bcl_findBySize (TVSystem system, int xres, int yres, char *size,
 {
   int n;
 
-  DPRINTF ("bcl_findBySize %i %i,%i %s\n", system, xres, yres, size);
+  RAISE (MSG_DEBUG, "bcl_findBySize %i %i,%i %s", system, xres, yres, size);
   pipeWriteCmd (pipe_out, PCmd_FindBySize);
   pipeWriteArgs (pipe_out, 4, sizeof(system), &system, 
 		 sizeof(xres), &xres, sizeof(yres), &yres,
@@ -317,7 +320,7 @@ Bool bcl_findByOverscan (TVSystem system, int xres, int yres,
 {
   int n;
  
-  DPRINTF ("bcl_findByOC %i %i,%i\n", system, xres, yres);
+  RAISE (MSG_DEBUG, "bcl_findByOC %i %i,%i", system, xres, yres);
   pipeWriteCmd (pipe_out, PCmd_FindByOverscan);
   pipeWriteArgs (pipe_out, 5, sizeof(system), &system, 
 		 sizeof(xres), &xres, sizeof(yres), &yres,
@@ -333,7 +336,7 @@ Bool bcl_findByOverscan (TVSystem system, int xres, int yres,
 
 void bcl_initSharedView (int *view_x, int *view_y)
 {
-  DPRINTF ("bcl_initSharedView\n");
+  RAISE (MSG_DEBUG, "bcl_initSharedView");
   pipeWriteCmd (pipe_out, PCmd_InitSharedView);
   pipeWriteArgs (pipe_out, 0);
   pipeReadCmd (pipe_in);
@@ -344,7 +347,7 @@ Bool bcl_getTwinView (int *view_x, int *view_y)
 {
   Bool result;
 
-  DPRINTF ("bcl_getTwinVie\n");
+  RAISE (MSG_DEBUG, "bcl_getTwinVie");
   pipeWriteCmd (pipe_out, PCmd_GetTwinView);
   pipeWriteArgs (pipe_out, 0);
   pipeReadCmd (pipe_in);
@@ -357,7 +360,7 @@ Bool bcl_adjustViewport (int flags, int *view_x, int *view_y)
 {
   Bool result;
 
-  DPRINTF ("bcl_adjustViewport\n");
+  RAISE (MSG_DEBUG, "bcl_adjustViewport");
   pipeWriteCmd (pipe_out, PCmd_AdjustView);
   pipeWriteArgs (pipe_out, 3, sizeof(int), &flags, 
 		 sizeof(int), view_x, sizeof(int), view_y);
@@ -372,7 +375,7 @@ Bool bcl_serviceViewportCursor (int flags, int cursor_x, int cursor_y,
 {
   Bool result;
 
-  DPRINTF ("bcl_serviceViewportCursor\n");
+  RAISE (MSG_DEBUG, "bcl_serviceViewportCursor");
   pipeWriteCmd (pipe_out, PCmd_ServiceVC);
   pipeWriteArgs (pipe_out, 5, sizeof(int), &flags, 
 		 sizeof(int), &cursor_x, sizeof(int), &cursor_y, 
@@ -390,17 +393,17 @@ Bool bcl_serviceViewportCursor (int flags, int cursor_x, int cursor_y,
 #ifdef DEBUG_PROBE
 void bcl_probeSystem (CardPtr card_list)
 {
-  DPRINTF ("bcl_probeSystem\n");
+  RAISE (MSG_DEBUG, "bcl_probeSystem");
 }
 
 void bcl_probeCard (void)
 {
-  DPRINTF ("bcl_probeCard\n");
+  RAISE (MSG_DEBUG, "bcl_probeCard");
 }
 
 I2CChainPtr bcl_probeBus (void)
 {
-  DPRINTF ("bcl_probeBus\n");
+  RAISE (MSG_DEBUG, "bcl_probeBus");
   return NULL;
 }
 #endif
@@ -471,7 +474,7 @@ Bool back_client_avail (void)
   pipeReadArgs (pipe_in, 1, sizeof(version), &version);
   bcl_closePipes ();
   if (version != PIPE_VERSION)
-    ERROR ("Server protocol version mismatch\n");
+    RAISE (MSG_ABORT, "Server protocol version mismatch.");
   return (version == PIPE_VERSION);
 }
 
@@ -493,6 +496,7 @@ CardPtr back_client_init (void)
     card->arch = "";
     card->chips = NULL;
     card->dev = NULL;
+    card->reg_mapped = NULL;
   }
   return bcl_root;
 }
