@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: back_direct.c,v 1.32 2004/03/10 07:18:03 dthierbach Exp $
  *
  * Contents:
  *
@@ -39,24 +39,14 @@
 #include "tv_common.h"
 #endif
 
-/* -------- Handle X screen stuff -------- */
-
-/* (This is needed in the X I2C routines) */
-
-ScrnInfoRec bdir_fakeScreen =
- {scrnIndex:0, driverPrivate: NULL};
-
-ScrnInfoPtr bdir_fakeScreens[1] = {&bdir_fakeScreen};
-
-ScrnInfoPtr *xf86Screens = bdir_fakeScreens;
-
 /* -------- Generic find routines -------- */
 
 /*
  *  List all modes (by system)
  */
 
-int bdir_listModes (CardType card, DataFunc *data, 
+int 
+bdir_listModes (CardType card, DataFunc *data, 
   TVSystem system, TVMode *(list[]))
 {
   if (!data) return FALSE;
@@ -67,7 +57,8 @@ int bdir_listModes (CardType card, DataFunc *data,
  *  Find mode by size (and resolution). 
  */
 
-Bool bdir_findBySize (CardType card, DataFunc *data, 
+Bool 
+bdir_findBySize (CardType card, DataFunc *data, 
   TVSystem system, int xres, int yres, char *size, TVMode *mode)
 {
   TVMode *tvm;
@@ -83,7 +74,8 @@ Bool bdir_findBySize (CardType card, DataFunc *data,
  *  Find mode by overscan (and resolution)
  */
 
-Bool bdir_findByOverscan (CardType card, DataFunc *data, TVSystem system, 
+Bool 
+bdir_findByOverscan (CardType card, DataFunc *data, TVSystem system, 
   int xres, int yres, double hoc, double voc, TVMode *mode)
 {
   TVMode *tvm;
@@ -101,7 +93,8 @@ Bool bdir_findByOverscan (CardType card, DataFunc *data, TVSystem system,
  *  Open card. For now, just pass that on.
  */
 
-void bdir_openCard (CardPtr card)
+void 
+bdir_openCard (CardPtr card)
 {
   switch (card->type)
   {
@@ -116,7 +109,7 @@ void bdir_openCard (CardPtr card)
       back_card= &bi810_func;
       break;
     default:
-      ERROR ("Fatal: wrong card type in bdir_openCard\n");
+      RAISE (MSG_ABORT, "Wrong card type in bdir_openCard.");
       exit (1);
   }
   back_card->openCard (card);
@@ -126,9 +119,10 @@ void bdir_openCard (CardPtr card)
  * Close card, For now, just pass that on.
  */
 
-void bdir_closeCard (void)
+void 
+bdir_closeCard (void)
 {
-  back_card->closeCard ();
+  if (back_card) back_card->closeCard ();
   back_card = NULL;
 }
 
@@ -140,7 +134,8 @@ void bdir_closeCard (void)
    for it. */
 
 #ifdef DEBUG_PROBE
-void bdir_probe_ch1 (I2CDevPtr dev)
+void 
+bdir_probe_ch1 (I2CDevPtr dev)
 {
   I2CByte res;
   int i;
@@ -157,7 +152,8 @@ void bdir_probe_ch1 (I2CDevPtr dev)
 #endif
 
 #ifdef DEBUG_PROBE
-void bdir_probe_ch2 (I2CDevPtr dev)
+void 
+bdir_probe_ch2 (I2CDevPtr dev)
 {
   I2CByte res;
   int i;
@@ -174,7 +170,8 @@ void bdir_probe_ch2 (I2CDevPtr dev)
 #endif
 
 #ifdef DEBUG_PROBE
-void bdir_probe_cx (I2CDevPtr dev)
+void 
+bdir_probe_cx (I2CDevPtr dev)
 {
   I2CByte res;
   int i;
@@ -195,7 +192,8 @@ void bdir_probe_cx (I2CDevPtr dev)
 #endif
 
 #ifdef DEBUG_PROBE
-void bdir_probe_ph (I2CDevPtr dev)
+void 
+bdir_probe_ph (I2CDevPtr dev)
 {
   I2CByte res;
   I2CByte rgb[3];
@@ -236,7 +234,8 @@ void bdir_probe_ph (I2CDevPtr dev)
 #endif
 
 #ifdef DEBUG_PROBE
-void bdir_probeSystem (CardPtr card_list)
+void 
+bdir_probeSystem (CardPtr card_list)
 {
   CardPtr card;
   I2CChainPtr chain, root;
@@ -255,9 +254,9 @@ void bdir_probeSystem (CardPtr card_list)
     }
     printf ("\n\n");
     for (chain = root; chain; chain = chain->next) {
-      if (chain->chip != TV_NO_CHIP) {
+      if (chain->type != TV_NO_CHIP) {
 	printf ("  Device %1s:%02X = %s\n", I2C_ID(chain->dev), chain->name);
-	switch (chain->chip & TV_ENCODER) 
+	switch (chain->type & TV_ENCODER) 
 	{
 	  case TV_CHRONTEL_MODEL1:
 	    bdir_probe_ch1 (chain->dev); break;
@@ -288,7 +287,8 @@ BackAccessRec bdir_func = {
 
 /* -------- Helper -------- */
 
-void bdir_freeChips (CardPtr card)
+void 
+bdir_freeChips (CardPtr card)
 {
   ChipPtr chip, del;
 
@@ -302,7 +302,8 @@ void bdir_freeChips (CardPtr card)
   card->chips = NULL;
 }
 
-ChipPtr bdir_copyChips (I2CChainPtr root)
+ChipPtr 
+bdir_copyChips (I2CChainPtr root)
 {
   ChipPtr chip, ins, first;
   I2CChainPtr chain;
@@ -310,13 +311,18 @@ ChipPtr bdir_copyChips (I2CChainPtr root)
   first = ins = NULL;
   for (chain = root; chain; chain = chain->next)
   {
-    chip = malloc (sizeof(ChipInfo));
+    chip = (ChipPtr) xcalloc (1, sizeof(ChipInfo));
     chip->private = chain;
     chip->next = NULL;
-    chip->chip = chain->chip;
+    chip->type = chain->type;
     chip->name = chain->name;
-    chip->bus  = chain->dev->pI2CBus->BusName[2];
-    chip->addr = chain->dev->SlaveAddr;
+    if (chain->dev) {
+      chip->bus  = chain->dev->pI2CBus->BusName[2];
+      chip->addr = chain->dev->SlaveAddr;
+    } else {
+      chip->bus = -1;
+      chip->addr = 0;
+    }
     if (ins) {
       ins->next = chip;
     } else {
