@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: back_direct.h,v 1.14 2003/10/10 16:16:57 dthierbach Exp $
  *
  * Contents:
  *
@@ -27,7 +27,6 @@
 #define _BACK_DIRECT_H
 
 #include "local.h" 
-#include "xfree.h"
 
 #include "backend.h"
 #include "tv_common.h"
@@ -57,8 +56,6 @@ Bool bdir_findByOverscan (CardType card, DataFunc *data, TVSystem system,
 
 /* additional helper methods */
 
-extern ScrnInfoRec bdir_fakeScreen;
-
 void bdir_freeChips (CardPtr chips);
 ChipPtr bdir_copyChips (I2CChainPtr root);
 
