@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: back_null.c,v 1.27 2004/01/27 17:05:25 dthierbach Exp $
  *
  * Contents:
  *
@@ -30,33 +30,38 @@
 #include "back_null.h"
 #include "data.h"
 
+static ChipInfo null_chip_nx = {
+  name: "<Null Nvidia chip> (6:00)", type: TV_NVIDIA, 
+  bus:6, addr:0x00, next: NULL
+};
+  
 static ChipInfo null_chip_ph2 = {
-  name: "<Null Philips-2 chip> (5:8a)", chip: TV_PHILIPS_MODEL2, 
-  bus:4, addr:0x8a, next: NULL
+  name: "<Null Philips-2 chip> (5:8a)", type: TV_PHILIPS_MODEL2, 
+  bus:5, addr:0x8a, next: &null_chip_nx
 };
   
 static ChipInfo null_chip_ph1 = {
-  name: "<Null Philips-1 chip> (4:8a)", chip: TV_PHILIPS_MODEL1, 
-  bus:3, addr:0x8a, next: &null_chip_ph2
+  name: "<Null Philips-1 chip> (4:8a)", type: TV_PHILIPS_MODEL1, 
+  bus:4, addr:0x8a, next: &null_chip_ph2
 };
   
 static ChipInfo null_chip_ch2 = {
-  name: "<Null Chrontel-2 chip> (3:ec)", chip: TV_CHRONTEL_MODEL2, 
-  bus:2, addr:0xec, next: &null_chip_ph1
+  name: "<Null Chrontel-2 chip> (3:ec)", type: TV_CHRONTEL_MODEL2, 
+  bus:3, addr:0xec, next: &null_chip_ph1
 };
   
 static ChipInfo null_chip_ch1 = {
-  name: "<Null Chrontel-1 chip> (2:ea)", chip: TV_CHRONTEL_MODEL1, 
+  name: "<Null Chrontel-1 chip> (2:ea)", type: TV_CHRONTEL_MODEL1, 
   bus:2, addr:0xea, next: &null_chip_ch2
 };
   
 static ChipInfo null_chip_cx = {
-  name: "<Null Conexant chip> (1:8a)", chip: TV_CONEXANT, 
+  name: "<Null Conexant chip> (1:8a)", type: TV_CONEXANT, 
   bus:1, addr:0x8a, next: &null_chip_ch1
 };
   
 static ChipInfo null_chip_bt = {
-  name: "<Null Brooktree chip> (0:88)", chip: TV_BROOKTREE, 
+  name: "<Null Brooktree chip> (0:88)", type: TV_BROOKTREE, 
   bus:0, addr:0x88, next: &null_chip_cx
 };
 
@@ -93,56 +98,56 @@ static CardPtr bnull_card = NULL;
 
 void bnull_openCard (CardPtr card)
 {
-  DPRINTF ("bnull_open\n");
+  RAISE (MSG_DEBUG, "bnull_open");
   bnull_card = card;
-  bnull_data = data_func (card->type, card->chips->chip);
+  bnull_data = data_func (card->type, card->chips->type);
 }
 
 void bnull_closeCard (void)
 {
-  DPRINTF ("bnull_close\n");
+  RAISE (MSG_DEBUG, "bnull_close");
 }
 
 #ifdef DEBUG_PROBE
 void bnull_probeSystem (CardPtr card_list)
 {
-  DPRINTF ("bnull_probeSystem\n");
+  RAISE (MSG_DEBUG, "bnull_probeSystem");
 }
 
 void bnull_probeCard (void)
 {
-  DPRINTF ("bnull_probeCard\n");
+  RAISE (MSG_DEBUG, "bnull_probeCard");
 }
 
 I2CChainPtr bnull_probeBus (void)
 {
-  DPRINTF ("bnull_probeBus\n");
+  RAISE (MSG_DEBUG, "bnull_probeBus");
   return NULL;
 }
 #endif
 
 void bnull_setHeads (int main, int tv, int video)
 {
-  DPRINTF ("bnull_setHeads\n");
+  RAISE (MSG_DEBUG, "bnull_setHeads");
 }
 
 void bnull_getHeads (int *main, int *tv, int *video, int *max) 
 {
-  DPRINTF ("bnull_getHeads\n");
+  RAISE (MSG_DEBUG, "bnull_getHeads");
   if (max)   *max = 2;
 }
 
 void bnull_getHeadDev (int head, int *devFlags) 
 {
-  DPRINTF ("bnull_getHeadDev\n");
+  RAISE (MSG_DEBUG, "bnull_getHeadDev");
   if (devFlags) *devFlags = DEV_MONITOR;
 }
 
 void bnull_probeChips (void)
 {
-  DPRINTF ("bnull_probe\n");
+  RAISE (MSG_DEBUG, "bnull_probe");
   if (bnull_card) {
-    bnull_data = data_func (bnull_card->type, bnull_card->chips->chip);
+    bnull_data = data_func (bnull_card->type, bnull_card->chips->type);
   } else {
     bnull_data = &null_func;
   }
@@ -150,49 +155,49 @@ void bnull_probeChips (void)
 
 void bnull_setChip (ChipPtr chip, Bool init)
 {
-  DPRINTF ("bnull_setChip %s %i\n", chip->name, init);
-  bnull_data = data_func (bnull_card->type, chip->chip);
+  RAISE (MSG_DEBUG, "bnull_setChip %s %i", chip->name, init);
+  bnull_data = data_func (bnull_card->type, chip->type);
 }
 
 void bnull_setSettings (TVSettings *set)
 {
-  DPRINTF ("bnull_setSettings\n");
+  RAISE (MSG_DEBUG, "bnull_setSettings");
 }
 
 void bnull_getSettings (TVSettings *set)
 {
-  DPRINTF ("bnull_getSettings\n");
+  RAISE (MSG_DEBUG, "bnull_getSettings");
 }
 
 void bnull_setMode (TVRegs *r)
 {
-  DPRINTF ("bnull_setMode\n");
+  RAISE (MSG_DEBUG, "bnull_setMode");
 }
 
 void bnull_getMode (TVRegs *r)
 {
-  DPRINTF ("bnull_getMode\n");
+  RAISE (MSG_DEBUG, "bnull_getMode");
 }
 
 void bnull_setModeSettings (TVRegs *r, TVSettings *set)
 {
-  DPRINTF ("bnull_setModeSettings\n");
+  RAISE (MSG_DEBUG, "bnull_setModeSettings");
 }
 
 void bnull_setTestImage (TVEncoderRegs *tv, TVSettings *set)
 {
-  DPRINTF ("bnull_setTestImage\n");
+  RAISE (MSG_DEBUG, "bnull_setTestImage");
 }
 
 long bnull_getStatus (int index)
 {
-  DPRINTF ("bnull_getStatus\n");
+  RAISE (MSG_DEBUG, "bnull_getStatus");
   return 0;
 }
 
 TVConnect bnull_getConnection (void)
 {
-  DPRINTF ("bnull_getConnection\n");
+  RAISE (MSG_DEBUG, "bnull_getConnection");
   return CONNECT_NONE;
 }
 
@@ -206,7 +211,7 @@ Bool bnull_findBySize (TVSystem system, int xres, int yres, char *size,
 {
   TVMode *tvm;
 
-  DPRINTF ("bnull_findBySize %i %i,%i %s\n", system, xres, yres, size);
+  RAISE (MSG_DEBUG, "bnull_findBySize %i %i,%i %s", system, xres, yres, size);
   /* FIXME: All cards */
   tvm = data_find (bnull_data->modes (), system, xres, yres, size);
   if (!tvm) return FALSE;
@@ -217,7 +222,7 @@ Bool bnull_findBySize (TVSystem system, int xres, int yres, char *size,
 Bool bnull_findByOverscan (TVSystem system, int xres, int yres, 
     double hoc, double voc, TVMode *mode)
 {
-  DPRINTF ("bnull_findByOC %i %i,%i\n", system, xres, yres);
+  RAISE (MSG_DEBUG, "bnull_findByOC %i %i,%i", system, xres, yres);
   return FALSE;
 }
 
