@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: back_nvidia.c,v 1.45 2004/03/01 21:08:10 dthierbach Exp $
  *
  * Contents:
  *
@@ -59,6 +59,7 @@
  */
 
 static CardPtr bnv_card;
+static int bnv_fd = -1; /* fd/handle for mmap */
 
 static int bnv_main_head = 1;
 static int bnv_tv_head = 1;
@@ -68,6 +69,7 @@ static NVRec bnv_nv =
 {
   TvChain: NULL,
   TvBusses: {NULL, NULL, NULL},
+  TvMaxBus: 0,
 };
 
 static DataFunc *bnv_data = NULL;
@@ -91,7 +93,8 @@ static int bnv_view_tv_y = 0;
 
 void bnv_updateCrt (int head)
 {
-  NVGetAllRegs (&bnv_nv, 0, &bnv_regs);
+  RAISE (MSG_DEBUG, "bnv update crt %i", head);
+  NVGetAllRegs (&bnv_nv, head-1, &bnv_regs);
 }
 
 /*
@@ -102,7 +105,7 @@ void bnv_initHeads (void)
 {
   int devFlags[2];
 
-  DPRINTF ("bnv init heads %i\n", bnv_nv.arch.heads);
+  RAISE (MSG_DEBUG, "bnv init heads %i", bnv_nv.arch.heads);
   if (bnv_nv.arch.heads == 1) {
     bnv_main_head = bnv_tv_head = bnv_video_head = 1;
     bnv_nv.TvHead = 0;
@@ -143,12 +146,11 @@ void bnv_initHeads (void)
 
 void bnv_initCard (void)
 {
-  bnv_updateCrt (0);
-  bdir_fakeScreen.driverPrivate = &bnv_nv;
+  bnv_updateCrt (1);
   if (bnv_nv.arch.exact == 0x2A) {
-    if (!XBoxBusInit (&bdir_fakeScreen)) return;
+    if (!XBoxBusInit (&bnv_nv)) return;
   } else {
-    if (!NVTvBusInit (&bdir_fakeScreen)) return;
+    if (!NVTvBusInit (&bnv_nv)) return;
   }
   bnv_initHeads ();
   bnv_updateCrt (bnv_tv_head);
@@ -161,22 +163,23 @@ void bnv_initCard (void)
 
 void bnv_mapNvMem (CardPtr card, NVPtr pNv, int heads)
 {
-  int fd;
-
-  fd = openDevMem (card);
-  pNv->riva.PMC     = mapDevMem(card, fd, pNv->IOAddress+0x000000, 0x1000);
-  pNv->riva.PFB     = mapDevMem(card, fd, pNv->IOAddress+0x100000, 0x1000);
-  pNv->riva.PEXTDEV = mapDevMem(card, fd, pNv->IOAddress+0x101000, 0x1000);
-  pNv->riva.PCRTC   = mapDevMem(card, fd, pNv->IOAddress+0x600000, 
+  bnv_fd = openDevMem (card);
+  pNv->riva.PMC     = mapDevMem(card, bnv_fd, pNv->IOAddress+0x000000, 0x1000);
+  pNv->riva.PFB     = mapDevMem(card, bnv_fd, pNv->IOAddress+0x100000, 0x1000);
+  pNv->riva.PEXTDEV = mapDevMem(card, bnv_fd, pNv->IOAddress+0x101000, 0x1000);
+  pNv->riva.PCRTC   = mapDevMem(card, bnv_fd, pNv->IOAddress+0x600000, 
 				0x2000 * heads);
-  pNv->riva.PCIO    = ((U008 *) pNv->riva.PCRTC) + 0x1000;     
-  pNv->riva.PVIO    = mapDevMem(card, fd, pNv->IOAddress+0x0C0000, 0x1000);
-  pNv->riva.PRAMDAC = mapDevMem(card, fd, pNv->IOAddress+0x680000, 
+  pNv->riva.PCIO    = ((CARD8 *) pNv->riva.PCRTC) + 0x1000;     
+  pNv->riva.PVIO    = mapDevMem(card, bnv_fd, pNv->IOAddress+0x0C0000, 0x1000);
+  pNv->riva.PRAMDAC = mapDevMem(card, bnv_fd, pNv->IOAddress+0x680000, 
 				0x2000 * heads);
-  pNv->riva.PDIO    = ((U008 *) pNv->riva.PRAMDAC) + 0x1000;     
-  pNv->riva.PVIDEO  = mapDevMem(card, fd, pNv->IOAddress+0x008000, 0x1000);
-  pNv->riva.PTIMER  = mapDevMem(card, fd, pNv->IOAddress+0x009000, 0x1000);
-  closeDevMem (card, fd);
+  pNv->riva.PDIO    = ((CARD8 *) pNv->riva.PRAMDAC) + 0x1000;     
+  pNv->riva.PVIDEO  = mapDevMem(card, bnv_fd, pNv->IOAddress+0x008000, 0x1000);
+  pNv->riva.PTIMER  = mapDevMem(card, bnv_fd, pNv->IOAddress+0x009000, 0x1000);
+  pNv->riva.PTV     = NULL;
+  if (pNv->arch.exact == 0x17) {
+    pNv->riva.PTV   = mapDevMem(card, bnv_fd, pNv->IOAddress+0x00d000, 0x1000);
+  }
 }
 
 void bnv_unmapNvMem (CardPtr card, NVPtr pNv, int heads)
@@ -187,6 +190,8 @@ void bnv_unmapNvMem (CardPtr card, NVPtr pNv, int heads)
   unmapDevMem(card, pNv->IOAddress+0x600000, 0x2000 * heads);
   unmapDevMem(card, pNv->IOAddress+0x680000, 0x2000 * heads);
   unmapDevMem(card, pNv->IOAddress+0x008000, 0x1000);
+  closeDevMem (card, bnv_fd);
+  bnv_fd = -1;
 }
 
 void bnv_setNvArch (CardPtr card, NVPtr pNv);
@@ -288,7 +293,7 @@ void bnv_openCard (CardPtr card)
   NVPtr pNv;
   int i;
 
-  DPRINTF ("bnv open\n");
+  RAISE (MSG_DEBUG, "bnv open");
   bnv_card = card;
   pNv = &bnv_nv;
   pNv->IOAddress = card->reg_base;
@@ -322,109 +327,57 @@ void bnv_openCard (CardPtr card)
       break;
     default:
       pNv->arch.major = (card->pci_id & 0x0f00) >> 4;
+      break;
   }
   pNv->arch.exact = pNv->arch.major; /* To be safe */
   switch (card->pci_id) {
-      case PCI_CHIP_RIVA128:
-	pNv->arch.exact = 0x03; /* NV3 */
-	break;
-      case PCI_CHIP_TNT:
-	pNv->arch.exact = 0x04; /* NV4 */
-	break;
-      case PCI_CHIP_TNT2:
-      case PCI_CHIP_TNT2_A:
-      case PCI_CHIP_TNT2_B:
-      case PCI_CHIP_UTNT2:
-      case PCI_CHIP_VTNT2:
-      case PCI_CHIP_UVTNT2:
-	pNv->arch.exact = 0x05; /* NV5 */
-	break;
-      case PCI_CHIP_ITNT2:
-	pNv->arch.exact = 0x0A; /* NV0A */
-	break;
-      case PCI_CHIP_GEFORCE256:
-      case PCI_CHIP_GEFORCEDDR:
-      case PCI_CHIP_QUADRO:
-	pNv->arch.exact = 0x10; /* NV10 */
-	break;
-      case PCI_CHIP_GEFORCE2MX:
-      case PCI_CHIP_GEFORCE2MXDDR:
-      case PCI_CHIP_QUADRO2MXR:
-      case PCI_CHIP_GEFORCE2GO:
-	pNv->arch.exact = 0x11; /* NV11 */
-	pNv->arch.heads = 2;
-	break;
-      case PCI_CHIP_GEFORCE2GTS:
-      case PCI_CHIP_GEFORCE2TI:
-      case PCI_CHIP_GEFORCE2ULTRA:
-      case PCI_CHIP_QUADRO2PRO:
-	pNv->arch.exact = 0x15; /* NV15 */
-	break;
-      case PCI_CHIP_GEFORCE4MX_460:
-      case PCI_CHIP_GEFORCE4MX_440:
-      case PCI_CHIP_GEFORCE4MX_420:
-      case PCI_CHIP_GEFORCE4MX_440SE:
-      case PCI_CHIP_GEFORCE4GO_440:
-      case PCI_CHIP_GEFORCE4GO_420:
-      case PCI_CHIP_GEFORCE4GO_420_32M:
-      case PCI_CHIP_GEFORCE4GO_460:
-      case PCI_CHIP_QUADRO4XGL_500:
-      case PCI_CHIP_GEFORCE4GO_440_64M:
-      case PCI_CHIP_QUADRO4NVS:
-      case PCI_CHIP_QUADRO4XGL_550:
-      case PCI_CHIP_QUADRO4GOGL_500:
-      case PCI_CHIP_GEFORCE4GO_410_16M:
-	pNv->arch.exact = 0x17; /* NV17 */
-	pNv->arch.heads = 2; /* from X */
-	break;
-      case PCI_CHIP_GEFORCE4MX_440_AGP8X:  
-      case PCI_CHIP_GEFORCE4MX_440SE_AGP8X:
-      case PCI_CHIP_GEFORCE4MX_420_AGP8X:  
-      case PCI_CHIP_QUADRO4_XGL_580: 
-      case PCI_CHIP_QUADRO4_NVS_280: 
-      case PCI_CHIP_QUADRO4_XGL_380: 
-	pNv->arch.exact = 0x18; /* NV18 */
-	pNv->arch.heads = 2; /* from X */
-	break;
-      case PCI_CHIP_GEFORCE4GPU:
-	pNv->arch.exact = 0x1f; /* NV1F */
-	pNv->arch.heads = 2; /* from X */
-	break;
-      case PCI_CHIP_GEFORCE2GPU:
-	pNv->arch.exact = 0x1A; /* NV1A */
-	break;
-      case PCI_CHIP_GEFORCE3:
-      case PCI_CHIP_GEFORCE3TI_200:
-      case PCI_CHIP_GEFORCE3TI_500:
-      case PCI_CHIP_QUADRO_DDC:
-	pNv->arch.exact = 0x20; /* NV20 */
-	break;
-      case PCI_CHIP_GEFORCE3_MCPX:
-	pNv->arch.exact = 0x2a; /* NV2A */
-	pNv->arch.heads = 1; 
-	break;
-      case PCI_CHIP_GEFORCE4TI_4600:
-      case PCI_CHIP_GEFORCE4TI_4400:
-      case PCI_CHIP_GEFORCE4TI_4200:
-      case PCI_CHIP_QUADRO4XGL_900:
-      case PCI_CHIP_QUADRO4XGL_750:
-      case PCI_CHIP_QUADRO4XGL_700:
-	pNv->arch.exact = 0x25; /* NV25 */
-	pNv->arch.heads = 2; /* from X */
-	break;
-      case PCI_CHIP_GEFORCE4TI_4800:
-      case PCI_CHIP_GEFORCE4TI_4200_AGP8X:
-      case PCI_CHIP_GEFORCE4TI_4800SE: 
-      case PCI_CHIP_GEFORCE4GO_4200:
-      case PCI_CHIP_QUADRO4XGL_980: 
-      case PCI_CHIP_QUADRO4XGL_780: 
-      case PCI_CHIP_QUADRO4GOGL_700:
-	pNv->arch.exact = 0x28; /* NV28 */
-	pNv->arch.heads = 2; /* from X */
-	break;
-      default:
-        fprintf (stderr, "Unknown pci card %04x\n", pNv->Chipset);
+    case PCI_CHIP_RIVA128:
+      pNv->arch.exact = 0x03; /* NV3 */
+      break;
+    case PCI_CHIP_TNT:
+      pNv->arch.exact = 0x04; /* NV4 */
+      break;
+    case PCI_CHIP_TNT2:
+    case PCI_CHIP_TNT2_A:
+    case PCI_CHIP_TNT2_B:
+    case PCI_CHIP_UTNT2:
+    case PCI_CHIP_VTNT2:
+    case PCI_CHIP_UVTNT2:
+      pNv->arch.exact = 0x05; /* NV5 */
+      break;
+    case PCI_CHIP_ITNT2:
+      pNv->arch.exact = 0x0A; /* NV0A */
+      break;
+    default:
+      pNv->arch.exact = (card->pci_id & 0x0ff0) >> 4;
+      break;
   }
+  /* X says two head if: >= 0x11 && != 0x15,0x1a,0x20 */
+  switch (pNv->arch.exact) {
+    case 0x03:
+    case 0x04:
+    case 0x05:
+    case 0x0a:
+    case 0x10:
+    case 0x15:
+    case 0x1a:
+    case 0x20:
+    case 0x2a:
+      pNv->arch.heads = 1;
+      break;
+    case 0x11:
+    case 0x17:
+    case 0x18:
+    case 0x1f:
+    case 0x25: /* from X */
+    case 0x28: /* from X */
+    default: /* FIXME */
+      pNv->arch.heads = 2;
+      break;
+  }
+  /* TODO: NV31 and NV36 have twostagePLL. 'nv' source claims (wrongly?)
+     NV34 does, too. */
+  /* Number of DVO ports: NV11, NV20, NV17: 1. NV25, NV17/18/better: 2 */
   bnv_mapNvMem (card, &bnv_nv, bnv_nv.arch.heads);
   bnv_setNvArch (card, &bnv_nv);
   bnv_initArch (&bnv_nv);
@@ -437,7 +390,7 @@ void bnv_openCard (CardPtr card)
 
 void bnv_closeCard (void)
 {
-  NVDestroyBusses (&bnv_nv);
+  TVDestroyAll (&bnv_nv.TvChain, &bnv_nv.TvBusses, bnv_nv.TvMaxBus);
   bnv_unmapNvMem (bnv_card, &bnv_nv, bnv_nv.arch.heads);
 }
 
@@ -445,10 +398,9 @@ void bnv_closeCard (void)
  *  Set heads used for tv, monitor, video scaler.
  */
 
-
 void bnv_setHeads (int main, int tv, int video)
 {
-  DPRINTF ("bnv_setHeads %i %i %i\n", main, tv, video);
+  RAISE (MSG_DEBUG, "bnv_setHeads %i %i %i", main, tv, video);
   if (main  > 0) bnv_main_head = main;
   if (tv    > 0) bnv_tv_head = tv;
   if (video > 0) bnv_video_head = video;
@@ -464,7 +416,7 @@ void bnv_setHeads (int main, int tv, int video)
 
 void bnv_getHeads (int *main, int *tv, int *video, int *max) 
 {
-  DPRINTF ("bnv_getHeads\n");
+  RAISE (MSG_DEBUG, "bnv_getHeads");
   if (main ) *main  = bnv_main_head;
   if (tv   ) *tv    = bnv_tv_head;
   if (video) *video = bnv_video_head;
@@ -473,7 +425,7 @@ void bnv_getHeads (int *main, int *tv, int *video, int *max)
 
 void bnv_getHeadDev (int head, int *devFlags) 
 {
-  DPRINTF ("bnv_getHeadDev\n");
+  RAISE (MSG_DEBUG, "bnv_getHeadDev");
   if (devFlags) *devFlags = NVGetDevFlags (&bnv_nv, head-1);
 }
 
@@ -483,9 +435,9 @@ void bnv_getHeadDev (int head, int *devFlags)
 
 void bnv_probeChips (void)
 {
-  DPRINTF ("bnv_probeChips\n");
+  RAISE (MSG_DEBUG, "bnv_probeChips");
   bdir_freeChips (bnv_card);
-  NVDestroyDevices (&bnv_nv); 
+  TVDestroyChainDevices (&bnv_nv.TvChain, &bnv_nv.TvBusses, bnv_nv.TvMaxBus);
   NVProbeTvDevices (&bnv_nv);
   bnv_card->chips = bdir_copyChips (bnv_nv.TvChain);
   bnv_setChip (bnv_card->chips, FALSE); /* don't init */
@@ -497,28 +449,28 @@ void bnv_probeChips (void)
 
 void bnv_setChip (ChipPtr chip, Bool init)
 {
-  DPRINTF ("bnv_setChip %s %i\n", chip ? chip->name : "NULL", init);
+  RAISE (MSG_DEBUG, "bnv_setChip %s %i", chip ? chip->name : "NULL", init);
   if (!chip) {
     bnv_enc_avail = FALSE;
     return;
   }
   bnv_enc_avail = TRUE;
   NVSetTvDevice (&bnv_nv, (I2CChainPtr) chip->private, init);
-  bnv_data = data_func (CARD_NVIDIA, chip->chip);
+  bnv_data = data_func (CARD_NVIDIA, chip->type);
   bnv_data->defaults (&bnv_set);
   bnv_set_avail = TRUE;
 }
 
 void bxbox_setChip (ChipPtr chip, Bool init)
 {
-  DPRINTF ("bnv_setChip %s %i\n", chip ? chip->name : "NULL", init);
+  RAISE (MSG_DEBUG, "bnv_setChip %s %i", chip ? chip->name : "NULL", init);
   if (!chip) {
     bnv_enc_avail = FALSE;
     return;
   }
   bnv_enc_avail = TRUE;
   NVSetTvDevice (&bnv_nv, (I2CChainPtr) chip->private, init);
-  bnv_data = data_func (CARD_XBOX, chip->chip);
+  bnv_data = data_func (CARD_XBOX, chip->type);
   bnv_data->defaults (&bnv_set);
   bnv_set_avail = TRUE;
 }
@@ -534,9 +486,9 @@ void bnv_apply (void)
 {
   TVRegs temp;
 
-  bnv_data->clamp (&bnv_set, &bnv_regs);
   temp = bnv_regs;
-  if (bnv_set_avail) {
+  if ((temp.devFlags & DEV_TELEVISION) && bnv_set_avail) {
+    bnv_data->clamp (&bnv_set, &bnv_regs);
     bnv_data->setup (&bnv_set, &temp);
   }
   if (bnv_nv.arch.exact == 0x2A) {
@@ -588,6 +540,7 @@ void bnv_setMode (TVRegs *r)
 void bnv_getMode (TVRegs *r)
 
 {
+  RAISE (MSG_DEBUG, "get mode nv %i", bnv_tv_head);
   bnv_updateCrt (bnv_tv_head);
   /* FIXME: Make sure chip is set here */
   if (bnv_regs.devFlags & DEV_TELEVISION) {
@@ -774,7 +727,7 @@ Bool bnv_checkViewport (int flags, int *pofs)
     NVGetCrtLayout (&bnv_nv, bnv_main_head - 1, &addr, &ofs);
     x = addr % (ofs * 2);
     y = addr / (ofs * 2);
-    DPRINTF ("bnv check main %i,%i\n", x, y);
+    RAISE (MSG_DEBUG, "bnv check main %i,%i", x, y);
     if (x != bnv_view_main_x || y != bnv_view_main_y) {
       changed = TRUE;
       bnv_view_tv_x = bnv_view_main_x = x;
@@ -800,7 +753,7 @@ Bool bnv_adjustViewport (int flags, int *view_x, int *view_y)
   int addr, ofs;
   Bool changed;
 
-  DPRINTF ("bnv adj view %i,%i\n", *view_x, *view_y);
+  RAISE (MSG_DEBUG, "bnv adj view %i,%i", *view_x, *view_y);
   changed = bnv_checkViewport (flags, &ofs);
   if (changed) {
     *view_x = bnv_view_tv_x;
@@ -821,7 +774,7 @@ Bool bnv_serviceViewportCursor (int flags, int cursor_x, int cursor_y,
   int addr, ofs;
   Bool changed;
 
-  DPRINTF ("bnv service %i,%i\n", cursor_x, cursor_y);
+  RAISE (MSG_DEBUG, "bnv service %i,%i", cursor_x, cursor_y);
   changed = bnv_checkViewport (flags, &ofs);
   if (flags & BACK_SERVICE_VIEW_CURSOR) {
     if (cursor_x < bnv_view_tv_x) {
@@ -991,10 +944,11 @@ void bnv_probeCard (void)
 
 I2CChainPtr bnv_probeBus (void)
 {
-  NVDestroyDevices (&bnv_nv); 
+  TVDestroyChainDevices (&bnv_nv.TvChain, &bnv_nv.TvBusses, bnv_nv.TvMaxBus);
   /* probe needs unlocked crt */
   NVUpdateTvState (&bnv_nv);
-  TVProbeCreateAll (bnv_nv.TvBusses, bnv_nv.TvMaxBus, &bnv_nv.TvChain); 
+  TVCheckChain (bnv_nv.TvChain);
+  bnv_nv.TvChain = TVProbeCreateAll (bnv_nv.TvBusses, bnv_nv.TvMaxBus, NULL); 
   return bnv_nv.TvChain;
 }
 
