@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: backend.h,v 1.20 2004/01/27 17:05:25 dthierbach Exp $
  *
  * Contents:
  *
@@ -51,21 +51,25 @@
 typedef struct chip_info {
   struct chip_info *next; /* must be 1st entry! */
   char *name;    /* must be 2nd entry! (name including version and I2C addr) */
-  TVChip chip;   /* chip type */
+  TVChip type;   /* chip type */
   int bus, addr; /* I2C bus and address */
   void *private; /* identify device, backend private */
 } ChipInfo, *ChipPtr;
 
 /* List of all NVidia cards available */
 
+#define CARD32 unsigned int
+
 typedef struct card_info {
   struct card_info *next; /* must be 1st entry! */
   char *name; /* must be 2nd entry! (name including bus addr) */
   char *dev;  /* name of device for mmap (not via pipe) */
   char *arch; /* architecture (not via pipe) */
   CardType type;
-  unsigned long reg_base;
-  unsigned long pio_base;
+  void* reg_mapped; /* not via pipe */
+  CARD32 reg_base;
+  CARD32 reg_size;
+  CARD32 pio_base;
   int addr_bus, addr_slot, addr_func;
   int pci_id;
   ChipPtr chips;  
