@@ -9,9 +9,11 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <math.h>
 
-#include "nv_tvchip.h"
+#include "local.h"
+#include "tv_chip.h"
 #include "calc_bt.h"
 
 #define DebugOut(x) printf(x)
@@ -49,6 +51,7 @@
 #define BT868_IMPLEMENTATION_FLAGS_DISCONFREE	(1 << 0)
 #define BT868_IMPLEMENTATION_FLAGS_CCIR601	(1 << 1)
 #define BT868_IMPLEMENTATION_FLAGS_DOUBLEPLL	(1 << 2)
+#define BT_IMPLEMENTATION_FLAGS_CONEXANT	(1 << 3)
 
 /*  dwInOutScanFlags; */
 
@@ -57,9 +60,11 @@
 
 /*  ucTVOutMode
 
-0 = NTSC,  1 = NTSC-60, 2 = PAL-BDGHI, 3 = PAL-N, 4 = PAL-Nc,
+0 = NTSC,  1 = NTSC-50, 2 = PAL-BDGHI, 3 = PAL-N, 4 = PAL-Nc,
 5 = PAL-M, 6 = PAL-M60, 7 = PAL-60 */
 
+/* vsr = vertical scale ratio? vsrq = vsr quantized ? */
+
 struct settings {
   double dCHARCLK;
   double dFXTAL; // 13500000;
@@ -84,7 +89,7 @@ struct settings {
   double dHXTAL; 	
   double dPHZINC; 	
   double dVBLANKO; 	
-  double dVSRQ; 	
+  double dVSRQ;
   double dMaxHsyncDrift; 
   double dMinFrontPorchIn;
   double dFrontPorchIn; 	
@@ -94,17 +99,17 @@ struct settings {
   double dFCLK; 	
   double dHACTIVEI; 	// 640;   //  { 640, 800 } 
   double dVACTIVEI; 	// 480;   //  { 480, 600 } 
-  double dHOC; 	// .1379;   //  { 0 - .25 } 
-  double dVOC; 	// .1358;   //  { 0 - .25 } 
-  double dVSR; 	// 0;
-  double dTLI; 	// 0;
+  double dHOC; 		// .1379;   //  { 0 - .25 } 
+  double dVOC; 		// .1358;   //  { 0 - .25 } 
+  double dVSR; 		// 0;
+  double dTLI; 		// 0;
   double dVLINESI; 	// 0;
   double dHCLKO; 	// 0;
   double dHCLKI; 	// 0;
-  double dTPI; 	// 0;
+  double dTPI; 		// 0;
   double dHFRACT; 	// 0;
-  double dMFP; 	// 0;
-  double dMBP; 	// 0;
+  double dMFP; 		// 0;
+  double dMBP; 		// 0;
   double dVACTIVEO; 	// 0;
   double dVBLANKI; 	// 0;
   double dPLLINT; 	// 0;
@@ -118,6 +123,7 @@ struct settings {
   double dMCR; 	// 0;
   double dMCB; 	// 0;
   double dVSCALE; 	// 0;
+  double dCLKRATIO; /* NEW */
   long ucPHASEOFF; 	// 0;
   long ucLUMADLY; 	// 0;
   long ucOUTMODE; 	// 0;
@@ -161,7 +167,7 @@ inline double max (double a, double b)
 
 void  Bt868EncoderInitializeDefault( PENCODER_SETTINGS pS )
 {
-  pS->dCHARCLK = 8;   	/*  { 1, 8, 9 } */
+  pS->dCHARCLK = 1; /* FIXME TEST 8; */ /*  { 1, 8, 9 } */
   pS->dFXTAL = 13500000;   	/*  { any }*/
   pS->dMinFrontPorchBC = 13;   	/*  { 0 - 20 }*/
   pS->dMinBackPorchBC = 3;   	/*  { 0 - 20 }*/
@@ -185,6 +191,7 @@ void  Bt868EncoderInitializeDefault( PENCODER_SETTINGS pS )
   pS->dPHZINC = 0;
   pS->dVBLANKO = 0;
   pS->dVSRQ = 0;
+  pS->dCLKRATIO = 1.0;
   pS->dMaxHsyncDrift = 0;
   pS->dMinFrontPorchIn = 0;
   pS->dFrontPorchIn = 0;
@@ -320,7 +327,7 @@ static void recalc_outfield (PENCODER_SETTINGS pS)
 {
   if( ( pS->ucTVOutMode == 0 ) 
         || ( pS->ucTVOutMode == 1 ) 
-        || ( pS->ucTVOutMode == 5 ) 
+        || ( pS->ucTVOutMode == 5 ) /* PAL_M */
         || ( pS->ucTVOutMode == 6 ) 
         || ( pS->ucTVOutMode == 7 ) )
   {
@@ -356,7 +363,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
 
   switch( pS->ucTVOutMode )
   {
-    case 0:
+    case 0: /* NTSC */
       pS->dFSC = 3579545;
       dBURSTSTART = .0000053;
       dBURSTEND = .00000782;
@@ -364,7 +371,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
       dACTIVEBEGIN = .0000094;
       dIMAGECENTER = .000035667;
       break;
-    case 1:
+    case 1: /* NTSC-50 Hz */
       pS->dFSC = 3579545;
       dBURSTSTART = .0000053;
       dBURSTEND = .00000782;
@@ -372,7 +379,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
       dACTIVEBEGIN = .0000094;
       dIMAGECENTER = .000035667;
       break;
-    case 2:
+    case 2: /* PAL-BDGHI */
       pS->dFSC = 4433618.75;
       dBURSTSTART = .0000056;
       dBURSTEND = .00000785;
@@ -380,7 +387,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
       dACTIVEBEGIN = .0000105;
       dIMAGECENTER = .000036407;
       break;
-    case 3:
+    case 3: /* PAL-N */
       pS->dFSC = 4433618.75;
       dBURSTSTART = .0000056;
       dBURSTEND = .00000785;
@@ -388,7 +395,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
       dACTIVEBEGIN = .0000094;
       dIMAGECENTER = .000035667;
       break;
-    case 4:
+    case 4: /* PAL-Nc */
       pS->dFSC = 3582056.25;
       dBURSTSTART = .0000056;
       dBURSTEND = .00000811;
@@ -396,7 +403,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
       dACTIVEBEGIN = .0000105;
       dIMAGECENTER = .000036407;
       break;
-    case 5:
+    case 5: /* PAL-M */
       pS->dFSC = 3575611.88;
       dBURSTSTART = .0000058;
       dBURSTEND = .00000832;
@@ -404,7 +411,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
       dACTIVEBEGIN = .0000094;
       dIMAGECENTER = .000035667;
       break;
-    case 6:
+    case 6: /* PAL-M60 */
       pS->dFSC = 3575611.88;
       dBURSTSTART = .0000058;
       dBURSTEND = .00000832;
@@ -412,7 +419,7 @@ static void recalc_tvformat (PENCODER_SETTINGS pS)
       dACTIVEBEGIN = .0000094;
       dIMAGECENTER = .000035667;
       break;
-    case 7:
+    case 7: /* PAL-60 */ 
       pS->dFSC = 4433619.49;
       dBURSTSTART = .0000056;
       dBURSTEND = .00000785;
@@ -436,18 +443,20 @@ static void recalc_prepare (PENCODER_SETTINGS pS)
   pS->dVLINESI = (long)pS->dTLI;
   pS->dHCLKO = ( 2 * ( ( pS->dHACTIVEI / ( 1.0 - pS->dHOC ) ) 
                    * ( pS->dTTO / pS->dATO ) ) ) + 0.5;
-  pS->dTPI = ( ( pS->dHCLKO / pS->dVSR ) * 256 + 0.5 ) / 256.0;
+  pS->dTPI = ( ( pS->dHCLKO * pS->dCLKRATIO / pS->dVSR ) * 256 + 0.5 ) / 256.0;
   pS->dHFRACT = (long)( ( pS->dTPI - (long)pS->dTPI ) * 256 );
   pS->dHCLKI = (long)pS->dTPI;
   pS->dMFP = max( 14.0, pS->dMinFrontPorchBC );
   pS->dMBP = max( 4.0, pS->dMinBackPorchBC );
 
+#if 0
   if( pS->dTPI < ( pS->dHACTIVEI + pS->dMFP + pS->dMBP ) )
   {
     //  error
-    DebugOut( "error: dTPI < ( dHACTIVEI + dMFP + dMBP )" );
+    DebugOut( "error: dTPI < ( dHACTIVEI + dMFP + dMBP )\n" );
         return;
   }
+#endif
   pS->dVACTIVEO = ( ( pS->dVACTIVEI + 3 ) / pS->dVSR ) + 1;
 }
 
@@ -455,101 +464,119 @@ static void recalc_find_oc (PENCODER_SETTINGS pS,
   double dMinHOC, double dMaxHOC, double dMinVOC, double dMaxVOC,
   RecalcFindRes callback)
 {
-  if( pS->dwImplementationFlags & BT868_IMPLEMENTATION_FLAGS_DISCONFREE )
+  double  dBestVOC = 0;
+  double  dTempVOC = 0;
+  double  dBestHOC = 0;
+  double  dTempHOC = 0;
+  double  dMinTLI = 0;
+  double  dMaxTLI = 0;
+  double  dTempTLI = 0;
+  double  dBestTLI = 0;
+  double  dMinHCLKO = 0;
+  double  dMaxHCLKO = 0;
+  double  dBestHCLKO = 0;
+  double  dBestMetric = 1000;
+  double  dTempVSR = 0;
+  double  dBestVSR = 0;
+  double  dMinTPI = 0;
+  double  dTempTPI = 0;
+  double  dBestTPI = 0;
+  double  dTempCLKRATIO = 1; /* NEW */
+  double  dBestCLKRATIO = 1; /* NEW */
+  int     actCLKRATIO;
+  int     maxCLKRATIO;
+  double  dTempHCLKO = 0;
+  double  dTempVACTIVEO = 0;
+  double  dTempTCCI = 0;
+  double  dDelta = 0;
+  double  dMetric = 0;
+  double  dAspect = 0;
+  double  dMinHBT = 0;
+
+  if (dMinHOC < 0) dMinHOC = 0;
+  if (dMinVOC < 0) dMinVOC = 0;
+
+  dMinTLI= (long)(pS->dVACTIVEI / ((1 - dMinVOC) * pS->dALO) * pS->dTLO);
+  dMaxTLI = (long)(pS->dVACTIVEI / ((1 - dMaxVOC) * pS->dALO) * pS->dTLO);
+  dMinHCLKO = (long) ((pS->dHACTIVEI * 2) / 
+		      ((1 - dMinHOC) * (pS->dATO / pS->dTTO)));
+  dMaxHCLKO = (long) ((pS->dHACTIVEI * 2) / 
+		      ((1 - dMaxHOC) * (pS->dATO / pS->dTTO)));
+
+  maxCLKRATIO = 0;
+  if (pS->dwImplementationFlags & BT_IMPLEMENTATION_FLAGS_CONEXANT) {
+    maxCLKRATIO = 1;
+  }
+  for (actCLKRATIO = 0; actCLKRATIO <= maxCLKRATIO; actCLKRATIO++)
   {
-    double  dBestVOC = 0;
-    double  dTempVOC = 0;
-    double  dBestHOC = 0;
-    double  dTempHOC = 0;
-    double  dMinTLI = 0;
-    double  dMaxTLI = 0;
-    double  dTempTLI = 0;
-    double  dBestTLI = 0;
-    double  dMinHCLKO = 0;
-    double  dMaxHCLKO = 0;
-    double  dBestHCLKO = 0;
-    double  dBestMetric = 1000;
-    double  dTempVSR = 0;
-    double  dBestVSR = 0;
-    double  dMinTPI = 0;
-    double  dTempTPI = 0;
-    double  dBestTPI = 0;
-    double  dTempHPIXELO = 0;
-    double  dTempVACTIVEO = 0;
-    double  dTempTCCI = 0;
-    double  dDelta = 0;
-    double  dMetric = 0;
-    double  dAspect = 0;
-    double  dMinHBT = 0;
-    double  dCLKRATIO = 1.0;
-
-    if (dMinHOC < 0) dMinHOC = 0;
-    if (dMinVOC < 0) dMinVOC = 0;
-
-    dMinTLI= (long)(pS->dVACTIVEI / ((1 - dMinVOC) * pS->dALO) * pS->dTLO);
-    dMaxTLI = (long)(pS->dVACTIVEI / ((1 - dMaxVOC) * pS->dALO) * pS->dTLO);
-    dMinHCLKO = (long) ((pS->dHACTIVEI * 2) / 
-			((1 - dMinHOC) * (pS->dATO / pS->dTTO)));
-    dMaxHCLKO = (long) ((pS->dHACTIVEI * 2) / 
-			((1 - dMaxHOC) * (pS->dATO / pS->dTTO)));
-
+    dTempCLKRATIO = 1.0; 
+    if (actCLKRATIO) dTempCLKRATIO = 3.0/2.0; 
     for(dTempTLI = dMinTLI; dTempTLI <= dMaxTLI; dTempTLI++)
     {                                   
       dTempVSR = dTempTLI / pS->dTLO;
       dTempVACTIVEO = (long)(((pS->dVACTIVEI * pS->dTLO) + 
-                              (dTempTLI - 1)) / dTempTLI);
+			      (dTempTLI - 1)) / dTempTLI);
       dTempVOC = 1 - dTempVACTIVEO / pS->dALO;
       dMinTPI = pS->dHACTIVEI / (1 - (dMinHBT / (pS->dTTO * dTempVSR)));
 
-      for(dTempHPIXELO = dMinHCLKO; dTempHPIXELO <= dMaxHCLKO; 
-	   dTempHPIXELO++)
+      for(dTempHCLKO = dMinHCLKO; dTempHCLKO <= dMaxHCLKO; 
+	   dTempHCLKO++)
       {                                 
-        dTempTPI = (dTempHPIXELO * dCLKRATIO) * (pS->dTLO / dTempTLI);
-        dTempTCCI = dTempTPI / pS->dCHARCLK;
+	dTempTPI = (dTempHCLKO * dTempCLKRATIO) * (pS->dTLO / dTempTLI);
+	dTempTCCI = dTempTPI / pS->dCHARCLK;
 
-        if((dTempTCCI == (long)dTempTCCI) && (dTempTPI >= dMinTPI))
-        {
-          dTempHOC = 1 - ((pS->dHACTIVEI / (dTempHPIXELO / 2)) / 
+	if((dTempTCCI == (long)dTempTCCI) && (dTempTPI >= dMinTPI))
+	{
+	  int flag;
+
+	  dTempHOC = 1 - ((pS->dHACTIVEI / (dTempHCLKO / 2)) / 
 			  (pS->dATO / pS->dTTO));
-          dDelta = fabs(dTempHOC - pS->dHOC) + fabs(dTempVOC - pS->dVOC);
-          dMetric = ((dTempHOC - pS->dHOC) * (dTempHOC - pS->dHOC)) + 
-	            ((dTempVOC - pS->dVOC) * (dTempVOC - pS->dVOC)) + 
-	            (2 * dDelta * dDelta);
+	  dDelta = fabs(dTempHOC - pS->dHOC) + fabs(dTempVOC - pS->dVOC);
+	  dMetric = ((dTempHOC - pS->dHOC) * (dTempHOC - pS->dHOC)) + 
+		    ((dTempVOC - pS->dVOC) * (dTempVOC - pS->dVOC)) + 
+		    (2 * dDelta * dDelta);
 	  dAspect =  (1.0 - dTempHOC) / (1.0 - dTempVOC);
-	  if (callback) callback (dTempHOC, dTempVOC, dMetric, dAspect);
-          if(dMetric < dBestMetric)
-          {
-            dBestMetric = dMetric;
-            dBestHOC = dTempHOC;
-            dBestVOC = dTempVOC;
-            dBestVSR = dTempVSR;
-            dBestTLI = dTempTLI;
-            dBestTPI = dTempTPI;
-            dBestHCLKO = dTempHPIXELO;
-          }
-        }                             
-      }                                 
-    }                                            
+	  flag = 0;
+	  if (((int) dTempTPI) % 8 == 0) flag |= BT_CALC_CHARCLK8;
+	  if (((int) dTempTPI) % 9 == 0) flag |= BT_CALC_CHARCLK9;
+	  if (actCLKRATIO) flag |= BT_CALC_RATIO32;
+	  if (callback) callback (dTempHOC, dTempVOC, dMetric, dAspect, 
+				  flag);
+	  if(dMetric < dBestMetric)
+	  {
+	    dBestMetric = dMetric;
+	    dBestHOC = dTempHOC;
+	    dBestVOC = dTempVOC;
+	    dBestVSR = dTempVSR;
+	    dBestTLI = dTempTLI;
+	    dBestTPI = dTempTPI;
+	    dBestHCLKO = dTempHCLKO;
+	    dBestCLKRATIO = dTempCLKRATIO;
+	  }
+	} /* valid solution */                             
+      } /* dTempHCLKO loop */
+    } /* dTempTLI loop */
+  } /* CLKRATIO loop */
 
-    if(dBestMetric == 1000)
-    {
-      //  error
+  if(dBestMetric == 1000)
+  {
+    //  error
 #if 0
-      DebugOut("error: dBestMetric == 1000");
+    DebugOut("error: dBestMetric == 1000");
 #endif
-      return;
-    }
-
-    pS->dVSR = dBestVSR;
-    pS->dVSCALE = (long)((dBestVSR - 1) * 4096 + 0.5);
-    pS->dVLINESI = (long)dBestTLI;
-    pS->dHCLKO = (long)dBestHCLKO;
-    pS->dHFRACT = 0;
-    pS->dHCLKI = (long)dBestTPI;
-    pS->dVACTIVEO = (long)((pS->dVACTIVEI + 3) / dBestVSR) + 1;
-    pS->dHOC = dBestHOC;
-    pS->dVOC = dBestVOC;
+    return;
   }
+
+  pS->dVSR = dBestVSR;
+  pS->dVSCALE = (long)((dBestVSR - 1) * 4096 + 0.5);
+  pS->dVLINESI = (long)dBestTLI;
+  pS->dHCLKO = (long)dBestHCLKO;
+  pS->dHFRACT = 0;
+  pS->dHCLKI = (long)dBestTPI;
+  pS->dVACTIVEO = (long)((pS->dVACTIVEI + 3) / dBestVSR) + 1;
+  pS->dHOC = dBestHOC;
+  pS->dVOC = dBestVOC;
+  pS->dCLKRATIO = dBestCLKRATIO;
 }
 
 void Bt868ReCalc(PENCODER_SETTINGS pS)
@@ -560,11 +587,15 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
   double  dHeadRoom = 0;
   double  dMaxHR = 0;
   double  dFifoMargin = 0;
-  double  dFifoSize = 800;
+  double  dFifoSize;
 
+  dFifoSize = 800;
+  if (pS->dwImplementationFlags & BT_IMPLEMENTATION_FLAGS_CONEXANT) {
+    dFifoSize = 1024;
+  }
   recalc_outfield (pS);
   recalc_tvformat (pS); /* on pS->ucTVOutMode */
-  recalc_prepare (pS);
+  recalc_prepare (pS); /* FIXME: shouldn't have to do full prepare ... */
   recalc_find_oc (pS, pS->dHOC - .05, pS->dHOC + .05, 
 		      pS->dVOC - .05, pS->dVOC + .05, NULL);
   if( pS->dwImplementationFlags & BT868_IMPLEMENTATION_FLAGS_CCIR601)
@@ -591,10 +622,13 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
     pS->dwInOutScanFlags &= ~BT868_INOUTSCAN_FLAGS_IN_INTERLACED;
   }
 
+  /* Calculate parameters relative to HCLKO, i.e. numbers represent fractions
+     of a whole line. The HXTAL constant is F_sc / F_line, i.e.
+     HXTAL = F_xtal / F_line. */
   switch( pS->ucTVOutMode )
   {
-    case 0:
-    case 1:
+    case 0: /* NTSC  */
+    case 1: /* NTSC-50 ?? */
       pS->dVBLANKO = (long)( 140 - ( pS->dVACTIVEO / 2.0 ) + 0.5 );
       pS->dHBURSTBEGIN = 2 * (long)( ( 0.0416958041958 * pS->dHCLKO ) + 0.5 );
       pS->dHBURSTEND = 2 * (long)( ( 0.06270104895105 * pS->dHCLKO ) + 0.5) - 128;
@@ -603,7 +637,7 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
       pS->dMSC = (long)( 0x80000000 * ( 455.0 / pS->dHCLKO ) + 0.5 ) + pS->dPHZINC;
       pS->dHXTAL = ( pS->dFXTAL * 227.5 ) / pS->dFSC;
       break;
-    case 3:
+    case 3: /* PAL-N */
       pS->dVBLANKO = (long)( 167 - ( pS->dVACTIVEO / 2.0 ) + 0.5 );
       pS->dHBURSTBEGIN = (long)( ( 0.0875 * pS->dHCLKO ) + 0.5 );
       pS->dHBURSTEND = (long)( ( 0.12265625 * pS->dHCLKO ) + 0.5 ) - 128;
@@ -612,7 +646,7 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
       pS->dMSC = (long)( ( ( 567.5032 / pS->dHCLKO ) * 0x80000000 ) + 0.5 ) + pS->dPHZINC;
       pS->dHXTAL = ( pS->dFXTAL * 283.7516 ) / pS->dFSC;
       break;
-    case 4:
+    case 4: /* PAL-Nc */
       pS->dVBLANKO = (long)( 167 - ( pS->dVACTIVEO / 2.0 ) + 0.5 );
       pS->dHBURSTBEGIN = 2 * (long)( ( 0.04375 * pS->dHCLKO ) + 0.5 );
       pS->dHBURSTEND = 2 * (long)( ( 0.063359375 * pS->dHCLKO ) + 0.5 ) - 128;
@@ -621,7 +655,7 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
       pS->dMSC = (long)( ( ( 458.5032 / pS->dHCLKO ) * 0x80000000 ) + 0.5 ) + pS->dPHZINC;
       pS->dHXTAL = ( pS->dFXTAL * 229.2516 ) / pS->dFSC;
       break;
-    case 5:
+    case 5: /* PAL-M */
       pS->dVBLANKO = (long)( 140 - ( pS->dVACTIVEO / 2.0 ) + 0.5);
       pS->dHBURSTBEGIN = (long)( ( 0.09125874125874 * pS->dHCLKO ) + 0.5 );
       pS->dHBURSTEND = 2 * (long)( ( 0.06545454545455 * pS->dHCLKO ) + 0.5 ) - 128;
@@ -630,7 +664,7 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
       pS->dMSC = (long)( 0x80000000 * ( 454.5 / pS->dHCLKO ) + 0.5 ) + pS->dPHZINC;
       pS->dHXTAL = ( pS->dFXTAL * 227.25 ) / pS->dFSC;
       break;
-    case 7:
+    case 7: /* PAL-60 ? */
       pS->dVBLANKO = (long)( 140 - ( pS->dVACTIVEO / 2.0 ) + 0.5 );
       pS->dHBURSTBEGIN = 2 * (long)( 151.2 * ( pS->dHCLKO / 3432 ) + 0.5 );
       pS->dHBURSTEND = 2 * (long)( ( 211.95 * ( pS->dHCLKO / 3432 ) ) + 0.5 ) - 128;
@@ -642,7 +676,7 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
       //pS->dHXTAL = ( pS->dFXTAL * 281.7811338363 ) / pS->dFSC;
       pS->dHXTAL = ( pS->dFXTAL * ( 2216809375.0 / 7867132.0 ) ) / ( pS->dFSC );
       break;
-    default:
+    default: /* PAL-BDGHI, PAL-M60 ? */
       //  625-line Systems
       pS->dVBLANKO = (long)( 167 - ( pS->dVACTIVEO / 2.0 ) + 0.5 );
       pS->dHBURSTBEGIN = 2 * (long)( ( 0.04375 * pS->dHCLKO ) + 0.5 );
@@ -754,8 +788,8 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
   }
 
 
-  pS->dPLLRATIO = (long)( 6.0 * ( pS->dHCLKO / pS->dHXTAL ) * 0x10000 + 0.5 );
-
+  pS->dPLLRATIO = (long)(6.0 * (pS->dHCLKO / pS->dHXTAL) * pS->dCLKRATIO *
+			 0x10000 + 0.5);
   
   if( pS->dwImplementationFlags & BT868_IMPLEMENTATION_FLAGS_DOUBLEPLL )
   {
@@ -810,7 +844,7 @@ void Bt868ReCalc(PENCODER_SETTINGS pS)
   }
 }
 
-void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
+void recalc_system (TVSystem system, PENCODER_SETTINGS pS, TVBtRegs *b)
 {
   pS->dwInOutScanFlags |= BT868_INOUTSCAN_FLAGS_OUT_INTERLACED;
   pS->ucCLPF = 3;
@@ -818,7 +852,7 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
   switch (system) 
     {
     case TV_SYSTEM_NTSC:
-      pS->ucTVOutMode = 0;
+      pS->ucTVOutMode = 0; /* NTSC */
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_VSYNCDUR;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_625LINE;
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_SETUP;
@@ -826,15 +860,15 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
       if (b) b->flags1 = BT_FLAG1_NTSC;
       break;
     case TV_SYSTEM_NTSC_J:
-      pS->ucTVOutMode = 1; /* NTSC_60 */
+      pS->ucTVOutMode = 0; /* NTSC */
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_VSYNCDUR;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_625LINE;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_SETUP;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_PAL;
-      if (b) b->flags1 = BT_FLAG1_NTSC_JAPAN;
+      if (b) b->flags1 = BT_FLAG1_NTSC_J;
       break;
     case TV_SYSTEM_PAL: 
-      pS->ucTVOutMode = 2; 
+      pS->ucTVOutMode = 2; /* PAL-BDGHI */
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_VSYNCDUR;
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_625LINE;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_SETUP;
@@ -842,7 +876,7 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
       if (b) b->flags1 = BT_FLAG1_PAL_BDGHI;
       break;
     case TV_SYSTEM_PAL_N:
-      pS->ucTVOutMode = 3; 
+      pS->ucTVOutMode = 3; /* PAL-N */
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_VSYNCDUR;
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_625LINE;
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_SETUP;
@@ -850,7 +884,7 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
       if (b) b->flags1 = BT_FLAG1_PAL_N;
       break;
     case TV_SYSTEM_PAL_NC:
-      pS->ucTVOutMode = 4; 
+      pS->ucTVOutMode = 4; /* PAL-Nc */ 
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_VSYNCDUR;
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_625LINE;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_SETUP;
@@ -858,7 +892,7 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
       if (b) b->flags1 = BT_FLAG1_PAL_BDGHI;
       break;
     case TV_SYSTEM_PAL_M:
-      pS->ucTVOutMode = 5; 
+      pS->ucTVOutMode = 5; /* PAL-M */
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_VSYNCDUR;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_625LINE;
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_SETUP;
@@ -866,7 +900,7 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
       if (b) b->flags1 = BT_FLAG1_PAL_M;
       break;
     case TV_SYSTEM_PAL_60: 
-      pS->ucTVOutMode = 7; 
+      pS->ucTVOutMode = 7; /* PAL-60 */
       pS->dwRegisterFlags |= BT868_REGISTER_FLAGS_VSYNCDUR;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_625LINE;
       pS->dwRegisterFlags &= ~BT868_REGISTER_FLAGS_SETUP;
@@ -874,7 +908,7 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
       if (b) b->flags1 = BT_FLAG1_PAL_60;
       break;
     case TV_SYSTEM_PAL_M60:
-      pS->ucTVOutMode = 6; 
+      pS->ucTVOutMode = 6; /* PAL-60 */
       /* Not documented ... */
       break; 
     default:
@@ -882,13 +916,14 @@ void recalc_system (NVSystem system, PENCODER_SETTINGS pS, NVBtRegs *b)
   }
 }
 
-void recalc_find (NVSystem system, int hres, int vres, 
+void recalc_find (TVSystem system, int hres, int vres, 
   double hoc_min, double hoc_max, double voc_min, double voc_max,
-  RecalcFindRes callback)
+  RecalcFindRes callback, int flags)
 {
   struct settings s;
 
   Bt868EncoderInitializeDefault (&s);
+  s.dwImplementationFlags |= flags;
   recalc_system (system, &s, NULL);
   s.dHOC = (hoc_min + hoc_max) / 2.0; /* FIXME */
   s.dVOC = (voc_min + voc_max) / 2.0;
@@ -896,68 +931,142 @@ void recalc_find (NVSystem system, int hres, int vres,
   s.dVACTIVEI = vres;
   recalc_outfield (&s);
   recalc_tvformat (&s); /* on s.ucTVOutMode */
-  recalc_prepare (&s);
+  recalc_prepare (&s); /* FIXME: shouldn't have to do full prepare ... */
   recalc_find_oc (&s, hoc_min, hoc_max, voc_min, voc_max, callback);
 }
 
-void recalc_custom (NVSystem system, int hres, int vres, 
-  double hoc, double voc, NVBtRegs *b, NVCrtRegs *m)
+void recalc_bt_find (TVSystem system, int hres, int vres, 
+  double hoc_min, double hoc_max, double voc_min, double voc_max,
+  RecalcFindRes callback)
+{
+  recalc_find (system, hres, vres, hoc_min, hoc_max, voc_min, voc_max,
+	       callback, 0);
+}
+
+void recalc_cx_find (TVSystem system, int hres, int vres, 
+  double hoc_min, double hoc_max, double voc_min, double voc_max,
+  RecalcFindRes callback)
+{
+  recalc_find (system, hres, vres, hoc_min, hoc_max, voc_min, voc_max,
+	       callback, BT_IMPLEMENTATION_FLAGS_CONEXANT);
+}
+
+void calc_nv_btcx (int hres, int vres, TVRegs *r)
+{
+  /* This is just in approximation, and by no means accurate. */
+
+  r->crtc.nv.HDisplay    = hres;
+  r->crtc.nv.HTotal      = r->enc.bt.h_clki & ~0x7;
+  /* Some attempts to get good sync values: */
+  /* r->crtc.nv.HSyncStart  = 
+    (r->crtc.nv.HTotal + r->crtc.nv.HDisplay) / 2 + 8; */
+  r->crtc.nv.HSyncStart = 
+    ((r->crtc.nv.HTotal - r->enc.bt.h_blanki) & ~0x7) + 48;
+  /* 48 seems to work in many PAL cases, but it may be completely wrong. */
+
+  /* Now try to sanitize at least the worst cases */
+  if (r->crtc.nv.HSyncStart <= r->crtc.nv.HDisplay) 
+    r->crtc.nv.HSyncStart = r->crtc.nv.HDisplay + 8;
+  if (r->crtc.nv.HSyncStart >= r->crtc.nv.HTotal) 
+    r->crtc.nv.HSyncStart = r->crtc.nv.HDisplay + 8;
+  r->crtc.nv.HSyncEnd    = r->crtc.nv.HSyncStart + 24; /* wild guess */
+  if (r->crtc.nv.HSyncEnd >= r->crtc.nv.HTotal) 
+    r->crtc.nv.HSyncStart = r->crtc.nv.HTotal;
+
+  r->crtc.nv.VDisplay    = vres;
+  r->crtc.nv.VTotal      = r->enc.bt.v_linesi; 
+  /* Some attempts to get good sync values: */
+  /* r->crtc.nv.VSyncStart = 
+     (r->crtc.nv.VTotal + r->crtc.nv.VDisplay) / 2 + 2; */
+  r->crtc.nv.VSyncStart = r->crtc.nv.VTotal - r->enc.bt.v_blanki;
+  if (r->crtc.nv.VSyncStart <= r->crtc.nv.VDisplay) 
+    r->crtc.nv.VSyncStart = r->crtc.nv.VDisplay + 8;
+  r->crtc.nv.VSyncEnd    = r->crtc.nv.VSyncStart + 2;
+
+  r->crtc.nv.flags       = 0;
+  r->devFlags = DEV_TELEVISION | DEV_MONITOR;
+}
+
+void recalc_bt_copy (PENCODER_SETTINGS s, TVBtRegs *r)
+{
+  r->hsynoffset   = (int) s->dHSYNOFFSET;
+  r->vsynoffset   = (int) s->dVSYNOFFSET;
+  r->hsynwidth    = (int) s->dHSYNWIDTH;
+  r->vsynwidth    = (int) s->dVSYNWIDTH;
+  r->h_clko       = (int) s->dHCLKO;
+  r->h_active     = (int) s->dHACTIVEI;
+  r->hsync_width  = (int) s->dHSYNCWIDTH;
+  r->hburst_begin = (int) s->dHBURSTBEGIN;
+  r->hburst_end   = (int) s->dHBURSTEND;
+  r->h_blanko     = (int) s->dHBLANKO;
+  r->v_blanko     = (int) s->dVBLANKO;
+  r->v_activeo    = (int) s->dVACTIVEO;
+  r->h_fract      = (int) s->dHFRACT;
+  r->h_clki       = (int) s->dHCLKI;
+  r->h_blanki     = (int) s->dHBLANKI;
+  r->v_linesi     = (int) s->dVLINESI;
+  r->v_blanki     = (int) s->dVBLANKI;
+  r->v_activei    = (int) s->dVACTIVEI;
+  r->v_scale      = (int) s->dVSCALE;
+  r->pll_fract    = (int) s->dPLLFRACT;
+  r->pll_int      = (int) s->dPLLINT;
+  r->sync_amp     = (int) s->dSYNCAMP;
+  r->bst_amp      = (int) s->dBSTAMP;
+  r->mcr          = (int) s->dMCR;
+  r->mcb          = (int) s->dMCB;
+  r->my           = (int) s->dMY;
+  r->msc          = (long) s->dMSC;
+}
+
+void recalc_bt_custom (TVSystem system, int hres, int vres, 
+  double hoc, double voc, TVRegs *r)
 {
   struct settings s;
 
   Bt868EncoderInitializeDefault (&s);
-  recalc_system (system, &s, b);
+  recalc_system (system, &s, &r->enc.bt);
   s.dHOC = hoc;
   s.dVOC = voc;
   s.dHACTIVEI = hres & ~0x7;
   s.dVACTIVEI = vres;
   Bt868ReCalc (&s);
-  b->hsynoffset   = (int) s.dHSYNOFFSET;
-  b->vsynoffset   = (int) s.dVSYNOFFSET;
-  b->hsynwidth    = (int) s.dHSYNWIDTH;
-  b->vsynwidth    = (int) s.dVSYNWIDTH;
-  b->h_clko       = (int) s.dHCLKO;
-  b->h_active     = (int) s.dHACTIVEI;
-  b->hsync_width  = (int) s.dHSYNCWIDTH;
-  b->hburst_begin = (int) s.dHBURSTBEGIN;
-  b->hburst_end   = (int) s.dHBURSTEND;
-  b->h_blanko     = (int) s.dHBLANKO;
-  b->v_blanko     = (int) s.dVBLANKO;
-  b->v_activeo    = (int) s.dVACTIVEO;
-  b->h_fract      = (int) s.dHFRACT;
-  b->h_clki       = (int) s.dHCLKI;
-  b->h_blanki     = (int) s.dHBLANKI;
-  b->v_blank_dly  = 0;
-  b->v_linesi     = (int) s.dVLINESI;
-  b->v_blanki     = (int) s.dVBLANKI;
-  b->v_activei    = (int) s.dVACTIVEI;
-  b->v_scale      = (int) s.dVSCALE;
-  b->pll_fract    = (int) s.dPLLFRACT;
-  b->pll_int      = (int) s.dPLLINT;
-  b->sync_amp     = (int) s.dSYNCAMP;
-  b->bst_amp      = (int) s.dBSTAMP;
-  b->mcr          = (int) s.dMCR;
-  b->mcb          = (int) s.dMCB;
-  b->my           = (int) s.dMY;
-  b->msc          = (long) s.dMSC;
-
-  m->HDisplay    = hres;
-  m->HTotal      = b->h_clki & ~0x7;
-  /* Some attempts to get good sync values: */
-  /* m->HSyncStart  = (m->HTotal + m->HDisplay) / 2 + 8; */
-  m->HSyncStart  = (m->HTotal - b->h_blanki) & ~0x7 + 48;
-  /* Try to sanitize at least the worst cases */
-  if (m->HSyncStart <= m->HDisplay) m->HSyncStart = m->HDisplay + 8;
-  m->HSyncEnd    = m->HSyncStart + 24; /* wild guess */
-
-  m->VDisplay    = vres;
-  m->VTotal      = b->v_linesi; 
-  /* Some attempts to get good sync values: */
-  /* m->VSyncStart  = (m->VTotal + m->VDisplay) / 2 + 2; */
-  m->VSyncStart  = m->VTotal - b->v_blanki;
-  if (m->VSyncStart <= m->VDisplay) m->VSyncStart = m->VDisplay + 8;
-  m->VSyncEnd    = m->VSyncStart + 2;
-
-  m->Flags       = 0;
-  m->PrivFlags   = NV_MODE_TVMODE | NV_MODE_DUALVIEW | NV_MODE_MACROVIS_OFF;
+  recalc_bt_copy (&s, &r->enc.bt);
 }
+
+void recalc_cx_custom (TVSystem system, int hres, int vres, 
+  double hoc, double voc, TVRegs *r)
+{
+  struct settings s;
+
+  Bt868EncoderInitializeDefault (&s);
+  s.dwImplementationFlags |= BT_IMPLEMENTATION_FLAGS_CONEXANT;
+  recalc_system (system, &s, &r->enc.cx.bt);
+  s.dHOC = hoc;
+  s.dVOC = voc;
+  s.dHACTIVEI = hres & ~0x7;
+  s.dVACTIVEI = vres;
+  Bt868ReCalc (&s);
+  recalc_bt_copy (&s, &r->enc.cx.bt);
+  r->enc.bt.flags1 |= CX_FLAG1_EXT;
+  r->enc.cx.wsdat = 0;
+  if (s.dCLKRATIO != 1.0) r->enc.cx.flags5 |= CX_FLAG5_PLL_32CLK;
+  /* Remaining init will be done in data_init_cx */
+}
+
+/*
+
+Mode     Total
+         
+         872 +185 1000  34.5  MHz  815  +43
+Large    840 +209 1000  33.25 MHz  791  +49
+Large    832 +217   "     "
+
+Small800 816 +153  960  36    MHz  807   +9
+Small800 784 +185   "
+
+Small768 816 +241 1000  34    MHz  759  +57
+
+         816 +241 1000  32.75 MHz
+
+*/
+
