@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: card_direct.h,v 1.5 2003/10/10 16:16:57 dthierbach Exp $
  *
  * Contents:
  *
@@ -41,6 +41,4 @@ void unmapDevMem (CardPtr card, unsigned long Base, unsigned long Size);
 void scan_cards_pci (CardPtr *card_root_list, char *devname);
 void scan_cards_nvdev (CardPtr *card_nvdev_list);
 
-#endif
-
-
+#endif /* _CARD_DIRECT_H */
