@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: data.c,v 1.27 2003/10/15 10:57:58 dthierbach Exp $
  *
  * Contents:
  *
@@ -111,10 +111,10 @@ TVMode *data_find (TVMode *modes, TVSystem system,
     modes++;
   }
   if (modes->spec.system != TV_SYSTEM_NONE) {
-    DPRINTF ("found mode\n");
+    RAISE (MSG_DEBUG, "found mode");
     return modes;
   } else {
-    DPRINTF ("found no mode\n");
+    RAISE (MSG_DEBUG, "found no mode");
     return NULL;
   }
 }
@@ -210,12 +210,16 @@ DataFunc *data_func (CardType card, TVChip chip)
       return &data_nv_ph1_func; 
     case (CARD_NVIDIA << CARD_SHIFT) | TV_PHILIPS_MODEL2:  
       return &data_nv_ph2_func; 
+    case (CARD_NVIDIA << CARD_SHIFT) | TV_NVIDIA:  
+      return &data_nv_nx_func; 
     case (CARD_XBOX   << CARD_SHIFT) | TV_CONEXANT: 
       return &data_xbox_cx_func;
     case (CARD_TDFX   << CARD_SHIFT) | TV_BROOKTREE: 
       return &data_tdfx_bt_func;
     case (CARD_I810   << CARD_SHIFT) | TV_CHRONTEL_MODEL1:  
       return &data_i810_ch1_func; 
+    case (CARD_I810   << CARD_SHIFT) | TV_CHRONTEL_MODEL2:  
+      return &data_i810_ch2_func; 
     default: 
       return &null_func;
   }
