@@ -16,20 +16,56 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: data_ch.c,v 1.33 2004/01/30 08:47:01 dthierbach Exp $
  *
  * Contents:
  *
  * Data tables and setup routines for the Chrontel chip.
  */ 
 
+#include "local.h" /* before everything else */
+
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
 #include "nv_type.h"
 #include "data_ch.h"
 
-/* -------- CH -------- NTSC -------- */
+#define fsci_t unsigned long 
+
+/* Absolute overscan (CH2 docs have it negative) to name:
+
+-4   0     4 6 10    13   14 16 17  18  20 22 25 27 30   33
+cjlm abfhn e d f     aikn cd ej glm abn fh d  e  cl bikm g
+
+<--Huge----> <Large> <----Small---> <--------Tiny--------->
+
+TODO: Rename some large to huge modes, and calculate overscan properly
+FIX calc overscan
+
+NTSC 640x480 640x400 720x400
+PAL  800x600 640x480
+
+CH1:
+
+a = 640x480
+b = 800x600
+c = 640x400
+d = 720x400
+e = 512x384
+f = DVD (800x600) 768x576
+g = DVD (800x600) 800x450
+
+CH2:
+
+a = 1024x768
+b = 800x600
+c = 720x480
+d = 720x576
+
+*/
+
+/* -------- CH1 -------- NTSC -------- */
 
 /* FSCI: NTSC, NTSC-NDC, PAL-M, PAL-60 */
 
@@ -37,7 +73,7 @@
 
 /* -------- 640x480 -------- */
 
-TVChRegs ch_ntsc_large_a = { /* Mode 16: 640x480 1:1  */
+TVChRegs ch1_ntsc_huge_a = { /* Mode 16: 640x480 1:1  */
   dmr_ir  : 3,
   dmr_vs  : 1,
   dmr_sr  : 1,
@@ -51,10 +87,9 @@ TVChRegs ch_ntsc_large_a = { /* Mode 16: 640x480 1:1  */
   mode    : 16,
 };
 
-unsigned long ch_fsci_ntsc_large_a [] = 
-  {623153737, 623156346, 622468953, 770448543};
+fsci_t ch1_fsci_ntsc_huge_a [] = {623153737, 623156346, 622468953, 770448543};
 
-TVChRegs ch_ntsc_small_a = { /* Mode 17: 640x480 7:8  */
+TVChRegs ch1_ntsc_small_a = { /* Mode 17: 640x480 7:8  */
   dmr_ir  : 3,
   dmr_vs  : 1,
   dmr_sr  : 2,
@@ -68,10 +103,9 @@ TVChRegs ch_ntsc_small_a = { /* Mode 17: 640x480 7:8  */
   mode    : 17,
 };
 
-unsigned long ch_fsci_ntsc_small_a [] = 
-  {545259520, 545261803, 544660334, 674142475};
+fsci_t ch1_fsci_ntsc_small_a [] = {545259520, 545261803, 544660334, 674142475};
 
-TVChRegs ch_ntsc_tiny_a = { /* Mode 18: 640x480 5:6  */
+TVChRegs ch1_ntsc_tiny_a = { /* Mode 18: 640x480 5:6  */
   dmr_ir  : 3,
   dmr_vs  : 1,
   dmr_sr  : 3,
@@ -85,12 +119,11 @@ TVChRegs ch_ntsc_tiny_a = { /* Mode 18: 640x480 5:6  */
   mode    : 18,
 };
 
-unsigned long ch_fsci_ntsc_tiny_a [] = 
-  {508908885, 508911016, 508349645, 629199644};
+fsci_t ch1_fsci_ntsc_tiny_a [] = {508908885, 508911016, 508349645, 629199644};
 
 /* -------- 800x600 -------- */
 
-TVChRegs ch_ntsc_huge_b = { /* Mode 22: 800x600 5:6  */
+TVChRegs ch1_ntsc_huge_b = { /* Mode 22: 800x600 5:6  */
   dmr_ir  : 4,
   dmr_vs  : 1,
   dmr_sr  : 3,
@@ -104,10 +137,9 @@ TVChRegs ch_ntsc_huge_b = { /* Mode 22: 800x600 5:6  */
   mode    : 22,
 };
 
-unsigned long ch_fsci_ntsc_huge_b [] = 
-  {521957831, 521960016, 521384251, 645332967};
+fsci_t ch1_fsci_ntsc_huge_b [] = {521957831, 521960016, 521384251, 645332967};
 
-TVChRegs ch_ntsc_large_b = { /* Mode 23: 800x600 3:4  */
+TVChRegs ch1_ntsc_large_b = { /* Mode 23: 800x600 3:4  */
   dmr_ir  : 4,
   dmr_vs  : 1,
   dmr_sr  : 4,
@@ -121,10 +153,9 @@ TVChRegs ch_ntsc_large_b = { /* Mode 23: 800x600 3:4  */
   mode    : 23,
 };
 
-unsigned long ch_fsci_ntsc_large_b [] = 
-  {469762048, 469764015, 469245826, 580799671};
+fsci_t ch1_fsci_ntsc_large_b [] = {469762048, 469764015, 469245826, 580799671};
 
-TVChRegs ch_ntsc_small_b = { /* Mode 24: 800x600 7:10  */
+TVChRegs ch1_ntsc_small_b = { /* Mode 24: 800x600 7:10  */
   dmr_ir  : 4,
   dmr_vs  : 1,
   dmr_sr  : 5,
@@ -138,12 +169,11 @@ TVChRegs ch_ntsc_small_b = { /* Mode 24: 800x600 7:10  */
   mode    : 24,
 };
 
-unsigned long ch_fsci_ntsc_small_b [] = 
-  {428554851, 428556645, 428083911, 529852331};
+fsci_t ch1_fsci_ntsc_small_b [] = {428554851, 428556645, 428083911, 529852331};
 
 /* -------- 640x400 -------- */
 
-TVChRegs ch_ntsc_large_c = { /* Mode 10: 640x400 5:4  */
+TVChRegs ch1_ntsc_huge_c = { /* Mode 10: 640x400 5:4  */
   dmr_ir  : 2,
   dmr_vs  : 1,
   dmr_sr  : 0,
@@ -157,10 +187,9 @@ TVChRegs ch_ntsc_large_c = { /* Mode 10: 640x400 5:4  */
   mode    : 10,
 };
 
-unsigned long ch_fsci_ntsc_large_c [] = 
-  {646233505, 646236211, 645523358, 798983674};
+fsci_t ch1_fsci_ntsc_huge_c [] = {646233505, 646236211, 645523358, 798983674};
 
-TVChRegs ch_ntsc_small_c = { /* Mode 11: 640x400 1:1  */
+TVChRegs ch1_ntsc_small_c = { /* Mode 11: 640x400 1:1  */
   dmr_ir  : 2,
   dmr_vs  : 1,
   dmr_sr  : 1,
@@ -174,10 +203,9 @@ TVChRegs ch_ntsc_small_c = { /* Mode 11: 640x400 1:1  */
   mode    : 11,
 };
 
-unsigned long ch_fsci_ntsc_small_c [] = 
-  {516986804, 516988968, 516418687, 639186940};
+fsci_t ch1_fsci_ntsc_small_c [] = {516986804, 516988968, 516418687, 639186940};
 
-TVChRegs ch_ntsc_tiny_c = { /* Mode 12: 640x400 7:8  */
+TVChRegs ch1_ntsc_tiny_c = { /* Mode 12: 640x400 7:8  */
   dmr_ir  : 2,
   dmr_vs  : 1,
   dmr_sr  : 3,
@@ -191,12 +219,11 @@ TVChRegs ch_ntsc_tiny_c = { /* Mode 12: 640x400 7:8  */
   mode    : 12,
 };
 
-unsigned long ch_fsci_ntsc_tiny_c [] = 
-  {452363454, 452365347, 451866351, 559288572};
+fsci_t ch1_fsci_ntsc_tiny_c [] = {452363454, 452365347, 451866351, 559288572};
 
 /* -------- 720x400 -------- */
 
-TVChRegs ch_ntsc_large_d = { /* Mode 6: 720x400 5:4  */
+TVChRegs ch1_ntsc_huge_d = { /* Mode 6: 720x400 5:4  */
   dmr_ir  : 1,
   dmr_vs  : 1,
   dmr_sr  : 0,
@@ -210,10 +237,9 @@ TVChRegs ch_ntsc_large_d = { /* Mode 6: 720x400 5:4  */
   mode    : 6,
 };
 
-unsigned long ch_fsci_ntsc_large_d [] = 
-  {574429782, 574432187, 573798541, 710207711};
+fsci_t ch1_fsci_ntsc_huge_d [] = {574429782, 574432187, 573798541, 710207711};
 
-TVChRegs ch_ntsc_small_d = { /* Mode 7: 720x400 1:1  */
+TVChRegs ch1_ntsc_small_d = { /* Mode 7: 720x400 1:1  */
   dmr_ir  : 1,
   dmr_vs  : 1,
   dmr_sr  : 1,
@@ -227,12 +253,11 @@ TVChRegs ch_ntsc_small_d = { /* Mode 7: 720x400 1:1  */
   mode    : 7,
 };
 
-unsigned long ch_fsci_ntsc_small_d [] = 
-  {463962517, 463964459, 463452668, 573629305};
+fsci_t ch1_fsci_ntsc_small_d [] = {463962517, 463964459, 463452668, 573629305};
 
 /* -------- 512x384 -------- */
 
-TVChRegs ch_ntsc_large_e = { /* Mode 2: 512x384 5:4  */
+TVChRegs ch1_ntsc_large_e = { /* Mode 2: 512x384 5:4  */
   dmr_ir  : 0,
   dmr_vs  : 1,
   dmr_sr  : 0,
@@ -246,10 +271,9 @@ TVChRegs ch_ntsc_large_e = { /* Mode 2: 512x384 5:4  */
   mode    : 2,
 };
 
-unsigned long ch_fsci_ntsc_large_e [] = 
-  {763363328, 763366524, 762524467, 943799465};
+fsci_t ch1_fsci_ntsc_large_e [] = {763363328, 763366524, 762524467, 943799465};
 
-TVChRegs ch_ntsc_small_e = { /* Mode 3: 512x384 1:1  */
+TVChRegs ch1_ntsc_small_e = { /* Mode 3: 512x384 1:1  */
   dmr_ir  : 0,
   dmr_vs  : 1,
   dmr_sr  : 1,
@@ -263,8 +287,7 @@ TVChRegs ch_ntsc_small_e = { /* Mode 3: 512x384 1:1  */
   mode    : 3,
 };
 
-unsigned long ch_fsci_ntsc_small_e [] = 
-  {623153737, 623156346, 622468953, 770448543};
+fsci_t ch1_fsci_ntsc_small_e [] = {623153737, 623156346, 622468953, 770448543};
 
 /* -------- DVD convenience modes -------- */
 
@@ -274,7 +297,7 @@ unsigned long ch_fsci_ntsc_small_e [] =
    Based on 800x600 NTSC Large (Mode 23)
 */
 
-TVChRegs ch_ntsc_large_f = { /* Mode 23: 800x600 3:4  */
+TVChRegs ch1_ntsc_large_f = { /* Mode 23: 800x600 3:4  */
   dmr_ir  : 4,
   dmr_vs  : 1,
   dmr_sr  : 4,
@@ -288,7 +311,7 @@ TVChRegs ch_ntsc_large_f = { /* Mode 23: 800x600 3:4  */
   mode    : 23,
 };
 
-TVChRegs ch_ntsc_large_g = { /* Mode 23: 800x600 3:4  */
+TVChRegs ch1_ntsc_large_g = { /* Mode 23: 800x600 3:4  */
   dmr_ir  : 4,
   dmr_vs  : 1,
   dmr_sr  : 4,
@@ -302,13 +325,13 @@ TVChRegs ch_ntsc_large_g = { /* Mode 23: 800x600 3:4  */
   mode    : 23,
 };
 
-/* Use ch_fsci_ntsc_large_b */
+/* Use ch1_fsci_ntsc_large_b */
 
 /* -------- 720x{576,480} DVD -------- */
 
 /* Note: Not used yet, crt values not tested */
 
-TVChRegs ch_ntsc_interl_dvd = { /* Mode 26: 720x480 IL  */
+TVChRegs ch1_ntsc_interl_dvd = { /* Mode 26: 720x480 IL  */
   dmr_ir  : 5,
   dmr_vs  : 1,
   dmr_sr  : 1,
@@ -326,13 +349,13 @@ TVChRegs ch_ntsc_interl_dvd = { /* Mode 26: 720x480 IL  */
 /* Note: Mode 26 must use 2X XClk. So need to program that. For
    CRT, need to use double hscan, and interlace. */
 
-/* -------- CH -------- PAL -------- */
+/* -------- CH1 -------- PAL -------- */
 
 /* FSCI: PAL, PAL-N, PAL-X */
 
 /* -------- 640x480 -------- */
 
-TVChRegs ch_pal_large_a = { /* Mode 13: 640x480 5:4  */
+TVChRegs ch1_pal_huge_a = { /* Mode 13: 640x480 5:4  */
   dmr_ir  : 3,
   dmr_vs  : 0,
   dmr_sr  : 0,
@@ -346,9 +369,9 @@ TVChRegs ch_pal_large_a = { /* Mode 13: 640x480 5:4  */
   mode    : 13,
 };
 
-unsigned long ch_fsci_pal_large_a [] = {806021060, 651209077, 806120400};
+fsci_t ch1_fsci_pal_huge_a [] = {806021060, 651209077, 806120400};
 
-TVChRegs ch_pal_small_a = { /* Mode 14: 640x480 1:1  */
+TVChRegs ch1_pal_small_a = { /* Mode 14: 640x480 1:1  */
   dmr_ir  : 3,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -362,9 +385,9 @@ TVChRegs ch_pal_small_a = { /* Mode 14: 640x480 1:1  */
   mode    : 14,
 };
 
-unsigned long ch_fsci_pal_small_a [] = {644816848, 520967262, 644862000};
+fsci_t ch1_fsci_pal_small_a [] = {644816848, 520967262, 644862000};
 
-TVChRegs ch_pal_tiny_a = { /* Mode 15: 640x480 5:6  */
+TVChRegs ch1_pal_tiny_a = { /* Mode 15: 640x480 5:6  */
   dmr_ir  : 3,
   dmr_vs  : 0,
   dmr_sr  : 3,
@@ -378,11 +401,11 @@ TVChRegs ch_pal_tiny_a = { /* Mode 15: 640x480 5:6  */
   mode    : 15,
 };
 
-unsigned long ch_fsci_pal_tiny_a [] = {537347373, 434139385, 537430027};
+fsci_t ch1_fsci_pal_tiny_a [] = {537347373, 434139385, 537430027};
 
 /* -------- 800x600 -------- */
 
-TVChRegs ch_pal_large_b = { /* Mode 19: 800x600 1:1  */
+TVChRegs ch1_pal_huge_b = { /* Mode 19: 800x600 1:1  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -396,9 +419,9 @@ TVChRegs ch_pal_large_b = { /* Mode 19: 800x600 1:1  */
   mode    : 19,
 };
 
-unsigned long ch_fsci_pal_large_b [] = {645499916, 521519134, 645527100};
+fsci_t ch1_fsci_pal_huge_b [] = {645499916, 521519134, 645527100};
 
-TVChRegs ch_pal_small_b = { /* Mode 20: 800x600 5:6  */
+TVChRegs ch1_pal_small_b = { /* Mode 20: 800x600 5:6  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 3,
@@ -412,9 +435,9 @@ TVChRegs ch_pal_small_b = { /* Mode 20: 800x600 5:6  */
   mode    : 20,
 };
 
-unsigned long ch_fsci_pal_small_b [] = {528951320, 427355957, 529002000};
+fsci_t ch1_fsci_pal_small_b [] = {528951320, 427355957, 529002000};
 
-TVChRegs ch_pal_tiny_b = { /* Mode 21: 800x600 3:4  */
+TVChRegs ch1_pal_tiny_b = { /* Mode 21: 800x600 3:4  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 4,
@@ -428,13 +451,13 @@ TVChRegs ch_pal_tiny_b = { /* Mode 21: 800x600 3:4  */
   mode    : 21,
 };
 
-unsigned long ch_fsci_pal_tiny_b [] = {488262757, 394482422, 488330000};
+fsci_t ch1_fsci_pal_tiny_b [] = {488262757, 394482422, 488330000};
 
 /* Note: Mode 21 has 627 PAL lines instead of 625 */
 
 /* -------- 640x400 -------- */
 
-TVChRegs ch_pal_small_c = { /* Mode 8: 640x400 5:4  */
+TVChRegs ch1_pal_small_c = { /* Mode 8: 640x400 5:4  */
   dmr_ir  : 2,
   dmr_vs  : 0,
   dmr_sr  : 0,
@@ -448,9 +471,9 @@ TVChRegs ch_pal_small_c = { /* Mode 8: 640x400 5:4  */
   mode    : 8,
 };
 
-unsigned long ch_fsci_pal_small_c [] = {677057690, 547015625, 677157690};
+fsci_t ch1_fsci_pal_small_c [] = {677057690, 547015625, 677157690};
 
-TVChRegs ch_pal_tiny_c = { /* Mode 9: 640x400 1:1  */
+TVChRegs ch1_pal_tiny_c = { /* Mode 9: 640x400 1:1  */
   dmr_ir  : 2,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -464,11 +487,11 @@ TVChRegs ch_pal_tiny_c = { /* Mode 9: 640x400 1:1  */
   mode    : 9,
 };
 
-unsigned long ch_fsci_pal_tiny_c [] = {537347373, 434139385, 537447373};
+fsci_t ch1_fsci_pal_tiny_c [] = {537347373, 434139385, 537447373};
 
 /* -------- 720x400 -------- */
 
-TVChRegs ch_pal_small_d = { /* Mode 4: 720x400 5:4  */
+TVChRegs ch1_pal_small_d = { /* Mode 4: 720x400 5:4  */
   dmr_ir  : 1,
   dmr_vs  : 0,
   dmr_sr  : 0,
@@ -482,9 +505,9 @@ TVChRegs ch_pal_small_d = { /* Mode 4: 720x400 5:4  */
   mode    : 4,
 };
 
-unsigned long ch_fsci_pal_small_d [] = {601829058, 486236111, 601879058};
+fsci_t ch1_fsci_pal_small_d [] = {601829058, 486236111, 601879058};
 
-TVChRegs ch_pal_tiny_d = { /* Mode 5: 720x400 1:1  */
+TVChRegs ch1_pal_tiny_d = { /* Mode 5: 720x400 1:1  */
   dmr_ir  : 1,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -498,11 +521,11 @@ TVChRegs ch_pal_tiny_d = { /* Mode 5: 720x400 1:1  */
   mode    : 5,
 };
 
-unsigned long ch_fsci_pal_tiny_d [] = {485346014, 392125896, 485446014};
+fsci_t ch1_fsci_pal_tiny_d [] = {485346014, 392125896, 485446014};
 
 /* -------- 512x384 -------- */
 
-TVChRegs ch_pal_small_e = { /* Mode 0: 512x384 5:4  */
+TVChRegs ch1_pal_small_e = { /* Mode 0: 512x384 5:4  */
   dmr_ir  : 0,
   dmr_vs  : 0,
   dmr_sr  : 0,
@@ -516,9 +539,9 @@ TVChRegs ch_pal_small_e = { /* Mode 0: 512x384 5:4  */
   mode    : 0,
 };
 
-unsigned long ch_fsci_pal_small_e [] = {806021060, 651209077, 806121060};
+fsci_t ch1_fsci_pal_small_e [] = {806021060, 651209077, 806121060};
 
-TVChRegs ch_pal_tiny_e = { /* Mode 1: 512x384 1:1  */
+TVChRegs ch1_pal_tiny_e = { /* Mode 1: 512x384 1:1  */
   dmr_ir  : 0,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -532,7 +555,7 @@ TVChRegs ch_pal_tiny_e = { /* Mode 1: 512x384 1:1  */
   mode    : 1,
 };
 
-unsigned long ch_fsci_pal_tiny_e [] = {644816848, 520967262, 644916848};
+fsci_t ch1_fsci_pal_tiny_e [] = {644816848, 520967262, 644916848};
 
 /* -------- DVD convenience modes -------- */
 
@@ -543,7 +566,7 @@ unsigned long ch_fsci_pal_tiny_e [] = {644816848, 520967262, 644916848};
    the large mode is buggy (color flickers).
 */
 
-TVChRegs ch_pal_large_f = { /* Mode 19: 800x600 1:1  */
+TVChRegs ch1_pal_large_f = { /* Mode 19: 800x600 1:1  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -557,7 +580,7 @@ TVChRegs ch_pal_large_f = { /* Mode 19: 800x600 1:1  */
   mode    : 19,
 };
 
-TVChRegs ch_pal_large_g = { /* Mode 19: 800x600 1:1  */
+TVChRegs ch1_pal_large_g = { /* Mode 19: 800x600 1:1  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -571,9 +594,9 @@ TVChRegs ch_pal_large_g = { /* Mode 19: 800x600 1:1  */
   mode    : 19,
 };
 
-/* Use ch_fsci_pal_large_b */
+/* Use ch1_fsci_pal_large_b */
 
-TVChRegs ch_pal_small_f = { /* Mode 20: 800x600 5:6  */
+TVChRegs ch1_pal_small_f = { /* Mode 20: 800x600 5:6  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 3,
@@ -587,7 +610,7 @@ TVChRegs ch_pal_small_f = { /* Mode 20: 800x600 5:6  */
   mode    : 20,
 };
 
-TVChRegs ch_pal_small_g = { /* Mode 20: 800x600 5:6  */
+TVChRegs ch1_pal_small_g = { /* Mode 20: 800x600 5:6  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 3,
@@ -601,11 +624,11 @@ TVChRegs ch_pal_small_g = { /* Mode 20: 800x600 5:6  */
   mode    : 20,
 };
 
-/* Use ch_fsci_pal_small_b */
+/* Use ch1_fsci_pal_small_b */
 
 /* -------- 720x{576,480} DVD -------- */
 
-TVChRegs ch_pal_interl_dvd = { /* Mode 25: 720x576 IL  */
+TVChRegs ch1_pal_interl_dvd = { /* Mode 25: 720x576 IL  */
   dmr_ir  : 5,
   dmr_vs  : 0,
   dmr_sr  : 1,
@@ -623,6 +646,317 @@ TVChRegs ch_pal_interl_dvd = { /* Mode 25: 720x576 IL  */
 /* Note: Mode 25 must use 2X XClk. So need to program that. For
    CRT, need to use double hscan, and interlace. */
 
+/* -------- CH2 -------- NTSC -------- */
+
+/* Need Modes 19-38; the rest is identical to CH1 modes  */
+
+/* Are modes 37/38 interlaced */
+
+#if 0
+
+TVChRegs ch2_ntsc_huge_c  = { /* Mode 19:  720x480 /   882x525  1:1 oc   0 */
+  dmr_ir  : 4,
+  dmr_sr  : 1,
+  pll_n   : 124,
+  pll_m   : 63,
+  pllcap  : 0,
+  mode    : 19  
+};
+
+TVChRegs ch2_ntsc_small_c = { /* Mode 20:  720x480 /   882x600  7:8 oc -13 */
+  dmr_ir  : 4,
+  dmr_sr  : 2,
+  pll_n   : 142,
+  pll_m   : 63,
+  pllcap  : 0,
+  mode    : 20  
+};
+
+TVChRegs ch2_ntsc_tiny_c  = { /* Mode 21:  720x480 /   900x630  5:6 oc -18 */
+  dmr_ir  : 4,
+  dmr_sr  : 3,
+  pll_n   : 214,
+  pll_m   : 89,
+  pllcap  : 0,
+  mode    : 21  
+};
+
+TVChRegs ch2_ntsc_large_b = { /* Mode 28:  800x600 /  1040x700  3:4 oc  -6 */
+  dmr_ir  : 6,
+  dmr_sr  : 6,
+  pll_n   : 62,
+  pll_m   : 19,
+  pllcap  : 1,
+  mode    : 28  
+};
+
+TVChRegs ch2_ntsc_small_b = { /* Mode 29:  800x600 /  1064x750  7:10 oc -14 */
+  dmr_ir  : 6,
+  dmr_sr  : 7,
+  pll_n   : 302,
+  pll_m   : 89,
+  pllcap  : 0,
+  mode    : 29  
+};
+
+TVChRegs ch2_ntsc_tiny_b  = { /* Mode 30:  800x600 /  1040x840  5:8 oc -22 */
+  dmr_ir  : 6,
+  dmr_sr  : 5,
+  pll_n   : 126,
+  pll_m   : 33,
+  pllcap  : 1,
+  mode    : 30  
+};
+
+TVChRegs ch2_ntsc_huge_a  = { /* Mode 34: 1024x768 /  1160x840  5:8 oc   0 */
+  dmr_ir  : 7,
+  dmr_sr  : 5,
+  pll_n   : 565,
+  pll_m   : 137,
+  pllcap  : 0,
+  mode    : 34  
+};
+
+TVChRegs ch2_ntsc_large_a = { /* Mode 35: 1024x768 /  1160x945  5:9 oc -10 */
+  dmr_ir  : 7,
+  dmr_sr  : 6,
+  pll_n   : 333,
+  pll_m   : 71,
+  pllcap  : 0,
+  mode    : 35  
+};
+
+TVChRegs ch2_ntsc_tiny_a  = { /* Mode 36: 1024x768 / 1168x1050  1:2 oc -20 */
+  dmr_ir  : 7,
+  dmr_sr  : 7,
+  pll_n   : 917,
+  pll_m   : 177,
+  pllcap  : 0,
+  mode    : 36  
+};
+
+TVChRegs ch2_ntsc_dvd     = { /* Mode 38:  720x480 /   858x525  1:1 oc   0 */
+  dmr_ir  : 4,
+  dmr_sr  : 0,
+  pll_n   : 31,
+  pll_m   : 33,
+  pllcap  : 1,
+  mode    : 38  
+};
+
+
+TVChRegs ch2_pal_huge_d   = { /* Mode 22:  720x576 /   882x625  1:1 oc   0 */
+  dmr_ir  : 5,
+  dmr_sr  : 1,
+  pll_n   : 75,
+  pll_m   : 38,
+  pllcap  : 1,
+  mode    : 22  
+};
+
+TVChRegs ch2_pal_small_d  = { /* Mode 23:  720x576 /   900x750  5:6 oc -18 */
+  dmr_ir  : 5,
+  dmr_sr  : 3,
+  pll_n   : 31,
+  pll_m   : 12,
+  pllcap  : 1,
+  mode    : 23  
+};
+
+TVChRegs ch2_pal_tiny_d   = { /* Mode 24:  720x576 /   900x875  5:7 oc -30 */ 
+  dmr_ir  : 5,
+  dmr_sr  : 4,
+  pll_n   : 9,
+  pll_m   : 2,
+  pllcap  : 1,
+  mode    : 24  
+};
+
+TVChRegs ch2_pal_huge_b   = { /* Mode 25:  800x600 /   944x625  1:1 oc  +4 */
+  dmr_ir  : 6,
+  dmr_sr  : 1,
+  pll_n   : 647,
+  pll_m   : 313,
+  pllcap  : 0,
+  mode    : 25  
+};
+
+TVChRegs ch2_pal_small_b  = { /* Mode 26:  800x600 /   960x750  5:6 oc -14 */
+  dmr_ir  : 6,
+  dmr_sr  : 3,
+  pll_n   : 86,
+  pll_m   : 33,
+  pllcap  : 1,
+  mode    : 26  
+};
+
+TVChRegs ch2_pal_tiny_b   = { /* Mode 27:  800x600 /   960x875  5:7 oc -27 */
+  dmr_ir  : 6,
+  dmr_sr  : 4,
+  pll_n   : 42,
+  pll_m   : 13,
+  pllcap  : 1,
+  mode    : 27  
+};
+
+TVChRegs ch2_pal_huge_a   = { /* Mode 31: 1024x768 /  1400x875  5:7 oc  -4 */
+  dmr_ir  : 7,
+  dmr_sr  : 4,
+  pll_n   : 75,
+  pll_m   : 16,
+  pllcap  : 1,
+  mode    : 31  
+};
+
+TVChRegs ch2_pal_small_a  = { /* Mode 32: 1024x768 / 1400x1000  5:8 oc -16 */
+  dmr_ir  : 7,
+  dmr_sr  : 5,
+  pll_n   : 42,
+  pll_m   : 7,
+  pllcap  : 1,
+  mode    : 32  
+};
+
+TVChRegs ch2_pal_tiny_a   = { /* Mode 33: 1024x768 / 1400x1125  5:9 oc -25 */
+  dmr_ir  : 7,
+  dmr_sr  : 6,
+  pll_n   : 20,
+  pll_m   : 2,
+  pllcap  : 1,
+  mode    : 33  
+};
+
+TVChRegs ch2_pal_dvd      = { /* Mode 37:  720x576 /   864x625  1:1 oc   0 */
+  dmr_ir  : 5,
+  dmr_sr  : 0,
+  pll_n   : 31,
+  pll_m   : 33,
+  pllcap  : 1,
+  mode    : 37  
+};
+
+fsci_t ch2_fsci_ntsc_huge_c  [] = {553914433, 553916752, 553305736}; /* 19 */
+fsci_t ch2_fsci_ntsc_small_c [] = {484675129, 484677158, 484142519}; /* 20 */
+fsci_t ch2_fsci_ntsc_tiny_c  [] = {452363454, 452365347, 451866351}; /* 21 */
+fsci_t ch2_fsci_ntsc_large_b [] = {469762048, 469764015, 469245826}; /* 28 */
+fsci_t ch2_fsci_ntsc_small_b [] = {428554851, 428556645, 428083911}; /* 29 */
+fsci_t ch2_fsci_ntsc_tiny_b  [] = {391468373, 391470012, 391038188}; /* 30 */
+fsci_t ch2_fsci_ntsc_huge_a  [] = {526457468, 526459671, 525878943}; /* 34 */
+fsci_t ch2_fsci_ntsc_large_a [] = {467962193, 467964152, 467447949}; /* 35 */
+fsci_t ch2_fsci_ntsc_tiny_a  [] = {418281276, 418283027, 417821626}; /* 36 */
+fsci_t ch2_fsci_ntsc_dvd     [] = {569408543, 569410927, 568782819}; /* 38 */
+								           
+fsci_t ch2_fsci_pal_huge_d   [] = {690875194, 558179209, 690875194}; /* 22 */
+fsci_t ch2_fsci_pal_small_d  [] = {564214742, 455846354, 564214742}; /* 23 */
+fsci_t ch2_fsci_pal_tiny_d   [] = {483612636, 390725446, 483612636}; /* 24 */
+fsci_t ch2_fsci_pal_huge_b   [] = {645499916, 521519134, 645499916}; /* 25 */
+fsci_t ch2_fsci_pal_small_b  [] = {528951320, 427355957, 528951320}; /* 26 */
+fsci_t ch2_fsci_pal_tiny_b   [] = {453386846, 366305106, 453386846}; /* 27 */
+fsci_t ch2_fsci_pal_huge_a   [] = {621787675, 502361288, 621787675}; /* 31 */
+fsci_t ch2_fsci_pal_small_a  [] = {544064215, 439566127, 544064215}; /* 32 */
+fsci_t ch2_fsci_pal_tiny_a   [] = {483612636, 390725446, 483612636}; /* 33 */
+fsci_t ch2_fsci_pal_dvd      [] = {705268427, 569807942, 705268427}; /* 37 */
+
+#else
+
+/* -------- 800x600 -------- */
+
+/* For I845 */
+
+TVChRegs ch2_ntsc_small_b = { /* Mode 29: 800x600 / 1064x750 7:10 */
+  dmr_ir    : 6,
+  dmr_vs    : 1,
+  dmr_sr    : 7,
+  ffr_fc    : 2,
+  ffr_fy    : 1,
+  ffr_ft    : 3,
+  vbw_flff  : 0,
+  vbw_cbw   : 3,
+  vbw_ypeak : 0,
+  vbw_ysv   : 3,
+  vbw_ycv   : 2,
+  te        : 5,
+  sav       : 132,
+  hpr       : 58,
+  vpr       : 14,
+  blr       : 131,  /* i865: 129 */
+  ce        : 3,
+  pllcap    : 0,
+  pll_n     : 302,
+  pll_m     : 89,
+  civh      : 0,
+  dacg      : 0,
+  mode      : 29,
+};
+
+/* Use ch1_fsci_ntsc_small_b */
+
+/* -------- CH2 -------- PAL -------- */
+
+/* For I845 */
+
+TVChRegs ch2_pal_tiny_b = { /* Mode 27: 800x600 / 960x875 5:7 */
+  dmr_ir    : 6,
+  dmr_vs    : 0,
+  dmr_sr    : 4,
+  ffr_fc    : 2,
+  ffr_fy    : 1,
+  ffr_ft    : 3,
+  vbw_flff  : 0,
+  vbw_cbw   : 3,
+  vbw_ypeak : 0,
+  vbw_ysv   : 3,
+  vbw_ycv   : 2,
+  te        : 5,
+  sav       : 132,
+  hpr       : 90,
+  vpr       : 32,
+  blr       : 110,
+  ce        : 3,
+  pllcap    : 1,
+  pll_n     : 42,
+  pll_m     : 13,
+  civh      : 0,
+  dacg      : 0,
+  mode      : 27,
+};
+
+/* Different from ch1 with 3:4 instead of 5:7 */
+
+unsigned long ch2_fsci_pal_tiny_b [] = {453386846, 366305106, 453386846};
+
+/* For i830 */
+
+TVChRegs ch2_pal_small_a = { /* Mode 32: 1024x768 / 1400x1000  5:8 */
+  dmr_ir    : 7,
+  dmr_vs    : 0,
+  dmr_sr    : 5,
+  ffr_fc    : 2,
+  ffr_fy    : 1,
+  ffr_ft    : 3,
+  vbw_flff  : 0,
+  vbw_cbw   : 3,
+  vbw_ypeak : 0,
+  vbw_ysv   : 2,
+  vbw_ycv   : 3,
+  te        : 7,
+  sav       : 190,
+  hpr       : 60,
+  vpr       : 304,
+  blr       : 109,
+  ce        : 4,
+  pllcap    : 1,
+  pll_n     : 42,
+  pll_m     : 7,
+  civh      : 0,
+  dacg      : 1,
+  mode      : 32,
+};
+
+unsigned long ch2_fsci_pal_small_a [] = {544064215, 439566127, 544064215};
+
+#endif
+
 /* -------- -------- */
 
 static int clamp_contrast[8]  = {-75, -50, -25, 0, 25, 50, 75, 100};
@@ -631,7 +965,8 @@ static int clamp_sharpness[3] = {0, 50, 100};
 static int clamp_luma_bandwidth[5]   = {0, 25, 50, 75, 100};
 static int clamp_chroma_bandwidth[4] = {0, 50, 75, 100};
 
-void data_init_ch (TVSystem system, TVChRegs *r) 
+void 
+data_init_ch (TVSystem system, TVChRegs *r) 
 {
   if (!r) return;
   r->ffr_fc = 2;
@@ -644,6 +979,13 @@ void data_init_ch (TVSystem system, TVChRegs *r)
   r->vbw_ycv = 0;   /* low bandwidth */
   r->civh = 0;      /* civ hysteresis mode = 0 */
   r->ce = 3;        /* normal contrast */
+  r->flags = CH_FLAG_BOTH | CH_FLAG_CFRB; /* no ACIV, no CVBW */
+}
+
+void 
+data_init_ch1 (TVSystem system, TVChRegs *r) 
+{
+  data_init_ch (system, r);
   switch (system) {
     case TV_SYSTEM_NTSC: /* additional fsci values for NDC (no dot crawl) */
       r->dmr_vs = 1;
@@ -677,10 +1019,49 @@ void data_init_ch (TVSystem system, TVChRegs *r)
       r->blr = 105;
       break;
   }
-  r->flags = CH_FLAG_BOTH | CH_FLAG_CFRB; /* no ACIV, no CVBW */
 }
 
-void data_default_ch (TVSettings *s)
+void 
+data_init_ch2 (TVSystem system, TVChRegs *r) 
+{
+  data_init_ch (system, r);
+  switch (system) {
+    case TV_SYSTEM_NTSC: /* additional fsci values for NDC (no dot crawl) */
+      r->dmr_vs = 1;
+      r->dacg = 0;
+      r->blr = 131;
+      break;
+    case TV_SYSTEM_NTSC_J:
+      r->dmr_vs = 3;
+      r->dacg = 1;
+      r->blr = 102;
+      break;
+    case TV_SYSTEM_NONE:  /* to have a default */
+    case TV_SYSTEM_SECAM: /* makes most sense */
+    case TV_SYSTEM_PAL:
+    case TV_SYSTEM_PAL_X:
+    case TV_SYSTEM_PAL_N:  /* PAL-N has different fsci values */
+    case TV_SYSTEM_PAL_NC: /* guess */
+      r->dmr_vs = 0;
+      r->dacg = 1;
+      r->blr = 110;
+      break;
+    case TV_SYSTEM_PAL_M:   /* needs different fsci values */
+    case TV_SYSTEM_PAL_M60: /* guess */
+      r->dmr_vs = 2;
+      r->dacg = 0;
+      r->blr = 131;
+      break;
+    case TV_SYSTEM_PAL_60: /* modify PAL_M */
+      r->dmr_vs = 2;
+      r->dacg = 1;
+      r->blr = 110;
+      break;
+  }
+}
+
+void 
+data_default_ch (TVSettings *s)
 {
   s->tv_hoffset = s->mon_hoffset = 0;
   s->tv_voffset = s->mon_voffset = 0;
@@ -700,7 +1081,8 @@ void data_default_ch (TVSettings *s)
   /* FIXME flags */
 }
 
-void data_clamp_ch (TVSettings *s, TVRegs *r)
+void 
+data_clamp_ch (TVSettings *s, TVRegs *r)
 {
   if (s->tv_hoffset <= -100) s->tv_hoffset = -100;
   if (s->tv_hoffset >=  100) s->tv_hoffset =  100;
@@ -729,7 +1111,8 @@ void data_clamp_ch (TVSettings *s, TVRegs *r)
   /* FIXME: Operate on mode flags */
 }
 
-void data_setup_ch (TVSettings *s, TVRegs *r)
+void 
+data_setup_ch1 (TVSettings *s, TVRegs *r)
 {
   register int i;
   int vmax;
@@ -808,6 +1191,11 @@ void data_setup_ch (TVSettings *s, TVRegs *r)
   }
 }
 
+void 
+data_setup_ch2 (TVSettings *s, TVRegs *r)
+{
+}
+
 /* 
 
 4:3 * 4:3 -> 16:9
@@ -868,16 +1256,8 @@ Is not linear...
 
 Think arrangement over.
 
-Mode -> dmr_ir, dmr_sr, dmr_vs (lsb), pll_m, pll_n, pllcap
-
-System -> dmr_vs (msb), dacg, (either of two modesets)
-
-NTSC,NTSC-NDC,PAL-M  (NTSC-J ?)
-PAL,PAL-N
-
-crt values -> sav. hpr, vpr (are those different for nv/i810 ??)
+Mode -> dmr_ir, dmr_sr, pll_m, pll_n, pllcap, mode
+System -> dmr_vs (msb), blr, dacg (in init_ch1/2)
+crt values -> sav, hpr, vpr (those are different for nv/i810 ??)
 
 */
-
-
-
