@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: data_i810.h,v 1.4 2003/10/15 10:57:58 dthierbach Exp $
  *
  * Contents:
  *
@@ -32,10 +32,9 @@
 
 void data_init_i810 (TVI810Regs *r, int portHost);
 
-TVMode *data_modes_i810_ch (void);
-
 extern DataCardFunc data_i810_func;
 
 extern DataFunc data_i810_ch1_func;
+extern DataFunc data_i810_ch2_func;
 
 #endif /* _DATA_I810_H */
