@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: data_nv.c,v 1.23 2004/01/30 08:47:02 dthierbach Exp $
  *
  * Contents:
  *
@@ -34,6 +34,7 @@
 #include "data_cx.h"
 #include "data_ch.h"
 #include "data_ph.h"
+#include "data_nx.h"
 #include "data_nv.h"
 #include "calc_bt.h"
 
@@ -674,7 +675,8 @@ static TVMode *modes_nv_bt   = NULL;
  * count number of modes matching system; TV_SYSTEM_NONE matches all.
  */
 
-int data_count_nv_bt (TVSystem system)
+int 
+data_count_nv_bt (TVSystem system)
 {
   int count;
   TVTemplateNvBt *t;
@@ -686,7 +688,8 @@ int data_count_nv_bt (TVSystem system)
   return count;
 }
 
-static TVMode *loop_create_nv_bt (TVMode *m, TVTemplateNvBt *t, 
+static TVMode*
+loop_create_nv_bt (TVMode *m, TVTemplateNvBt *t, 
   TVSystem convert_from, TVSystem convert_to)
 {
   for (; t->spec.system != TV_SYSTEM_NONE; t++) {
@@ -724,7 +727,8 @@ static TVMode *loop_create_nv_bt (TVMode *m, TVTemplateNvBt *t,
   return m;
 }
 
-TVMode *data_modes_nv_bt (void)
+TVMode*
+data_modes_nv_bt (void)
 {
   int c;
   TVMode *m;
@@ -909,13 +913,15 @@ TVTemplateNvCx templ_nv_cx [] = {
 
 /* -------- -------- */
 
-static TVMode *modes_nv_cx = NULL;
+static TVMode*
+modes_nv_cx = NULL;
 
 /*
  * count number of modes matching system; TV_SYSTEM_NONE matches all.
  */
 
-int data_count_nv_cx (TVTemplateNvCx *t, TVSystem system)
+int 
+data_count_nv_cx (TVTemplateNvCx *t, TVSystem system)
 {
   int count;
 
@@ -930,7 +936,8 @@ int data_count_nv_cx (TVTemplateNvCx *t, TVSystem system)
  * Create CX modes from both BT and CX templates.
  */
 
-static TVMode *loop_create_nv_btcx (TVMode *m, TVTemplateNvBt *t, 
+static TVMode*
+loop_create_nv_btcx (TVMode *m, TVTemplateNvBt *t, 
   TVSystem convert_from, TVSystem convert_to)
 {
   for (; t->spec.system != TV_SYSTEM_NONE; t++) {
@@ -967,7 +974,8 @@ static TVMode *loop_create_nv_btcx (TVMode *m, TVTemplateNvBt *t,
   return m;
 }
 
-static TVMode *loop_create_nv_cx (TVMode *m, TVTemplateNvCx *t, 
+static TVMode*
+loop_create_nv_cx (TVMode *m, TVTemplateNvCx *t, 
   TVSystem convert_from, TVSystem convert_to)
 {
   for (; t->spec.system != TV_SYSTEM_NONE; t++) {
@@ -1002,7 +1010,8 @@ static TVMode *loop_create_nv_cx (TVMode *m, TVTemplateNvCx *t,
   return m;
 }
 
-TVMode *data_modes_nv_cx (void)
+TVMode 
+*data_modes_nv_cx (void)
 {
   int c;
   TVMode *m;
@@ -1036,11 +1045,11 @@ TVMode *data_modes_nv_cx (void)
 
 #define TV_DEF_CH (TV_CAP_MONOCHROME | TV_CAP_COLORFIX)
 
-/* -------- CH -------- NTSC -------- */
+/* -------- CH1 -------- NTSC -------- */
 
 /* -------- 640x480 -------- */
 
-TVNvRegs nv_ch_ntsc_large_a = { 
+TVNvRegs nv_ch1_ntsc_huge_a = { 
   HDisplay    : 640,
   HSyncStart  : 664-8,
   HSyncEnd    : 736-8,
@@ -1052,7 +1061,7 @@ TVNvRegs nv_ch_ntsc_large_a = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_small_a = { 
+TVNvRegs nv_ch1_ntsc_small_a = { 
   HDisplay    : 640,
   HSyncStart  : 664-8,
   HSyncEnd    : 736-8,
@@ -1064,7 +1073,7 @@ TVNvRegs nv_ch_ntsc_small_a = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_tiny_a = { 
+TVNvRegs nv_ch1_ntsc_tiny_a = { 
   HDisplay    : 640,
   HSyncStart  : 672-8,
   HSyncEnd    : 744-8,
@@ -1078,7 +1087,7 @@ TVNvRegs nv_ch_ntsc_tiny_a = {
 
 /* -------- 800x600 -------- */
 
-TVNvRegs nv_ch_ntsc_huge_b = { 
+TVNvRegs nv_ch1_ntsc_huge_b = { 
   HDisplay    : 800,
   HSyncStart  : 856-8,
   HSyncEnd    : 920-8,
@@ -1090,7 +1099,7 @@ TVNvRegs nv_ch_ntsc_huge_b = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_large_b = { 
+TVNvRegs nv_ch1_ntsc_large_b = { 
   HDisplay    : 800,
   HSyncStart  : 880-8,
   HSyncEnd    : 968-8,
@@ -1102,7 +1111,7 @@ TVNvRegs nv_ch_ntsc_large_b = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_small_b = { 
+TVNvRegs nv_ch1_ntsc_small_b = { 
   HDisplay    : 800,
   HSyncStart  : 896-8,
   HSyncEnd    : 960-8,
@@ -1116,7 +1125,7 @@ TVNvRegs nv_ch_ntsc_small_b = {
 
 /* -------- 640x400 -------- */
 
-TVNvRegs nv_ch_ntsc_large_c = { /* FIXME: Doesn't work */
+TVNvRegs nv_ch1_ntsc_huge_c = { /* FIXME: Doesn't work */
   HDisplay    : 640,
   HSyncStart  : 704-8,
   HSyncEnd    : 768-8,
@@ -1128,7 +1137,7 @@ TVNvRegs nv_ch_ntsc_large_c = { /* FIXME: Doesn't work */
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_small_c = { 
+TVNvRegs nv_ch1_ntsc_small_c = { 
   HDisplay    : 640,
   HSyncStart  : 696-8,
   HSyncEnd    : 760-8,
@@ -1140,7 +1149,7 @@ TVNvRegs nv_ch_ntsc_small_c = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_tiny_c = { 
+TVNvRegs nv_ch1_ntsc_tiny_c = { 
   HDisplay    : 640,
   HSyncStart  : 696-8,
   HSyncEnd    : 768-8,
@@ -1154,7 +1163,7 @@ TVNvRegs nv_ch_ntsc_tiny_c = {
 
 /* -------- 720x400 -------- */
 
-TVNvRegs nv_ch_ntsc_large_d = { 
+TVNvRegs nv_ch1_ntsc_huge_d = { 
   HDisplay    : 720,
   HSyncStart  : 784-8,
   HSyncEnd    : 864-8,
@@ -1166,7 +1175,7 @@ TVNvRegs nv_ch_ntsc_large_d = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_small_d = { 
+TVNvRegs nv_ch1_ntsc_small_d = { 
   HDisplay    : 720,
   HSyncStart  : 784-8,
   HSyncEnd    : 864-8,
@@ -1180,7 +1189,7 @@ TVNvRegs nv_ch_ntsc_small_d = {
 
 /* -------- 512x384 -------- */
 
-TVNvRegs nv_ch_ntsc_large_e = { 
+TVNvRegs nv_ch1_ntsc_large_e = { 
   HDisplay    : 512,
   HSyncStart  : 616-8,
   HSyncEnd    : 688-8,
@@ -1192,7 +1201,7 @@ TVNvRegs nv_ch_ntsc_large_e = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_small_e = { 
+TVNvRegs nv_ch1_ntsc_small_e = { 
   HDisplay    : 512,
   HSyncStart  : 608-8,
   HSyncEnd    : 672-8,
@@ -1206,7 +1215,7 @@ TVNvRegs nv_ch_ntsc_small_e = {
 
 /* -------- DVD convenience modes -------- */
 
-TVNvRegs nv_ch_ntsc_large_f = { /* test */
+TVNvRegs nv_ch1_ntsc_large_f = { /* test */
   HDisplay    : 768,
   HSyncStart  : 856-8,
   HSyncEnd    : 952-8,
@@ -1218,7 +1227,7 @@ TVNvRegs nv_ch_ntsc_large_f = { /* test */
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_ntsc_large_g = { /* test */
+TVNvRegs nv_ch1_ntsc_large_g = { /* test */
   HDisplay    : 800,
   HSyncStart  : 880-8,
   HSyncEnd    : 968-8,
@@ -1234,7 +1243,7 @@ TVNvRegs nv_ch_ntsc_large_g = { /* test */
 
 /* Note: Not used yet, crt values not tested */
 
-TVNvRegs nv_ch_ntsc_interl_dvd = { 
+TVNvRegs nv_ch1_ntsc_interl_dvd = { 
   HDisplay    : 720,
   HSyncStart  : 752-8,
   HSyncEnd    : 768-8,
@@ -1252,7 +1261,7 @@ TVNvRegs nv_ch_ntsc_interl_dvd = {
 
 /* FIXME: crt_pal_large_a -- no dualview, not in tvcc either */
 
-TVNvRegs nv_ch_pal_large_a = { 
+TVNvRegs nv_ch1_pal_huge_a = { 
   HDisplay    : 640,
   HSyncStart  : 688-8,
   HSyncEnd    : 760-8,
@@ -1264,7 +1273,7 @@ TVNvRegs nv_ch_pal_large_a = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_small_a = { 
+TVNvRegs nv_ch1_pal_small_a = { 
   HDisplay    : 640,
   HSyncStart  : 688-8,
   HSyncEnd    : 760-8,
@@ -1276,7 +1285,7 @@ TVNvRegs nv_ch_pal_small_a = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_tiny_a = { 
+TVNvRegs nv_ch1_pal_tiny_a = { 
   HDisplay    : 640,
   HSyncStart  : 688-8,
   HSyncEnd    : 760-8,
@@ -1290,9 +1299,9 @@ TVNvRegs nv_ch_pal_tiny_a = {
 
 /* -------- 800x600 -------- */
 
-/* FIXME: large has significant distortion on right hand side */
+/* FIXME: huge has significant distortion on right hand side */
 
-TVNvRegs nv_ch_pal_large_b = { 
+TVNvRegs nv_ch1_pal_huge_b = { 
   HDisplay    : 800,
   HSyncStart  : 808-8,
   HSyncEnd    : 872-8,
@@ -1304,7 +1313,7 @@ TVNvRegs nv_ch_pal_large_b = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_small_b = { 
+TVNvRegs nv_ch1_pal_small_b = { 
   HDisplay    : 800,
   HSyncStart  : 824-8,
   HSyncEnd    : 888-8,
@@ -1316,7 +1325,7 @@ TVNvRegs nv_ch_pal_small_b = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_tiny_b = { 
+TVNvRegs nv_ch1_pal_tiny_b = { 
   HDisplay    : 800,
   HSyncStart  : 840-8,
   HSyncEnd    : 904-8,
@@ -1332,7 +1341,7 @@ TVNvRegs nv_ch_pal_tiny_b = {
 
 /* -------- 640x400 -------- */
 
-TVNvRegs nv_ch_pal_small_c = { 
+TVNvRegs nv_ch1_pal_small_c = { 
   HDisplay    : 640,
   HSyncStart  : 824-8,
   HSyncEnd    : 888-8,
@@ -1344,7 +1353,7 @@ TVNvRegs nv_ch_pal_small_c = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_tiny_c = { 
+TVNvRegs nv_ch1_pal_tiny_c = { 
   HDisplay    : 640,
   HSyncStart  : 760-8,
   HSyncEnd    : 824-8,
@@ -1358,7 +1367,7 @@ TVNvRegs nv_ch_pal_tiny_c = {
 
 /* -------- 720x400 -------- */
 
-TVNvRegs nv_ch_pal_small_d = { 
+TVNvRegs nv_ch1_pal_small_d = { 
   HDisplay    : 720,
   HSyncStart  : 824-8,
   HSyncEnd    : 888-8,
@@ -1370,7 +1379,7 @@ TVNvRegs nv_ch_pal_small_d = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_tiny_d = { 
+TVNvRegs nv_ch1_pal_tiny_d = { 
   HDisplay    : 720,
   HSyncStart  : 840-8,
   HSyncEnd    : 904-8,
@@ -1384,7 +1393,7 @@ TVNvRegs nv_ch_pal_tiny_d = {
 
 /* -------- 512x384 -------- */
 
-TVNvRegs nv_ch_pal_small_e = { 
+TVNvRegs nv_ch1_pal_small_e = { 
   HDisplay    : 512,
   HSyncStart  : 640-8,
   HSyncEnd    : 704-8,
@@ -1396,7 +1405,7 @@ TVNvRegs nv_ch_pal_small_e = {
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_tiny_e = { 
+TVNvRegs nv_ch1_pal_tiny_e = { 
   HDisplay    : 512,
   HSyncStart  : 624-8,
   HSyncEnd    : 704-8,
@@ -1410,7 +1419,7 @@ TVNvRegs nv_ch_pal_tiny_e = {
 
 /* -------- DVD convenience modes -------- */
 
-TVNvRegs nv_ch_pal_large_f = { /* test FIXME: SyncStart ok ?? */
+TVNvRegs nv_ch1_pal_large_f = { /* test FIXME: SyncStart ok ?? */
   HDisplay    : 768,
   HSyncStart  : 800-8,
   HSyncEnd    : 872-8,
@@ -1422,7 +1431,7 @@ TVNvRegs nv_ch_pal_large_f = { /* test FIXME: SyncStart ok ?? */
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_large_g = { /* test */
+TVNvRegs nv_ch1_pal_large_g = { /* test */
   HDisplay    : 800,
   HSyncStart  : 808-8,
   HSyncEnd    : 872-8,
@@ -1434,7 +1443,7 @@ TVNvRegs nv_ch_pal_large_g = { /* test */
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_small_f = { /* test */
+TVNvRegs nv_ch1_pal_small_f = { /* test */
   HDisplay    : 768,
   HSyncStart  : 808-8,
   HSyncEnd    : 880-8,
@@ -1446,7 +1455,7 @@ TVNvRegs nv_ch_pal_small_f = { /* test */
   flags	      : 0,
 };
 
-TVNvRegs nv_ch_pal_small_g = { /* test */
+TVNvRegs nv_ch1_pal_small_g = { /* test */
   HDisplay    : 800,
   HSyncStart  : 824-8,
   HSyncEnd    : 888-8,
@@ -1460,7 +1469,7 @@ TVNvRegs nv_ch_pal_small_g = { /* test */
 
 /* -------- 720x{576,480} DVD -------- */
 
-TVNvRegs nv_ch_pal_interl_dvd = { 
+TVNvRegs nv_ch1_pal_interl_dvd = { 
   HDisplay    : 720,
   HSyncStart  : 752-8,
   HSyncEnd    : 768-8,
@@ -1472,7 +1481,251 @@ TVNvRegs nv_ch_pal_interl_dvd = {
   flags	      : 0,
 };
 
-/* -------- NV -------- Slave modes -------- */
+/* -------- CH2 -------- NTSC -------- */
+
+TVNvRegs nv_ch2_ntsc_huge_c  = { /* Mode 19 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 882,
+  VDisplay   : 480,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 525,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_small_c = { /* Mode 20 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 882,
+  VDisplay   : 480,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 600,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_tiny_c  = { /* Mode 21 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 900,
+  VDisplay   : 480,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 630,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_large_b = { /* Mode 28 */
+  HDisplay   : 800,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1040,
+  VDisplay   : 600,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 700,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_small_b = { /* Mode 29 */
+  HDisplay   : 800,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1064,
+  VDisplay   : 600,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 750,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_tiny_b  = { /* Mode 30 */
+  HDisplay   : 800,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1040,
+  VDisplay   : 600,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 840,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_huge_a  = { /* Mode 34 */
+  HDisplay   : 1024,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1160,
+  VDisplay   : 768,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 840,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_large_a = { /* Mode 35 */
+  HDisplay   : 1024,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1160,
+  VDisplay   : 768,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 945,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_tiny_a  = { /* Mode 36 */
+  HDisplay   : 1024,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1168,
+  VDisplay   : 768,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 1050,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_ntsc_dvd     = { /* Mode 38 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 858,
+  VDisplay   : 480,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 525,
+  flags	     : 0,
+};
+
+/* -------- CH2 -------- PAL -------- */
+
+TVNvRegs nv_ch2_pal_huge_d   = { /* Mode 22 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 882,
+  VDisplay   : 576,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 625,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_small_d  = { /* Mode 23 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 900,
+  VDisplay   : 576,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 750,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_tiny_d   = { /* Mode 24 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 900,
+  VDisplay   : 576,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 875,
+  flags	     : 0,
+};
+ 
+TVNvRegs nv_ch2_pal_huge_b   = { /* Mode 25 */
+  HDisplay   : 800,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 944,
+  VDisplay   : 600,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 625,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_small_b  = { /* Mode 26 */
+  HDisplay   : 800,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 960,
+  VDisplay   : 600,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 750,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_tiny_b   = { /* Mode 27 */
+  HDisplay   : 800,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 960,
+  VDisplay   : 600,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 875,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_huge_a   = { /* Mode 31 */
+  HDisplay   : 1024,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1400,
+  VDisplay   : 768,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 875,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_small_a  = { /* Mode 32 */
+  HDisplay   : 1024,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1400,
+  VDisplay   : 768,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 1000,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_tiny_a   = { /* Mode 33 */
+  HDisplay   : 1024,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 1400,
+  VDisplay   : 768,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 1125,
+  flags	     : 0,
+};
+
+TVNvRegs nv_ch2_pal_dvd      = { /* Mode 37 */
+  HDisplay   : 720,
+  HSyncStart : 0,
+  HSyncEnd   : 0,
+  HTotal     : 864,
+  VDisplay   : 576,
+  VSyncStart : 0,
+  VSyncEnd   : 0,
+  VTotal     : 625,
+  flags	     : 0,
+};
+
+/* -------- CH1/NV -------- Slave modes -------- */
 
 /*
 NTSC
@@ -1659,7 +1912,7 @@ Mode 24:  800 600   1064 750
 
 */
 
-TVNvRegs nv_ch_ntsc_slave_a = { /* Mode 17 small_a */
+TVNvRegs nv_ch1_ntsc_slave_a = { /* Mode 17 small_a */
   HDisplay    : 640,
   HSyncStart  : 664-8,
   HSyncEnd    : 736-8,
@@ -1679,7 +1932,7 @@ TVNvRegs nv_ch_ntsc_slave_a = { /* Mode 17 small_a */
   }
 };
 
-TVChRegs ch_ntsc_slave_b = { /* Mode 24: 800x600 7:10  */
+TVChRegs ch1_ntsc_slave_b = { /* Mode 24: 800x600 7:10  */
   dmr_ir  : 4,
   dmr_vs  : 1,
   dmr_sr  : 5,
@@ -1693,7 +1946,7 @@ TVChRegs ch_ntsc_slave_b = { /* Mode 24: 800x600 7:10  */
   mode    : 24,
 };
 
-TVNvRegs nv_ch_ntsc_slave_b = { /* Mode 24 small_b */
+TVNvRegs nv_ch1_ntsc_slave_b = { /* Mode 24 small_b */
   HDisplay    : 800,
   HSyncStart  : 864,
   HSyncEnd    : 968,
@@ -1713,7 +1966,7 @@ TVNvRegs nv_ch_ntsc_slave_b = { /* Mode 24 small_b */
   }
 };
 
-TVNvRegs nv_ch_pal_slave_a = { /* Mode 14 small_a */
+TVNvRegs nv_ch1_pal_slave_a = { /* Mode 14 small_a */
   HDisplay    : 640,
   HSyncStart  : 688-8,
   HSyncEnd    : 760-8,
@@ -1733,7 +1986,7 @@ TVNvRegs nv_ch_pal_slave_a = { /* Mode 14 small_a */
   }
 };
 
-TVChRegs ch_pal_slave_b = { /* Mode 20: 800x600 5:6  */
+TVChRegs ch1_pal_slave_b = { /* Mode 20: 800x600 5:6  */
   dmr_ir  : 4,
   dmr_vs  : 0,
   dmr_sr  : 3,
@@ -1746,7 +1999,7 @@ TVChRegs ch_pal_slave_b = { /* Mode 20: 800x600 5:6  */
   mode    : 20,
 };
 
-TVNvRegs nv_ch_pal_slave_b = { /* Mode 20 small_b */
+TVNvRegs nv_ch1_pal_slave_b = { /* Mode 20 small_b */
   HDisplay    : 800,
   HSyncStart  : 840,
   HSyncEnd    : 968,
@@ -1797,70 +2050,70 @@ typedef struct {
    This is now ignored -- the convenience modes get modified as well. 
 */
 
-TVTemplateNvCh templ_nv_ch_50hz [] = { /* 50 Hz: PAL, PAL-N, PAL-X */
+TVTemplateNvCh templ_nv_ch1_50hz [] = { /* 50 Hz: PAL, PAL-N, PAL-X */
 
   {{TV_SYSTEM_PAL, 800, 600, "Tiny",   "4:3", 21.875, 21.875}, /* Mode 21 */
-   &nv_ch_pal_tiny_b,  &ch_pal_tiny_b,  ch_fsci_pal_tiny_b,
+   &nv_ch1_pal_tiny_b,  &ch1_pal_tiny_b,  ch1_fsci_pal_tiny_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},
   {{TV_SYSTEM_PAL, 800, 600, "Small",  "4:3", 13.194, 13.194}, /* Mode 20 */
-   &nv_ch_pal_small_b, &ch_pal_small_b, ch_fsci_pal_small_b,
+   &nv_ch1_pal_small_b, &ch1_pal_small_b, ch1_fsci_pal_small_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
-  {{TV_SYSTEM_PAL, 800, 600, "Large",  "4:3", -4.167, -4.167}, /* Mode 19 */
-   &nv_ch_pal_large_b, &ch_pal_large_b, ch_fsci_pal_large_b,
+  {{TV_SYSTEM_PAL, 800, 600, "Huge",   "4:3", -4.167, -4.167}, /* Mode 19 */
+   &nv_ch1_pal_huge_b, &ch1_pal_huge_b, ch1_fsci_pal_huge_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},  
   {{TV_SYSTEM_PAL, 640, 480, "Tiny",   "4:3", 30.556, 30.556}, /* Mode 15 */
-   &nv_ch_pal_tiny_a,  &ch_pal_tiny_a,  ch_fsci_pal_tiny_a,
+   &nv_ch1_pal_tiny_a,  &ch1_pal_tiny_a,  ch1_fsci_pal_tiny_a,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},    
   {{TV_SYSTEM_PAL, 640, 480, "Small",  "4:3", 16.667, 16.667}, /* Mode 14 */
-   &nv_ch_pal_small_a, &ch_pal_small_a, ch_fsci_pal_small_a,
+   &nv_ch1_pal_small_a, &ch1_pal_small_a, ch1_fsci_pal_small_a,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},  
-  {{TV_SYSTEM_PAL, 640, 480, "Large",  "4:3", -4.167, -4.167}, /* Mode 13 */
-   &nv_ch_pal_large_a, &ch_pal_large_a, ch_fsci_pal_large_a,
+  {{TV_SYSTEM_PAL, 640, 480, "Huge",   "4:3", -4.167, -4.167}, /* Mode 13 */
+   &nv_ch1_pal_huge_a, &ch1_pal_huge_a, ch1_fsci_pal_huge_a,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},  
   {{TV_SYSTEM_PAL, 640, 400, "Tiny",   "8:5", 30.556, 30.556}, /* Mode 9  */
-   &nv_ch_pal_tiny_c,  &ch_pal_tiny_c,  ch_fsci_pal_tiny_c,
+   &nv_ch1_pal_tiny_c,  &ch1_pal_tiny_c,  ch1_fsci_pal_tiny_c,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},    
   {{TV_SYSTEM_PAL, 640, 400, "Small",  "8:5", 13.194, 13.194}, /* Mode 8  */
-   &nv_ch_pal_small_c, &ch_pal_small_c, ch_fsci_pal_small_c,
+   &nv_ch1_pal_small_c, &ch1_pal_small_c, ch1_fsci_pal_small_c,
    PORT_NVIDIA, TV_DEF_CH | TV_CAP_MACROVISION}, /* FIXME: No Dual view yet */
   {{TV_SYSTEM_PAL, 720, 400, "Tiny",   "9:5", 30.556, 30.556}, /* Mode 5  */
-   &nv_ch_pal_tiny_d,  &ch_pal_tiny_d,  ch_fsci_pal_tiny_d,
+   &nv_ch1_pal_tiny_d,  &ch1_pal_tiny_d,  ch1_fsci_pal_tiny_d,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},    
   {{TV_SYSTEM_PAL, 720, 400, "Small",  "9:5", 13.194, 13.194}, /* Mode 4  */
-   &nv_ch_pal_small_d, &ch_pal_small_d, ch_fsci_pal_small_d,
+   &nv_ch1_pal_small_d, &ch1_pal_small_d, ch1_fsci_pal_small_d,
    PORT_NVIDIA, TV_DEF_CH}, /* FIXME: No Dualview yet */
   {{TV_SYSTEM_PAL, 512, 384, "Tiny",   "4:3", 33.333, 33.333}, /* Mode 1  */
-   &nv_ch_pal_tiny_e,  &ch_pal_tiny_e,  ch_fsci_pal_tiny_e,
+   &nv_ch1_pal_tiny_e,  &ch1_pal_tiny_e,  ch1_fsci_pal_tiny_e,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},    
   {{TV_SYSTEM_PAL, 512, 384, "Small",  "4:3", 16.667, 16.667}, /* Mode 0  */
-   &nv_ch_pal_small_e, &ch_pal_small_e, ch_fsci_pal_small_e,
+   &nv_ch1_pal_small_e, &ch1_pal_small_e, ch1_fsci_pal_small_e,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
 
   {{TV_SYSTEM_PAL,  768, 576, "Small",  "4:3", 0.000,  0.000}, /* Mode 20' */
-   &nv_ch_pal_small_f, &ch_pal_small_f, ch_fsci_pal_small_b,
+   &nv_ch1_pal_small_f, &ch1_pal_small_f, ch1_fsci_pal_small_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},	      
   {{TV_SYSTEM_PAL,  768, 576, "Large",  "4:3", 0.000,  0.000}, /* Mode 19' */
-   &nv_ch_pal_large_f, &ch_pal_large_f, ch_fsci_pal_large_b,
+   &nv_ch1_pal_large_f, &ch1_pal_large_f, ch1_fsci_pal_huge_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},	      
   {{TV_SYSTEM_PAL,  800, 450, "Small", "16:9", 0.000,  0.000}, /* Mode 20" */
-   &nv_ch_pal_small_g, &ch_pal_small_g, ch_fsci_pal_small_b,	      
+   &nv_ch1_pal_small_g, &ch1_pal_small_g, ch1_fsci_pal_small_b,	      
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},	      
   {{TV_SYSTEM_PAL,  800, 450, "Large", "16:9", 0.000,  0.000}, /* Mode 19" */
-   &nv_ch_pal_large_g, &ch_pal_large_g, ch_fsci_pal_large_b,
+   &nv_ch1_pal_large_g, &ch1_pal_large_g, ch1_fsci_pal_huge_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},
 
 #ifdef CONFIG_SLAVE_CH
   {{TV_SYSTEM_PAL, 800, 600, "Slave$", "4:3", 13.194, 13.194}, /* Mode 20 */
-   &nv_ch_pal_slave_b, &ch_pal_slave_b, ch_fsci_pal_small_b,
+   &nv_ch1_pal_slave_b, &ch1_pal_slave_b, ch1_fsci_pal_small_b,
    PORT_NVIDIA_SYNC_SLAVE, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
   {{TV_SYSTEM_PAL, 640, 480, "Slave$", "4:3", 16.667, 16.667}, /* Mode 14 */
-   &nv_ch_pal_slave_a, &ch_pal_small_a, ch_fsci_pal_small_a,
+   &nv_ch1_pal_slave_a, &ch1_pal_small_a, ch1_fsci_pal_small_a,
    PORT_NVIDIA_SYNC_SLAVE, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},  
 #endif
 
 #if 0 /* not yet supported */
   {{TV_SYSTEM_PAL,  720, 576, "Interl", 0.000,  0.000},   /* Mode 25 */
-   &nv_ch_pal_interl_dvd, &ch_pal_interl_dvd, NULL, PORT_NVIDIA, TV_DEF_CH},
+   &nv_ch1_pal_interl_dvd, &ch1_pal_interl_dvd, NULL, PORT_NVIDIA, TV_DEF_CH},
 #endif
 
   {{TV_SYSTEM_NONE, 0, 0, NULL, NULL, 0.0, 0.0}, NULL, NULL, NULL, 0},
@@ -1870,66 +2123,66 @@ TVTemplateNvCh templ_nv_ch_50hz [] = { /* 50 Hz: PAL, PAL-N, PAL-X */
 
 /* 60 Hz: NTSC, NTSC-NDC, PAL-M, PAL-60 */
 
-TVTemplateNvCh templ_nv_ch_60hz [] = { 
+TVTemplateNvCh templ_nv_ch1_60hz [] = { 
   {{TV_SYSTEM_NTSC, 800, 600, "Small",  "4:3", 12.500, 12.500}, /* Mode 24 */
-   &nv_ch_ntsc_small_b, &ch_ntsc_small_b, ch_fsci_ntsc_small_b,
+   &nv_ch1_ntsc_small_b, &ch1_ntsc_small_b, ch1_fsci_ntsc_small_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
   {{TV_SYSTEM_NTSC, 800, 600, "Large",  "4:3",  6.250,  6.250}, /* Mode 23 */
-   &nv_ch_ntsc_large_b, &ch_ntsc_large_b, ch_fsci_ntsc_large_b,
+   &nv_ch1_ntsc_large_b, &ch1_ntsc_large_b, ch1_fsci_ntsc_large_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},
   {{TV_SYSTEM_NTSC, 800, 600, "Huge",   "4:3", -4.167, -4.167}, /* Mode 22 */
-   &nv_ch_ntsc_huge_b,  &ch_ntsc_huge_b,  ch_fsci_ntsc_huge_b,
+   &nv_ch1_ntsc_huge_b,  &ch1_ntsc_huge_b,  ch1_fsci_ntsc_huge_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},
   {{TV_SYSTEM_NTSC, 640, 480, "Tiny",   "4:3", 16.667, 16.667}, /* Mode 18 */
-   &nv_ch_ntsc_tiny_a,  &ch_ntsc_tiny_a,  ch_fsci_ntsc_tiny_a,
+   &nv_ch1_ntsc_tiny_a,  &ch1_ntsc_tiny_a,  ch1_fsci_ntsc_tiny_a,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},
   {{TV_SYSTEM_NTSC, 640, 480, "Small",  "4:3", 12.500, 12.500}, /* Mode 17 */
-   &nv_ch_ntsc_small_a, &ch_ntsc_small_a, ch_fsci_ntsc_small_a,
+   &nv_ch1_ntsc_small_a, &ch1_ntsc_small_a, ch1_fsci_ntsc_small_a,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
-  {{TV_SYSTEM_NTSC, 640, 480, "Large",  "4:3",  0.000,  0.000}, /* Mode 16 */
-   &nv_ch_ntsc_large_a, &ch_ntsc_large_a, ch_fsci_ntsc_large_a,
+  {{TV_SYSTEM_NTSC, 640, 480, "Huge",   "4:3",  0.000,  0.000}, /* Mode 16 */
+   &nv_ch1_ntsc_huge_a, &ch1_ntsc_huge_a, ch1_fsci_ntsc_huge_a,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW}, 
   {{TV_SYSTEM_NTSC, 640, 400, "Tiny",   "8:5", 27.083, 27.083}, /* Mode 12 */
-   &nv_ch_ntsc_tiny_c,  &ch_ntsc_tiny_c,  ch_fsci_ntsc_tiny_c,
+   &nv_ch1_ntsc_tiny_c,  &ch1_ntsc_tiny_c,  ch1_fsci_ntsc_tiny_c,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},	      
   {{TV_SYSTEM_NTSC, 640, 400, "Small",  "8:5", 16.667, 16.667}, /* Mode 11 */
-   &nv_ch_ntsc_small_c, &ch_ntsc_small_c, ch_fsci_ntsc_small_c,
+   &nv_ch1_ntsc_small_c, &ch1_ntsc_small_c, ch1_fsci_ntsc_small_c,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},	      
-  {{TV_SYSTEM_NTSC, 640, 400, "Large",  "8:5", -4.167, -4.167}, /* Mode 10 */
-   &nv_ch_ntsc_large_c, &ch_ntsc_large_c, ch_fsci_ntsc_large_c,
+  {{TV_SYSTEM_NTSC, 640, 400, "Huge",   "8:5", -4.167, -4.167}, /* Mode 10 */
+   &nv_ch1_ntsc_huge_c, &ch1_ntsc_huge_c, ch1_fsci_ntsc_huge_c,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},	      
   {{TV_SYSTEM_NTSC, 720, 400, "Small",  "9:5", 16.667, 16.667}, /* Mode 7  */
-   &nv_ch_ntsc_small_d, &ch_ntsc_small_d, ch_fsci_ntsc_small_d,
+   &nv_ch1_ntsc_small_d, &ch1_ntsc_small_d, ch1_fsci_ntsc_small_d,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},	      
-  {{TV_SYSTEM_NTSC, 720, 400, "Large",  "9:5", -4.167, -4.167}, /* Mode 6  */
-   &nv_ch_ntsc_large_d, &ch_ntsc_large_d, ch_fsci_ntsc_large_d,
+  {{TV_SYSTEM_NTSC, 720, 400, "Huge",   "9:5", -4.167, -4.167}, /* Mode 6  */
+   &nv_ch1_ntsc_huge_d, &ch1_ntsc_huge_d, ch1_fsci_ntsc_huge_d,
    PORT_NVIDIA, TV_DEF_CH}, /* FIXME: No Dualview yet */
   {{TV_SYSTEM_NTSC, 512, 384, "Small",  "4:3", 20.000, 20.000}, /* Mode 3  */
-   &nv_ch_ntsc_small_e, &ch_ntsc_small_e, ch_fsci_ntsc_small_e,
+   &nv_ch1_ntsc_small_e, &ch1_ntsc_small_e, ch1_fsci_ntsc_small_e,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
   {{TV_SYSTEM_NTSC, 512, 384, "Large",  "4:3",  0.000,  0.000}, /* Mode 2  */
-   &nv_ch_ntsc_large_e, &ch_ntsc_large_e, ch_fsci_ntsc_large_e,
+   &nv_ch1_ntsc_large_e, &ch1_ntsc_large_e, ch1_fsci_ntsc_large_e,
    PORT_NVIDIA, TV_DEF_CH}, /* FIXME: No Dualview */
 
   {{TV_SYSTEM_NTSC,  768, 576, "Large",  "4:3", 0.000, 0.000},  /* Mode 23' */
-   &nv_ch_ntsc_large_f, &ch_ntsc_large_f, ch_fsci_ntsc_large_b,
+   &nv_ch1_ntsc_large_f, &ch1_ntsc_large_f, ch1_fsci_ntsc_large_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},
   {{TV_SYSTEM_NTSC,  800, 450, "Large", "16:9", 0.000, 0.000},  /* Mode 23" */
-   &nv_ch_ntsc_large_g, &ch_ntsc_large_g, ch_fsci_ntsc_large_b,
+   &nv_ch1_ntsc_large_g, &ch1_ntsc_large_g, ch1_fsci_ntsc_large_b,
    PORT_NVIDIA, TV_DEF_CH | TV_DEF_DUALVIEW},
 
 #ifdef CONFIG_SLAVE_CH
   {{TV_SYSTEM_NTSC, 800, 600, "Slave$", "4:3", 12.500, 12.500}, /* Mode 24 */
-   &nv_ch_ntsc_slave_b, &ch_ntsc_slave_b, ch_fsci_ntsc_small_b,
+   &nv_ch1_ntsc_slave_b, &ch1_ntsc_slave_b, ch1_fsci_ntsc_small_b,
    PORT_NVIDIA_SYNC_SLAVE, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
   {{TV_SYSTEM_NTSC, 640, 480, "Slave$", "4:3", 12.500, 12.500}, /* Mode 17 */
-   &nv_ch_ntsc_slave_a, &ch_ntsc_small_a, ch_fsci_ntsc_small_a,
+   &nv_ch1_ntsc_slave_a, &ch1_ntsc_small_a, ch1_fsci_ntsc_small_a,
    PORT_NVIDIA_SYNC_SLAVE, TV_DEF_CH | TV_DEF_DUALVIEW | TV_CAP_MACROVISION},
 #endif
 
 #if 0 /* not yet supported, maybe never will */
   {{TV_SYSTEM_NTSC,  720, 480, "Interl", "DVD", 0.000,  0.000}, /* Mode 26 */
-   &nv_ch_ntsc_interl_dvd, &ch_ntsc_interl_dvd, NULL, PORT_NVIDIA, TV_DEF_CH},
+   &nv_ch1_ntsc_interl_dvd, &ch1_ntsc_interl_dvd, NULL, PORT_NVIDIA, TV_DEF_CH},
 #endif
 
   {{TV_SYSTEM_NONE, 0, 0, NULL, NULL, 0.0, 0.0}, NULL, NULL, NULL, 0},
@@ -1938,8 +2191,10 @@ TVTemplateNvCh templ_nv_ch_60hz [] = {
 /* -------- -------- */
 
 static TVMode *modes_nv_ch1 = NULL;
+static TVMode *modes_nv_ch2 = NULL;
 
-static TVMode *loop_create_nv_ch (TVMode *m, TVTemplateNvCh *t, 
+static TVMode*
+loop_create_nv_ch1 (TVMode *m, TVTemplateNvCh *t, 
   TVSystem system, int f, char *suffix)
 {
   for (; t->spec.system != TV_SYSTEM_NONE; t++) {
@@ -1959,37 +2214,82 @@ static TVMode *loop_create_nv_ch (TVMode *m, TVTemplateNvCh *t,
     m->regs.crtc.nv = *(t->nv);
     m->regs.enc.ch = *(t->ch);
     if (t->fsci) m->regs.enc.ch.fsci = t->fsci[f];
-    data_init_ch (system, &m->regs.enc.ch);
+    data_init_ch1 (system, &m->regs.enc.ch);
     data_init_nv (&m->regs.crtc.nv, m->regs.portHost, m->regs.devFlags);
     m++;
   }
   return m;
 }
 
-TVMode *data_modes_nv_ch1 (void)
+static TVMode*
+loop_create_nv_ch2 (TVMode *m, TVTemplateNvCh *t, 
+  TVSystem system, int f, char *suffix)
+{
+  for (; t->spec.system != TV_SYSTEM_NONE; t++) {
+    m->spec = t->spec;
+    m->spec.system = system;
+    m->descFlags = t->descFlags;
+    if (suffix) {
+      register char *s;
+
+      s = (char *) malloc ((strlen(t->spec.size) + strlen (suffix) + 1) *
+			   sizeof (char));
+      m->spec.size = strcat (strcpy (s, t->spec.size), suffix);
+    }
+    m->regs.devFlags = (t->descFlags & TV_CAP_DUALVIEW) ? 
+      (DEV_MONITOR | DEV_TELEVISION) : (DEV_TELEVISION);
+    m->regs.portHost = m->regs.portEnc = t->portFlags;
+    m->regs.crtc.nv = *(t->nv);
+    m->regs.enc.ch = *(t->ch);
+    if (t->fsci) m->regs.enc.ch.fsci = t->fsci[f];
+    data_init_ch2 (system, &m->regs.enc.ch);
+    data_init_nv (&m->regs.crtc.nv, m->regs.portHost, m->regs.devFlags);
+    m++;
+  }
+  return m;
+}
+
+
+TVMode*
+data_modes_nv_ch1 (void)
 {
   int c;
   TVMode *m;
 
   if (modes_nv_ch1) return modes_nv_ch1;
-  c = sizeof (templ_nv_ch_60hz) / sizeof(TVTemplateNvCh) * 5
-    + sizeof (templ_nv_ch_50hz) / sizeof(TVTemplateNvCh) * 3
+  c = sizeof (templ_nv_ch1_60hz) / sizeof(TVTemplateNvCh) * 5
+    + sizeof (templ_nv_ch1_50hz) / sizeof(TVTemplateNvCh) * 3
     + 1;
   modes_nv_ch1 = (TVMode *) malloc (sizeof (TVMode) * c);
   
   m = modes_nv_ch1;
-  m = loop_create_nv_ch (m, templ_nv_ch_60hz, TV_SYSTEM_NTSC,   0, NULL);
-  m = loop_create_nv_ch (m, templ_nv_ch_60hz, TV_SYSTEM_NTSC_J, 0, NULL);
-  m = loop_create_nv_ch (m, templ_nv_ch_60hz, TV_SYSTEM_NTSC,   1, "#");
-  m = loop_create_nv_ch (m, templ_nv_ch_60hz, TV_SYSTEM_PAL_M,  2, NULL);
-  m = loop_create_nv_ch (m, templ_nv_ch_60hz, TV_SYSTEM_PAL_60, 3, NULL);
-  m = loop_create_nv_ch (m, templ_nv_ch_50hz, TV_SYSTEM_PAL,    0, NULL);
-  m = loop_create_nv_ch (m, templ_nv_ch_50hz, TV_SYSTEM_PAL_N,  1, NULL);
-  m = loop_create_nv_ch (m, templ_nv_ch_50hz, TV_SYSTEM_PAL_X,  2, NULL);
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_60hz, TV_SYSTEM_NTSC,   0, NULL);
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_60hz, TV_SYSTEM_NTSC_J, 0, NULL);
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_60hz, TV_SYSTEM_NTSC,   1, "#");
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_60hz, TV_SYSTEM_PAL_M,  2, NULL);
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_60hz, TV_SYSTEM_PAL_60, 3, NULL);
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_50hz, TV_SYSTEM_PAL,    0, NULL);
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_50hz, TV_SYSTEM_PAL_N,  1, NULL);
+  m = loop_create_nv_ch1 (m, templ_nv_ch1_50hz, TV_SYSTEM_PAL_X,  2, NULL);
   m->spec.system = TV_SYSTEM_NONE;
   return modes_nv_ch1;
 }
 
+TVMode*
+data_modes_nv_ch2 (void)
+{
+  int c;
+  TVMode *m;
+
+  c = 0
+    + 1;
+  modes_nv_ch2 = (TVMode *) malloc (sizeof (TVMode) * c);
+  
+  m = modes_nv_ch2;
+  m->spec.system = TV_SYSTEM_NONE;
+  return modes_nv_ch2;
+}
+
 /* ======== PH ================================ */
 
 #define TV_DEF_PH (TV_CAP_MACROVISION | TV_CAP_NONINTERLACED \
@@ -2402,7 +2702,8 @@ static TVMode *modes_nv_ph2 = NULL;
  * count number of modes matching system; TV_SYSTEM_NONE matches all.
  */
 
-int data_count_nv_ph1 (TVSystem system)
+int 
+data_count_nv_ph1 (TVSystem system)
 {
   int count;
   TVTemplateNvPh1 *t;
@@ -2414,7 +2715,8 @@ int data_count_nv_ph1 (TVSystem system)
   return count;
 }
 
-int data_count_nv_ph2 (TVSystem system)
+int 
+data_count_nv_ph2 (TVSystem system)
 {
   int count;
   TVTemplateNvPh2 *t;
@@ -2426,7 +2728,8 @@ int data_count_nv_ph2 (TVSystem system)
   return count;
 }
 
-TVMode *data_modes_nv_ph1 (void)
+TVMode*
+data_modes_nv_ph1 (void)
 {
   int c;
   TVMode *m;
@@ -2453,7 +2756,8 @@ TVMode *data_modes_nv_ph1 (void)
   return modes_nv_ph1;
 }
 
-TVMode *data_modes_nv_ph2 (void)
+TVMode*
+data_modes_nv_ph2 (void)
 {
   int c;
   TVMode *m;
@@ -2496,6 +2800,141 @@ TVMode *data_modes_nv_ph2 (void)
   return modes_nv_ph2;
 }
 
+/* ======== NX ======== NVIDIA Internal ======================== */
+
+#define TV_DEF_NX 0
+
+/* -------- CRT -------- */
+
+TVNvRegs nv_nx_normal_a = { /* All 640x480 */
+  HDisplay    : 640,
+  HSyncStart  : 656,
+  HSyncEnd    : 752,
+  HTotal      : 880,
+  HBlankStart : 640,
+  HBlankEnd   : 880,
+  VDisplay    : 480,
+  VSyncStart  : 480,
+  VSyncEnd    : 492,
+  VTotal      : 525,
+  VBlankStart : 480,
+  VBlankEnd   : 525,
+  flags       : 0,
+  clock       : 0, 
+};
+
+TVNvRegs nv_nx_normal_b = { /* All 800x600 */
+  HDisplay    : 800,
+  HSyncStart  : 840,
+  HSyncEnd    : 968,
+  HTotal      : 1056,
+  HBlankStart : 800,
+  HBlankEnd   : 1056,
+  VDisplay    : 600,
+  VSyncStart  : 600,
+  VSyncEnd    : 605,
+  VTotal      : 628,
+  VBlankStart : 600,
+  VBlankEnd   : 628,
+  flags       : 0,
+  clock       : 0, 
+};
+
+TVNvRegs nv_nx_normal_c = { /* NTSC 1024x768 */
+  HDisplay    : 1024,
+  HSyncStart  : 1048,
+  HSyncEnd    : 1184,
+  HTotal      : 1344,
+  HBlankStart : 1024,
+  HBlankEnd   : 1344,
+  VDisplay    : 768,
+  VSyncStart  : 768,
+  VSyncEnd    : 777,
+  VTotal      : 806,
+  VBlankStart : 768,
+  VBlankEnd   : 806,
+  flags       : 0,
+  clock       : 0, 
+};
+
+/* -------- NX modes -------- */
+
+typedef struct {
+  TVModeSpec spec;
+  TVNvRegs *nv;
+  TVNxImgRegs *nx;
+  int portFlags;
+  int descFlags;
+} TVTemplateNvNx;
+
+TVTemplateNvNx templ_nv_nx_60hz [] = {
+  {{TV_SYSTEM_NTSC,  640, 480, "Normal", "4:3", 08.75, 08.75}, 
+   &nv_nx_normal_a, &nx_60_normal_a, PORT_NVIDIA, TV_DEF_NX},
+  {{TV_SYSTEM_NTSC,  800, 600, "Normal", "4:3", 08.75, 08.75}, 
+   &nv_nx_normal_b, &nx_60_normal_b, PORT_NVIDIA, TV_DEF_NX},
+  {{TV_SYSTEM_NTSC, 1024, 768, "Normal", "4:3", 08.75, 08.75}, 
+   &nv_nx_normal_c, &nx_60_normal_c, PORT_NVIDIA, TV_DEF_NX},
+
+  {{TV_SYSTEM_NONE, 0, 0, NULL, NULL, 0.0, 0.0}, NULL, NULL, 0},
+};
+
+TVTemplateNvNx templ_nv_nx_50hz [] = {
+  {{TV_SYSTEM_PAL, 640, 480, "Normal", "4:3", 08.75, 08.75},
+   &nv_nx_normal_a, &nx_50_normal_a, PORT_NVIDIA, TV_DEF_NX},
+  {{TV_SYSTEM_PAL, 800, 600, "Normal", "4:3", 08.75, 08.75},
+   &nv_nx_normal_b, &nx_50_normal_b, PORT_NVIDIA, TV_DEF_NX},
+  {{TV_SYSTEM_PAL, 1024, 768, "Normal", "4:3", 08.75, 08.75}, 
+   &nv_nx_normal_c, &nx_50_normal_c, PORT_NVIDIA, TV_DEF_NX},
+
+  {{TV_SYSTEM_NONE, 0, 0, NULL, NULL, 0.0, 0.0}, NULL, NULL, 0},
+};
+
+/* -------- -------- */
+
+static TVMode *modes_nv_nx = NULL;
+
+static TVMode*
+loop_create_nv_nx (TVMode *m, TVTemplateNvNx *t, TVSystem system)
+{
+  for (; t->spec.system != TV_SYSTEM_NONE; t++) {
+    m->spec = t->spec;
+    m->spec.system = system;
+    m->descFlags = t->descFlags;
+    m->regs.devFlags = DEV_TELEVISION | DEV_INTERNAL;
+    m->regs.portHost = m->regs.portEnc = t->portFlags;
+    m->regs.crtc.nv = *(t->nv);
+    m->regs.enc.nx.img = *(t->nx);
+    data_init_nx (system, &m->regs.enc.nx); /* initializes rest */
+    data_init_nv (&m->regs.crtc.nv, m->regs.portHost, m->regs.devFlags);
+    data_complete_nx (&m->regs);
+    m++;
+  }
+  return m;
+}
+
+TVMode*
+data_modes_nv_nx (void)
+{
+  int c;
+  TVMode *m;
+
+  if (modes_nv_nx) return modes_nv_nx;
+  c = sizeof (templ_nv_nx_60hz) / sizeof(TVTemplateNvNx) * 3
+    + sizeof (templ_nv_nx_50hz) / sizeof(TVTemplateNvNx) * 3
+    + 1;
+  modes_nv_nx = (TVMode *) malloc (sizeof (TVMode) * c);
+  
+  m = modes_nv_nx;
+  m = loop_create_nv_nx (m, templ_nv_nx_60hz, TV_SYSTEM_NTSC  );
+  m = loop_create_nv_nx (m, templ_nv_nx_60hz, TV_SYSTEM_NTSC_J);
+  m = loop_create_nv_nx (m, templ_nv_nx_60hz, TV_SYSTEM_PAL_M );
+  m = loop_create_nv_nx (m, templ_nv_nx_50hz, TV_SYSTEM_PAL   );
+  m = loop_create_nv_nx (m, templ_nv_nx_50hz, TV_SYSTEM_PAL_N );
+  m = loop_create_nv_nx (m, templ_nv_nx_50hz, TV_SYSTEM_PAL_NC);
+  m->spec.system = TV_SYSTEM_NONE;
+  return modes_nv_nx;
+}
+
 /* ======== ================================ */
 
 void data_init_nv (TVNvRegs *r, int portHost, int devFlags)
@@ -2615,10 +3054,16 @@ void data_setup_nv_cx (TVSettings *s, TVRegs *r)
   data_setup_cx (s, r);
 }
 
-void data_setup_nv_ch (TVSettings *s, TVRegs *r)
+void data_setup_nv_ch1 (TVSettings *s, TVRegs *r)
 {
   data_move_nv (s, r); 
-  data_setup_ch (s, r); 
+  data_setup_ch1 (s, r); 
+}
+
+void data_setup_nv_ch2 (TVSettings *s, TVRegs *r)
+{
+  data_move_nv (s, r); 
+  data_setup_ch2 (s, r); 
 }
 
 void data_setup_nv_ph (TVSettings *s, TVRegs *r)
@@ -2627,6 +3072,12 @@ void data_setup_nv_ph (TVSettings *s, TVRegs *r)
   data_setup_ph (s, r); 
 }
 
+void data_setup_nv_nx (TVSettings *s, TVRegs *r)
+{
+  data_move_nv (s, r); 
+  data_setup_nx (s, r); 
+}
+
 /* -------- -------- */
 
 DataCardFunc data_nv_func = {
@@ -2652,15 +3103,15 @@ DataFunc data_nv_cx_func = {
 DataFunc data_nv_ch1_func = {
   modes: data_modes_nv_ch1,
   defaults: data_default_ch,
-  setup: data_setup_nv_ch, 
+  setup: data_setup_nv_ch1, 
   clamp: data_clamp_ch,
   calc:  data_calc_null,
 };
 
 DataFunc data_nv_ch2_func = { /* FIXME */
-  modes: data_modes_nv_ch1,
+  modes: data_modes_nv_ch2, 
   defaults: data_default_ch,
-  setup: data_setup_nv_ch, 
+  setup: data_setup_nv_ch2, 
   clamp: data_clamp_ch,
   calc:  data_calc_null,
 };
@@ -2681,3 +3132,11 @@ DataFunc data_nv_ph2_func = {
   calc:  data_calc_null,
 };
 
+DataFunc data_nv_nx_func = {
+  modes: data_modes_nv_nx,
+  defaults: data_default_nx,
+  setup: data_setup_nv_nx, 
+  clamp: data_clamp_nx,
+  calc:  data_calc_null,
+};
+
