@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: gui_bt.c,v 1.23 2003/10/10 16:16:57 dthierbach Exp $
  *
  * Contents:
  *
@@ -403,7 +403,7 @@ void update_bt_cb (GtkObject *obj, gpointer data)
 
 void gui_map_bt_cb (GtkWidget *widget, gpointer data)
 {
-  DPRINTF ("gui_map_bt_cb\n");
+  RAISE (MSG_DEBUG, "gui_map_bt_cb");
 #ifndef DISABLE_TIMEOUT
   if (gui_timeout_bt_id == -1) {
     gui_timeout_bt_id = gtk_timeout_add (500 /* ms */, 
@@ -414,7 +414,7 @@ void gui_map_bt_cb (GtkWidget *widget, gpointer data)
 
 void gui_unmap_bt_cb (GtkWidget *widget, gpointer data)
 {
-  DPRINTF ("gui_unmap_bt_cb\n");
+  RAISE (MSG_DEBUG, "gui_unmap_bt_cb");
 #ifndef DISABLE_TIMEOUT
   if (gui_timeout_bt_id != -1) {
     gtk_timeout_remove (gui_timeout_bt_id);
