@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: gui_ch.c,v 1.20 2003/10/15 10:57:58 dthierbach Exp $
  *
  * Contents:
  *
@@ -105,7 +105,8 @@ static GuiRegMask ch_mask2_reg [] = {
 
 void reset_ch_cb (GtkWidget *widget, gpointer data)
 {
-  if ((gui_tv_chip & TV_ENCODER) == TV_CHRONTEL_MODEL1 && gui_act_mode) { 
+  if (((gui_tv_chip & TV_ENCODER) == TV_CHRONTEL_MODEL1 ||
+       (gui_tv_chip & TV_ENCODER) == TV_CHRONTEL_MODEL2) && gui_act_mode) { 
     gui_regs.enc.ch = gui_act_mode->regs.enc.ch; 
     gui_tv_set ();
     gtk_signal_emit_by_name (GTK_OBJECT (update_ch), "changed");
@@ -115,7 +116,8 @@ void reset_ch_cb (GtkWidget *widget, gpointer data)
 
 void update_ch_cb (GtkWidget *widget, gpointer data)
 {
-  if ((gui_tv_chip & TV_ENCODER) == TV_CHRONTEL_MODEL1) { 
+  if (((gui_tv_chip & TV_ENCODER) == TV_CHRONTEL_MODEL1 ||
+       (gui_tv_chip & TV_ENCODER) == TV_CHRONTEL_MODEL2) && gui_act_mode) { 
     gtk_signal_emit_by_name (GTK_OBJECT (update_ch), "changed");
   }
 }
@@ -219,7 +221,7 @@ gint check_civ_cb (struct mask_civ *m)
 
 void gui_map_ch_cb (GtkWidget *widget, gpointer data)
 {
-  DPRINTF ("gui_map_ch_cb\n");
+  RAISE (MSG_DEBUG, "gui_map_ch_cb");
 #ifndef DISABLE_TIMEOUT
   if (gui_timeout_ch_id == -1) {
     gui_timeout_ch_id = gtk_timeout_add (500 /* ms */, 
@@ -230,7 +232,7 @@ void gui_map_ch_cb (GtkWidget *widget, gpointer data)
 
 void gui_unmap_ch_cb (GtkWidget *widget, gpointer data)
 {
-  DPRINTF ("gui_unmap_ch_cb\n");
+  RAISE (MSG_DEBUG, "gui_unmap_ch_cb");
 #ifndef DISABLE_TIMEOUT
   if (gui_timeout_ch_id != -1) {
     gtk_timeout_remove (gui_timeout_ch_id);
