@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: gui_cx.c,v 1.11 2003/10/10 16:16:57 dthierbach Exp $
  *
  * Contents:
  *
@@ -286,7 +286,7 @@ void update_cx_cb (GtkObject *obj, gpointer data)
 
 void gui_map_cx_cb (GtkWidget *widget, gpointer data)
 {
-  DPRINTF ("gui_map_cx_cb\n");
+  RAISE (MSG_DEBUG, "gui_map_cx_cb");
 #ifndef DISABLE_TIMEOUT
   if (gui_timeout_cx_id == -1) {
     gui_timeout_cx_id = gtk_timeout_add (500 /* ms */, 
@@ -297,7 +297,7 @@ void gui_map_cx_cb (GtkWidget *widget, gpointer data)
 
 void gui_unmap_cx_cb (GtkWidget *widget, gpointer data)
 {
-  DPRINTF ("gui_unmap_cx_cb\n");
+  RAISE (MSG_DEBUG, "gui_unmap_cx_cb");
 #ifndef DISABLE_TIMEOUT
   if (gui_timeout_cx_id != -1) {
     gtk_timeout_remove (gui_timeout_cx_id);
