@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: nvtv.c,v 1.7 2004/03/01 21:08:10 dthierbach Exp $
  *
  * Contents:
  *
@@ -27,6 +27,8 @@
 #include "local.h" /* before everything else */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 #include <getopt.h>
 
@@ -34,7 +36,7 @@
 #include <unistd.h>
 #endif
 
-#if HAVE_X
+#ifdef HAVE_X
 #include <X11/Xlib.h>
 #endif
 
@@ -75,6 +77,8 @@
 
 /* -------- Global options -------- */
 
+Bool opt_debug = FALSE;
+
 Bool opt_no_root = FALSE;
 
 Bool opt_nvdev = FALSE;
@@ -113,7 +117,7 @@ int opt_pci_func = -1;
 
 TVSettings opt_set;
 
-static const char *short_options = "?12bcdf:ghmno:pqr:s:tw:A:C:FNPS:T:W:X";
+static const char *short_options = "?12bcdf:ghlmno:pqr:s:tw:A:C:FNPS:T:W:X";
 
 static struct option long_options[] =
   {{"tv-bars",        no_argument,       NULL, 'b'},
@@ -122,6 +126,7 @@ static struct option long_options[] =
    {"set-flicker",    required_argument, NULL, 'f'},
    {"gui",            no_argument,       NULL, 'g'},
    {"help",           no_argument,       NULL, 'h'},
+   {"list",           no_argument,       NULL, 'l'},
    {"tv-off",         no_argument,       NULL, 'm'},
    {"no-root",        no_argument,       NULL, 'n'},
    {"overscan",       required_argument, NULL, 'o'},
@@ -197,6 +202,7 @@ typedef enum {
   ACTION_DEBUG   = 6,
   ACTION_PROBE   = 7,
   ACTION_QUERY   = 8,
+  ACTION_LIST    = 9,
 } ActionMain;
 
 typedef enum {
@@ -205,7 +211,29 @@ typedef enum {
   ACTION_FETCH   = (1 << 2),
 } ActionAux;
 
-/* -------- */
+/* -------- Error handling -------- */
+
+void raise_msg (int class, char *format, ...)
+{
+  va_list args;
+
+  if (class >= MSG_DEBUG && !opt_debug) return;
+  switch (class) {
+    case MSG_ABORT:
+      fprintf (stderr, "Fatal: ");
+      break;
+    case MSG_WARNING:
+      fprintf (stderr, "Warning: ");
+      break;
+  }
+  va_start(args, format);
+  vfprintf (stderr, format, args);
+  va_end(args);
+  if (class != MSG_DEBUG_NL) fprintf (stderr, "\n");
+  if (class == MSG_ABORT) exit (1);
+}
+
+/* -------- Usage -------- */
 
 void usage (void)
 {
@@ -214,7 +242,9 @@ void usage (void)
   int i;
 
   fprintf (stderr,
-      "usage:  nvtv [-options ...]\n\n");
+      "NVTV version " PACKAGE_VERSION "\n\n");
+  fprintf (stderr,
+      "Usage: nvtv [-options ...]\n\n");
   fprintf (stderr,
       "where options include:\n");
   fprintf (stderr,
@@ -231,6 +261,8 @@ void usage (void)
       "  -p --print            calculate and print register values\n");
   fprintf (stderr,	       
       "  -F --fetch            fetch current mode (use only for printing)\n");
+  fprintf (stderr,	       
+      "  -l --list             list available modes on stdout\n");
   fprintf (stderr,	       
       "  -r --resolution x,y   Resolution\n");
   fprintf (stderr,	       
@@ -238,11 +270,13 @@ void usage (void)
   fprintf (stderr,	       
       "  -s --size <size>      Size (for predefined modes)\n");
   fprintf (stderr,	       
-      "  -S --system           NTSC,NTSC-J,PAL,PAL-60,PAL-NC,PAL-M,PAL-M60,SECAM\n");
+      "  -S --system           NTSC,NTSC-J,PAL,PAL-60,PAL-NC,PAL-M,PAL-M60,PAL-X,SECAM\n");
   fprintf (stderr,	       
       "  -C --connector        AUTO,COMPOSITE,SVIDEO,BOTH,CONVERT\n");
   fprintf (stderr,
-      "  -T --chip             BROOKTREE,CHRONTEL,CONEXANT,PHILIPS1/2 or <bus>:<addr>\n");
+      "  -T --chip             BROOKTREE,CONEXANT,NVIDIA,CHRONTEL1,CHRONTEL2,\n");
+  fprintf (stderr,
+      "                        PHILIPS1,PHILIPS2,NVIDIA or <bus>:<addr>\n");
   fprintf (stderr,
       "  -A --card-addr <addr> video card pci address as <bus>:<slot>.<func>\n");
   fprintf (stderr,
@@ -295,6 +329,24 @@ void usage (void)
   exit (1);
 }
 
+/* -------- -------- */
+
+ChipPtr findDefOptChip (CardPtr card)
+{
+  ChipPtr c, ca;
+
+  ca = card->chips; 
+  if (opt_tv_chip != TV_NO_CHIP) {
+    for (c = card->chips; c; c = c->next)
+    {
+      if (c->type == opt_tv_chip) {
+	ca = c; break;
+      }
+    }
+  }
+  return ca;
+}
+
 void printRegs (TVMode *mode, CardType card, TVChip chip)
 {
   printf ("*** Resolution %03i x %03i  Overscan %06.3f x %06.3f\n",
@@ -310,7 +362,56 @@ int doQuery (int head)
   if (head == -1) head = 1;
   back_card->getHeadDev (head, &dev);
   printf ("%i", dev);
-  return dev + 100;
+  return dev + 100; /* exit code */
+}
+
+int doList (CardPtr card)
+{
+  ChipPtr chip;
+  DataFunc *func;
+  TVMode *m;
+  static char* sys[] = {
+    "NTSC", "NTSC-J", "PAL", "PAL-60", "PAL-N", "PAL-Nc", "PAL-M", 
+    "PAL-M60", "PAL-X", "SECAM"
+  };
+
+  /* TODO FIXME: Use card->listModes */
+  /* Hm. Must set chip for that, which does init ... */
+
+#if 1
+
+  chip = findDefOptChip (card);
+  if (!chip) RAISE (MSG_ABORT, "No encoder chip found, and none specified.\n");
+  func = data_func (card->type, chip->type); 
+  m = func->modes ();
+  printf ("Modes for %s on %s\n", chip->name, card->name);
+  if (m) {
+    while (m->spec.system != TV_SYSTEM_NONE) {
+      if (opt_system == TV_SYSTEM_NONE || m->spec.system == opt_system) {
+	printf ("%s\t%4i,%4i\t%s\t%05.2f,%05.2f\t%s\n", sys[m->spec.system],
+		m->spec.res_x, m->spec.res_y, m->spec.size,
+		m->spec.hoc, m->spec.voc, m->spec.aspect);
+      }
+      m++;
+    }
+  }
+  
+#else
+
+  back_card->setChip(, FALSE);
+  /* FIXME change listModes to use card->type, chip->type ? */
+  n = back_card->listModes (opt_system, &modes);
+  printf ("Modes for %s on %s\n", chip->name, card->name);
+  for (m = modes, i = 1; i <= n; i++, m++) {
+	printf ("%s\t%4i,%4i\t%s\t%05.2f,%05.2f\t%s\n", sys[m->spec.system],
+		m->spec.res_x, m->spec.res_y, m->spec.size,
+		m->spec.hoc, m->spec.voc, m->spec.aspect);
+  }
+  xfree (modes);
+
+#endif
+  
+  return 0; /* exit code */
 }
 
 void doDebug (void)
@@ -376,8 +477,7 @@ void modifySet (TVSettings *s, OptIntVal *m)
   while (m) {
     i = (char *) m->addr - (char *) s;
     if (i < 0 || i > sizeof(TVSettings)) {
-      printf ("Fatal: Wrong settings to modify\n");
-      exit (1);
+      RAISE (MSG_ABORT, "Wrong settings to modify");
     }
     *(m->addr) = m->val;
     m = m->link;
@@ -397,22 +497,22 @@ Bool processOptSet (char *opt, OptIntVal **head, OptIntVal **tail)
   name = opt;
   s = strchr(opt, ':');
   if (!s) {
-    fprintf (stderr, "Illegal setting: %s\n", opt);
+    RAISE (MSG_ERROR, "Illegal setting: %s", opt);
     return FALSE;
   }
   *s++ = '\0';
-  DPRINTF ("process settings %s", name);
+  RAISE (MSG_DEBUG_NL, "process settings %s", name);
   for (p = opt_set_list; p->name; p++) {
     if (strcasecmp (name, p->name) == 0) break;
   }
   if (p->name) {
     if (sscanf (s, "%i", &val) != 1) {
-      fprintf (stderr, "Illegal setting: %s\n", opt);
+      RAISE (MSG_ERROR, "Illegal setting: %s", opt);
       return FALSE;
     }
-    DPRINTF (" = %i\n", val);
+    RAISE (MSG_DEBUG, " = %i", val);
     if (val < p->min || val > p->max) {
-      fprintf (stderr, "Setting %s out of range (%i to %i)\n", 
+      RAISE (MSG_ERROR, "Setting %s out of range (%i to %i)", 
 	       p->name, p->min, p->max);
       return FALSE;
     }
@@ -436,10 +536,11 @@ Bool processOptSet (char *opt, OptIntVal **head, OptIntVal **tail)
     else if (strcasecmp (s, "on") == 0) val = 1; 
     else if (strcasecmp (s, "1") == 0) val = 1; 
     else {
-      fprintf (stderr, "Illegal flag value: %s\n", s);
+      RAISE (MSG_DEBUG, "");
+      RAISE (MSG_ERROR, "Illegal flag value: %s", s);
       return FALSE;
     }
-    DPRINTF (" = %s\n", val ? "true" : "false");
+    RAISE (MSG_DEBUG, " = %s", val ? "true" : "false");
     *(r->addr_mask) |= r->mask;
     if (val) {
       *(r->addr_val) |= r->mask;
@@ -448,8 +549,7 @@ Bool processOptSet (char *opt, OptIntVal **head, OptIntVal **tail)
     }
     return TRUE;
   }
-  DPRINTF ("\n");
-  fprintf (stderr, "Unknown setting %s\n", name);
+  RAISE (MSG_ERROR, "Unknown setting %s", name);
   return FALSE;
 }
 
@@ -472,6 +572,10 @@ CardPtr findCard (CardPtr list)
   return card;
 }
 
+#ifdef TEST_HOOKS
+#define main xmain
+#endif
+
 int main (int argc, char *argv[])
 {
   CardPtr main_card_list, main_card = NULL;
@@ -516,7 +620,7 @@ int main (int argc, char *argv[])
 	  usage ();
 	break;
       case 'd':
-	act_next = ACTION_DEBUG;
+	opt_debug = TRUE;
 	break;
       case 'm':
 	act_next = ACTION_TVOFF;
@@ -539,6 +643,9 @@ int main (int argc, char *argv[])
       case 'q':
 	act_next = ACTION_QUERY;
 	break;
+      case 'l':
+        act_next = ACTION_LIST;
+	break;
       case 's':
 	if (optarg) opt_size = optarg;
 	break;
@@ -559,6 +666,8 @@ int main (int argc, char *argv[])
 	  opt_system = TV_SYSTEM_PAL_M; 
 	else if (optarg && strcasecmp (optarg, "pal-m60") == 0) 
 	  opt_system = TV_SYSTEM_PAL_M60; 
+	else if (optarg && strcasecmp (optarg, "pal-x") == 0) 
+	  opt_system = TV_SYSTEM_PAL_X; 
 	else if (optarg && strcasecmp (optarg, "secam") == 0) 
 	  opt_system = TV_SYSTEM_SECAM; 
 	else
@@ -599,6 +708,8 @@ int main (int argc, char *argv[])
 	  opt_tv_chip = TV_PHILIPS_MODEL2; 
 	else if (optarg && strcasecmp (optarg, "philips") == 0) 
 	  opt_tv_chip = TV_PHILIPS_MODEL1; 
+	else if (optarg && strcasecmp (optarg, "nvidia") == 0) 
+	  opt_tv_chip = TV_NVIDIA; 
 	else if (optarg && 
 		 sscanf (optarg, "%i:%x", &opt_tv_bus, &opt_tv_addr) == 2)
 	  opt_tv_chip = TV_CHIP_BY_ADDR;
@@ -621,7 +732,7 @@ int main (int argc, char *argv[])
 	  if (sscanf (optarg, "%ld", &opt_window) != 1) opt_window = None;
 	}
 	if (opt_window == None)
-	  fprintf (stderr, "Invalid window id format: %s\n.", optarg);
+	  RAISE (MSG_ERROR, "Invalid window id format: %s.", optarg);
 	break;
 #endif
       case 'c':
@@ -650,7 +761,7 @@ int main (int argc, char *argv[])
     if (act_next != ACTION_NONE) 
     {
       if (act_main != ACTION_NONE) {
-	fprintf (stderr, "nvtv: More than one action. Choosing last one.\n");
+	RAISE (MSG_WARNING, "More than one action. Choosing last one.");
       }
       act_main = act_next;
     }
@@ -676,15 +787,13 @@ int main (int argc, char *argv[])
       main_card_list = NULL;
     }
     if (!main_card_list) {
-      fprintf (stderr, "Either you are not root, or no NVidia card found.\n");
-      exit (1);
+      RAISE (MSG_ABORT, "Either you are not root, or no NVidia card found.");
     }
     main_card = findCard (main_card_list);
     bdir_probeSystem (main_card);
     return 0;
 #else
-    fprintf (stderr, "No probing enabled in this built.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "No probing enabled in this built.");
 #endif
   }
 
@@ -693,16 +802,16 @@ int main (int argc, char *argv[])
 
   if (opt_size || opt_hoc >= 0.0 || opt_voc >= 0.0) {
     if (opt_system == TV_SYSTEM_NONE) {
-      fprintf (stderr, "Defaulting to PAL TV system.\n");
+      RAISE (MSG_INFO, "Defaulting to PAL TV system.");
       opt_system = TV_SYSTEM_PAL;
     }
     if (opt_res_x <= 0) {
       opt_res_x = 800;
-      fprintf (stderr, "Defaulting to x resolution %i\n", opt_res_x);
+      RAISE (MSG_INFO, "Defaulting to x resolution %i", opt_res_x);
     }
     if (opt_res_y <= 0) {
       opt_res_y = 600;
-      fprintf (stderr, "Defaulting to y resolution %i\n", opt_res_y);
+      RAISE (MSG_INFO, "Defaulting to y resolution %i", opt_res_y);
     }
     if (act_main == ACTION_NONE) {
       act_main = ACTION_GUI;
@@ -713,57 +822,58 @@ int main (int argc, char *argv[])
     act_main = ACTION_GUI;
   }
 
-  /* Next, choose the backend. If a main action is present, always try to 
-     get a backend. If -n is specified, force the null backend. For
-     printing, we can get away with having the null backend, as well,
-     unless we have to fetch the mode. */
+  /* Next, choose the backend. If a main action is present, always try
+     to get a backend. If -n is specified, force the null backend. For
+     the list of modes and printing, we can get away with having the
+     null backend, as well, unless we have to fetch the mode. 
+  */
 
   if (act_main != ACTION_NONE) {
     if (opt_no_root) {
-      DPRINTF ("init null\n");
+      RAISE (MSG_DEBUG, "init null");
       main_card = main_card_list = back_null_init (); 
     } else {
 #ifdef USE_UNIX_BACKEND
       if (back_root_avail ()) {
-	DPRINTF ("init root\n");
+	RAISE (MSG_DEBUG, "init root");
 	main_card_list = back_root_init ();
       }
 #endif
 #ifdef USE_MSWIN_BACKEND
       if (back_win_avail ()) {
-	DPRINTF ("init root\n");
+	RAISE (MSG_DEBUG, "init root");
 	main_card_list = back_win_init ();
       }
 #endif
 #ifdef USE_CLIENT_BACKEND
       else if (back_client_avail ()) {
-	DPRINTF ("init client\n");
+	RAISE (MSG_DEBUG, "init client");
 	main_card_list = back_client_init ();
       }
 #endif
 #ifdef USE_UNIX_BACKEND
       else if (opt_nvdev && back_nvdev_avail (TRUE)) {
-	DPRINTF ("init nvdev\n");
+	RAISE (MSG_DEBUG, "init nvdev");
 	main_card_list = back_nvdev_init ();
       }
 #endif
-      else if (act_main == ACTION_PRINT && ! (act_aux & ACTION_FETCH)) {
-	DPRINTF ("init null\n");
+      else if (act_main == ACTION_PRINT && ! (act_aux & ACTION_FETCH) ||
+	       act_main == ACTION_LIST && (opt_tv_chip != TV_NO_CHIP)) {
+	RAISE (MSG_DEBUG, "init null");
 	main_card_list = back_null_init ();
       } else {
-	fprintf (stderr, "Cannot access video cards. Either you are not root, "
-                 "or the NVidia devices are\nnot accessible.\n");
-	exit (1);
+	RAISE (MSG_ABORT, "%s%s", "Cannot access video cards. Either you "
+	       "are not root, or the\nNVidia devices are not accessible.",
+	       (act_main == ACTION_LIST) ? "\n(For a list of modes, please "
+	       "specify the encoder chip in this case.)" : "");
       }
     }
     if (!main_card_list) {
-      fprintf (stderr, "No supported video card found.\n");
-      exit (1);
+      RAISE (MSG_ABORT, "No supported video card found.");
     }
     main_card = findCard (main_card_list);
     if (!main_card) {
-      fprintf (stderr, "No supported video card found at specified address.\n");
-      exit (1);
+      RAISE (MSG_ABORT, "No supported video card found at specified address.");
     }
   }
 
@@ -790,18 +900,17 @@ int main (int argc, char *argv[])
       ChipPtr chip;
 
       for (chip = main_card->chips; chip; chip = chip->next) {
-	if (opt_tv_chip == chip->chip || (opt_tv_chip == TV_CHIP_BY_ADDR &&
+	if (opt_tv_chip == chip->type || (opt_tv_chip == TV_CHIP_BY_ADDR &&
 	    opt_tv_bus == chip->bus && opt_tv_addr == chip->addr)) break;
       }
       if (chip) {
 	if (opt_tv_chip == TV_CHIP_BY_ADDR) {
-	  opt_tv_chip = chip->chip;
+	  opt_tv_chip = chip->type;
 	}
 	back_card->setChip (chip, TRUE);
 	/* FIXME: Should set head here */
       } else {
-	fprintf (stderr, "Cannot find specified chip.\n");
-	exit (1);
+	RAISE (MSG_ABORT, "Cannot find specified chip.");
       }
     }
   }
@@ -810,25 +919,23 @@ int main (int argc, char *argv[])
      fail if no mode is found. */
 
   if ((act_aux & ACTION_FETCH) && act_main != ACTION_PRINT) {
-    fprintf (stderr, "Can only fetch mode for printing. Ignoring option.\n");
+    RAISE (MSG_WARNING, "Can only fetch mode for printing. Ignoring option.");
   }
   if (act_main == ACTION_TVON || act_main == ACTION_PRINT) {
     if (opt_size) {
       if (! back_card->findBySize (opt_system, opt_res_x, opt_res_y, opt_size,
 				 &main_mode)) 
       {
-	fprintf (stderr, "Cannot find '%s' mode %i x %i \n", 
-		 opt_size, opt_res_x, opt_res_y);
-	fprintf (stderr, "Please specify as e.g. -r 800,600 -s Large\n");
-	exit (1);
+	RAISE (MSG_ABORT, "Cannot find '%s' mode %i x %i \n"
+	       "Please specify as e.g. -r 800,600 -s Large\n", 
+	       opt_size, opt_res_x, opt_res_y);
       }
     } else if (opt_hoc >= 0.0 || opt_voc >= 0.0) {
       if (opt_hoc < 0.0) opt_hoc = 0.10;
       if (opt_voc < 0.0) opt_voc = 0.10;
       /* FIXME: Defaulting message */
       /* FIXME: findByOverscan */
-      fprintf (stderr, "Specifying modes by overscan is not implemented yet\n");
-      exit (1);
+      RAISE (MSG_ABORT, "Specifying modes by overscan is not implemented yet.");
     } else if (act_main == ACTION_PRINT && (act_aux & ACTION_FETCH)) {
       back_card->getMode (&main_mode.regs);
       main_mode.spec.system = TV_SYSTEM_NONE;
@@ -840,9 +947,8 @@ int main (int argc, char *argv[])
       main_mode.spec.voc = 0.0;
       main_mode.descFlags = 0;
     } else {
-      fprintf (stderr, "No mode specified.\n");
-      fprintf (stderr, "Please specify as e.g. -r 800,600 -s Large\n");
-      exit (1);
+      RAISE (MSG_ABORT, "No mode specified.\n"
+	     "Please specify as e.g. -r 800,600 -s Large");
     }
   }
       
@@ -853,8 +959,10 @@ int main (int argc, char *argv[])
     /* For some reason, just XOpenDisplay is not enough if GDK is 
        linked in / to be used later. */
     gdk_init (&argc, &argv);
+#if HAVE_X
     main_dpy = gdk_display;
     main_screen = my_gdk_screen;
+#endif 
 #else
 #if HAVE_X
     main_dpy = XOpenDisplay (opt_dpy);
@@ -867,6 +975,7 @@ int main (int argc, char *argv[])
      and default resolution, if necessary. */
 
   if (act_main == ACTION_TVOFF) {
+    main_regs.devFlags = DEV_MONITOR;
 #if HAVE_X
     if (opt_res_x > 0 && opt_res_y > 0) {
       find_vidmode (main_dpy, main_screen, opt_res_x, opt_res_y, 
@@ -879,12 +988,9 @@ int main (int argc, char *argv[])
     if (opt_size && data_vesa_mode (opt_size, &main_regs.crtc, 
 				    data_card_func(main_card->type)->make)) {
     } else {
-      fprintf (stderr, "No monitor mode to switch tv off. "
-	               "You need the XVidMode extension.\n");
-      exit (1);
+      RAISE (MSG_ABORT, "No monitor mode to switch tv off. "
+	     "You need the XVidMode extension.\n");
     }
-    /* FIXME: Without XVidMode extension, try to get the mode from 
-       somewhere else, i.e. internal VESA database? */
     /* NOTE: There must be a valid mode in main_crt here */
   }
 
@@ -893,14 +999,15 @@ int main (int argc, char *argv[])
 
   if ((act_aux & ACTION_XMODE) && ((opt_res_x < 0) || (opt_res_y < 0))) {
     act_aux &= ~ACTION_XMODE;
-    fprintf (stderr, "No resolution found. Switching X mode disabled.\n");
+    RAISE (MSG_WARNING,  "No resolution found. Switching X mode disabled.");
   }
 
   /* Finally, modify the settings, unless the main action is none, gui,
-     or query. */
+     query, list, or tv off. */
 
   if (act_main != ACTION_NONE && act_main != ACTION_GUI &&
-      act_main != ACTION_DEBUG && act_main != ACTION_QUERY && back_card) 
+      act_main != ACTION_DEBUG && act_main != ACTION_QUERY && 
+      act_main != ACTION_TVOFF && act_main != ACTION_LIST && back_card) 
   {
     back_card->getSettings (&opt_set);
     if (opt_connect > CONNECT_NONE) {
@@ -924,13 +1031,15 @@ int main (int argc, char *argv[])
     case ACTION_PRINT:
       printRegs (&main_mode, main_card->type, 
         (opt_tv_chip == TV_NO_CHIP && main_card->chips) ? 
-	   main_card->chips->chip : opt_tv_chip);
+	   main_card->chips->type : opt_tv_chip);
       return 0;
     case ACTION_DEBUG:
       doDebug ();
       return 0;
     case ACTION_QUERY:
       return doQuery (opt_head);
+    case ACTION_LIST:
+      return doList (main_card);
     default:
       break;
   }
@@ -960,7 +1069,7 @@ int main (int argc, char *argv[])
     opt_window = Window_With_Name (main_dpy, 
       RootWindow(main_dpy, main_screen), opt_window_name);
     if (opt_window == None) {
-      fprintf (stderr, "No window with name '%s' exists.\n", opt_window_name);
+      RAISE (MSG_ERROR, "No window with name '%s' exists.", opt_window_name);
     }
   }
 #endif
@@ -972,8 +1081,7 @@ int main (int argc, char *argv[])
     if (opt_window != None) {
       center_window (main_dpy, main_screen, opt_window, opt_res_x, opt_res_y);
     } else {
-      fprintf (stderr, "No window specified.\n");
-      exit (1);
+      RAISE (MSG_ABORT, "No window specified.");
     }
   }
 #endif
@@ -997,8 +1105,7 @@ int main (int argc, char *argv[])
 #ifdef HAVE_GTK
     gui_main (argc, argv, main_card); 
 #else
-    fprintf (stderr, "Compiled without gui\n");
-    return 1;
+    RAISE (MSG_ABORT, "Compiled without gui");
 #endif
   }
 
