@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: pipe.c,v 1.12 2003/10/10 16:16:57 dthierbach Exp $
  *
  * Contents:
  *
@@ -51,9 +51,9 @@ PipeCmd pipeReadCmd (FILE *pipe)
 {
   PipeCmd cmd;
 
-  DPRINTF ("pipe read cmd\n");
+  RAISE (MSG_DEBUG, "pipe read cmd");
   fread (&cmd, sizeof(PipeCmd), 1, pipe);
-  DPRINTF ("pipe read cmd, got %i\n", cmd);
+  RAISE (MSG_DEBUG, "pipe read cmd, got %i", cmd);
   return cmd;
 }
 
@@ -63,10 +63,10 @@ PipeCmd pipeReadCmd (FILE *pipe)
 
 void pipeWriteCmd (FILE *pipe, PipeCmd cmd)
 {
-  DPRINTF ("pipe write cmd %i\n", cmd);
+  RAISE (MSG_DEBUG, "pipe write cmd %i", cmd);
   fwrite (&cmd, sizeof(PipeCmd), 1, pipe);
   fflush (pipe);
-  DPRINTF ("pipe write cmd done\n");
+  RAISE (MSG_DEBUG, "pipe write cmd done");
 }
 
 /* 
@@ -81,7 +81,7 @@ void pipeWriteArgs (FILE *pipe, int n, ...)
   int i, s;
   void *p;
 
-  DPRINTF ("pipe write args\n");
+  RAISE (MSG_DEBUG, "pipe write args");
   fwrite (&n, sizeof(n), 1, pipe);
   va_start(ap, n);
   for (i = 0; i < n; i++) {
@@ -156,7 +156,7 @@ int pipeReadArgs (FILE *pipe, int n, ...)
   va_list ap;
   int r;
 
-  DPRINTF ("pipe read args\n");
+  RAISE (MSG_DEBUG, "pipe read args");
   va_start(ap, n);
   r = pipeReadArgsMulti (pipe, 0, n, ap);
   va_end(ap);
@@ -176,7 +176,7 @@ int pipeReadArgsOpt (FILE *pipe, int n, ...)
   va_list ap;
   int r;
 
-  DPRINTF ("pipe read args opt\n");
+  RAISE (MSG_DEBUG, "pipe read args opt");
   va_start(ap, n);
   r = pipeReadArgsMulti (pipe, 1, n, ap);
   va_end(ap);
@@ -195,7 +195,7 @@ void* pipeReadList (FILE *pipe, int size)
   int i, j, n, t;
   char *s;
 
-  DPRINTF ("pipe read list\n");
+  RAISE (MSG_DEBUG, "pipe read list");
   root = p = NULL;
   fread (&n, sizeof(n), 1, pipe);
   n /= 2;
@@ -240,7 +240,7 @@ void pipeWriteList (FILE *pipe, int size, void *list)
   char *s;
   int n, k;
 
-  DPRINTF ("pipe write list\n");
+  RAISE (MSG_DEBUG, "pipe write list");
   n = 0;
   for (l = list; l; l = * (void **) l) n++;
   n *= 2;
@@ -271,10 +271,10 @@ void pipeReadArray (FILE *pipe, int *nelem)
 {
   int n;
 
-  DPRINTF ("pipe read array ");
+  RAISE (MSG_DEBUG, "pipe read array [");
   fread (&n, sizeof(n), 1, pipe);
   fread (nelem, sizeof(int), 1, pipe);
-  DPRINTF ("%i (%i)\n", *nelem, n);
+  RAISE (MSG_DEBUG, "] %i (%i)", *nelem, n);
 }
 
 /*
@@ -288,7 +288,7 @@ void pipeWriteArray (FILE *pipe, int nelem, int nsub)
 {
   int n;
 
-  DPRINTF ("pipe write array %i * %i\n", nelem, nsub);
+  RAISE (MSG_DEBUG, "pipe write array %i * %i", nelem, nsub);
   n = nelem * nsub + 1;
   fwrite (&n, sizeof(n), 1, pipe);
   fwrite (&nelem, sizeof(nelem), 1, pipe);
