@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: tv_ch1_7007.c,v 1.9 2004/01/30 08:47:02 dthierbach Exp $
  *
  * Contents:
  *
@@ -286,31 +286,28 @@ I2CByte *TVCh1MacroRegs (TVEncoderRegs *r)
  * 9: 1c-7  goenb1
  */
 
-void TVCh1Create (TVEncoderObj *this, TVChip chip, I2CDevPtr dev)
+void TVCh1Create (TVEncoderObj *this, TVChip chip_type, void* ctrl)
 {
+  register I2CDevPtr dev = (I2CDevPtr) ctrl;
   I2CByte result;
 
-  this->chip = chip;
-  this->dev = dev;
+  this->type = chip_type;
+  this->ctrl = ctrl;
   this->hwstate = 0x07;
   tvBusOk = TRUE;
-  TVReadBus (this->dev, 0x80|0x20, &result);
+  TVReadBus (dev, 0x80|0x20, &result);
   this->hwconfig = SetBitField(result, 3:0, 3:0);
-  TVReadBus (this->dev, 0x80|0x1b, &result);
+  TVReadBus (dev, 0x80|0x1b, &result);
   this->hwconfig |= SetBitField(result, 7:5, 7:5);
-  TVReadBus (this->dev, 0x80|0x1c, &result);
+  TVReadBus (dev, 0x80|0x1c, &result);
   this->hwconfig |= SetBitField(result, 7:6, 9:8);
 #ifdef FAKE_I2C
   this->hwconfig = 0x32a; /* power on value of my card */
 #endif
   if (!tvBusOk) {
-    fprintf (stderr, "Critical: Error reading Chrontel hardware config\n");
-    exit (1);
-  }
-  DPRINTF ("create ch hw=%08X\n", this->hwconfig);
-  if ((this->hwconfig & 0x2f) != 0x2a) {
-    fprintf (stderr, "Warning: Chrontel has unusual power settings.\n");
+    RAISE (MSG_ABORT, "Critical: Error reading Chrontel hardware config\n");
   }
+  RAISE (MSG_DEBUG, "create ch hw=%08X", this->hwconfig);
 }
 
 /* Init chrontel register values (all done in SetChRegs)
@@ -334,22 +331,18 @@ void TVCh1Create (TVEncoderObj *this, TVChip chip, I2CDevPtr dev)
 
 void TVCh1SetPort (TVEncoderObj *this, int port)
 {
-  DPRINTF ("tv port ch %08X\n", port);
+  RAISE (MSG_DEBUG, "tv port ch %08X", port);
   if ((port & PORT_FORMAT_MASK) != PORT_FORMAT_RGB) {
-    ERROR ("TVCh1SetPort: Only RGB multiplex format supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVCh1SetPort: Only RGB multiplex format supported.");
   }
   if ((port & PORT_BLANK_POLARITY) != PORT_BLANK_LOW) {
-    ERROR ("TVCh1SetPort: Only active low blank supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVCh1SetPort: Only active low blank supported.");
   }
   if ((port & PORT_BLANK_DIR) != PORT_BLANK_OUT) {
-    ERROR ("TVCh1SetPort: Only blank out supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVCh1SetPort: Only blank out supported.");
   }
   if ((port & PORT_BLANK_MODE) != PORT_BLANK_REGION) {
-    ERROR ("TVCh1SetPort: Only blank region supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVCh1SetPort: Only blank region supported.");
   }
   this->hwstate = 0x00
     | ((port & PORT_PCLK_POLARITY)  ? 0 : SetBit(4))
@@ -366,10 +359,10 @@ void TVCh1SetPort (TVEncoderObj *this, int port)
 
 void TVCh1GetPort (TVEncoderObj *this, int *port)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte res;
 
-  DPRINTF ("tv get port ch\n");
+  RAISE (MSG_DEBUG, "tv get port ch");
   *port = PORT_FORMAT_RGB | PORT_BLANK_LOW | PORT_BLANK_OUT | 
           PORT_BLANK_REGION;
   /* FIXME handle at least some of those */
@@ -384,7 +377,7 @@ void TVCh1GetPort (TVEncoderObj *this, int *port)
 
 void TVCh1InitRegs (TVEncoderObj *this, int port)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
 
   TVCh1SetPort (this, port);
 
@@ -407,11 +400,11 @@ void TVCh1InitRegs (TVEncoderObj *this, int port)
 
 void TVCh1SetRegs (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte *m;
   int i;
 
-  DPRINTF ("tv regs ch\n");
+  RAISE (MSG_DEBUG, "tv regs ch");
 #ifdef FAKE_I2C
   FPRINTF ("\ni2c [");
 #endif
@@ -508,7 +501,7 @@ void TVCh1SetRegs (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 
 void TVCh1GetRegs (TVEncoderObj *this, TVEncoderRegs *r)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte regs[0x40];
   int i;
 
@@ -560,10 +553,10 @@ void TVCh1GetRegs (TVEncoderObj *this, TVEncoderRegs *r)
  
 void TVCh1SetState (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte *m;
 
-  DPRINTF ("tv state ch %i\n", state);
+  RAISE (MSG_DEBUG, "tv state ch %i", state);
   tvState = state;
   m = TVCh1MacroRegs (r);
   switch (state)
@@ -584,7 +577,7 @@ void TVCh1SetState (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 
 TVConnect TVCh1GetConnect (TVEncoderObj *this)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte power, connect;
 
   TVReadBus  (dev, 0x80|0x0e, &power);
@@ -602,7 +595,8 @@ TVConnect TVCh1GetConnect (TVEncoderObj *this)
       case 0x0a: return CONNECT_CONVERT; break;
       case 0x0e: return CONNECT_NONE; break;
       default: 
-	ErrorF("Strange Chrontel connection status %02x\n", connect & 0x0e); 
+	RAISE (MSG_WARNING, "Strange Chrontel connection status %02x", 
+	       connect & 0x0e); 
       case 0x00: return CONNECT_BOTH; break;
     }
   } else {
@@ -612,7 +606,7 @@ TVConnect TVCh1GetConnect (TVEncoderObj *this)
 
 long TVCh1GetStatus (TVEncoderObj *this, int index)
 { 
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   long civ;
   I2CByte result;
 
@@ -653,7 +647,7 @@ char *TVDetectChrontel1 (I2CDevPtr dev, TVChip *encoder)
 #endif  
   tvBusOk = TRUE;  
   TVReadBus (dev, 0x80|0x25, &result);
-  DPRINTF ("ch version %02x (%s)\n", result, tvBusOk ? "ok" : "err");
+  RAISE (MSG_DEBUG, "ch version %02x (%s)", result, tvBusOk ? "ok" : "err");
   if (!tvBusOk) return NULL;
   *encoder = TV_CHRONTEL_MODEL1;
   switch (result) 
@@ -684,7 +678,7 @@ char *TVDetectChrontel1 (I2CDevPtr dev, TVChip *encoder)
 /* Clock range is a safe guess based on available modes */
 
 TVEncoderObj tvCh1Template = {
-  chip: TV_CHRONTEL_MODEL1, dev: NULL, minClock: 20000, maxClock: 40000, 
+  type: TV_CHRONTEL_MODEL1, ctrl: NULL, minClock: 20000, maxClock: 40000, 
   Create:     TVCh1Create,
   InitRegs:   TVCh1InitRegs, 
   SetRegs:    TVCh1SetRegs, 
