@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: tv_chip.h,v 1.36 2003/06/27 16:02:01 dthierbach Exp $
  *
  * Contents:
  *
@@ -89,6 +89,9 @@ typedef enum {
   TV_PHILIPS_7105    = 0x3201,
   TV_PHILIPS_7108A   = 0x3202, 
   TV_PHILIPS_7109A   = 0x3203,
+  TV_NVIDIA	     = 0x4000,
+  TV_NVIDIA_A	     = 0x4001,
+  TV_NVIDIA_B	     = 0x4002,
 } TVChip;
 
 #define CARD_SHIFT 0       /* must fit into TV_ENCODER mask */
@@ -458,6 +461,66 @@ typedef struct {
   int flags3;
 } TVPh2Regs;
 
+/* -------- NVidia internal -------- */
+
+#define NX_FLAG1_DACA	(1 << 0)
+#define NX_FLAG1_DACB	(1 << 1)
+#define NX_FLAG1_DACC	(1 << 2)
+
+#define NX_FLAG1_COMPOSITE (NX_FLAG1_DACB | NX_FLAG1_DACC)
+#define NX_FLAG1_SVIDEO    (NX_FLAG1_DACA | NX_FLAG1_DACB)
+#define NX_FLAG1_BOTH      (NX_FLAG1_DACA | NX_FLAG1_DACB | NX_FLAG1_DACC)
+
+#define NX_FLAG_CONN_3	(1 << 0)
+#define NX_FLAG_CONN_5	(1 << 1)
+#define NX_FLAG_SYS_0	(1 << 2)
+#define NX_FLAG_SYS_3	(1 << 3)
+#define NX_FLAG_SYS_6	(1 << 4)
+
+/* "Indirect" registers -- dependent on TV System */
+typedef struct {
+  unsigned long fsub; 
+  int nxreg04;
+  int nxreg08;
+  int nxreg09;
+  int nxreg0a;
+  int nxreg0b;
+  int nxreg0c;
+  int nxreg0e;
+  int nxreg10;
+  int nxreg17;
+  int nxreg1e;
+  int nxreg20;
+  int nxreg22;
+  int nxreg29;
+  int nxreg2c;
+  int nxreg31;  /* active_h * 2 */
+  int nxreg35;
+  int nxreg3c;
+  int nxsys12;  /* reg34/sysflags */
+  int flags;    /* reg34/sysflags, reg07/connflags */
+} TVNxIndRegs;  
+
+/* Image (resolution and size) dependent regs */
+typedef struct {
+  int scaler_h;
+  int scaler_v;
+  int overscan;
+  int vip1;
+  int vip2;
+} TVNxImgRegs;
+
+typedef struct {
+  TVNxIndRegs ind;
+  TVNxImgRegs img;
+  int filter;
+  int filt_x1[4][7];
+  int filt_x2[4][7];
+  int filt_y [4][7];
+  int muxa, muxb, muxc;
+  int flags1;
+} TVNxRegs;
+
 /* -------- CRT -------- */
 
 /* Flags for devices */
@@ -466,6 +529,7 @@ typedef struct {
 #define DEV_TELEVISION		(1 << 1)
 #define DEV_FLATPANEL		(1 << 2)
 #define DEV_OVERLAY		(1 << 3)
+#define DEV_INTERNAL		(1 << 4)
 
 /* Flags that describe the mode, capabilities and defaults 
    (in TVMode.descFlags). TODO: Get rid of Dualview, etc. */
@@ -600,6 +664,7 @@ typedef union {
   TVChRegs ch;
   TVPh1Regs ph1;
   TVPh2Regs ph2;
+  TVNxRegs nx;
 } TVEncoderRegs;
 
 typedef union {
