@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: tv_cx.c,v 1.22 2004/01/27 17:05:25 dthierbach Exp $
  *
  * Contents:
  *
@@ -41,27 +41,28 @@
  *  3a-4   14318_xtal = 0 (default)
  */
 
-void TVCxCreate (TVEncoderObj *this, TVChip chip, I2CDevPtr dev)
+void TVCxCreate (TVEncoderObj *this, TVChip chip_type, void *ctrl)
 {
+  register I2CDevPtr dev = (I2CDevPtr) ctrl;
   I2CByte result;
 
-  DPRINTF ("tv create cx\n");
-  this->chip = chip;
-  this->dev = dev;
+  RAISE (MSG_DEBUG, "tv create cx");
+  this->type = chip_type;
+  this->ctrl = ctrl;
   tvBusOk = TRUE;
-  TVReadBus (this->dev, 0x32, &result);
+  TVReadBus (dev, 0x32, &result);
   this->hwconfig = SetBitField(result,6:5,6:5);
-  TVReadBus (this->dev, 0x3a, &result);
+  TVReadBus (dev, 0x3a, &result);
   this->hwconfig |= SetBitField(result,4:4,4:4);
   if (!tvBusOk) {
     fprintf (stderr, "Critical: Error reading Conexant hardware config\n");
     exit (1);
   }
   if (GetBF(this->hwconfig,6:5) != 0) {
-    ErrorF ("Warning: Conexant in low voltage configuration!\n");
+    RAISE (MSG_WARNING, "Conexant in low voltage configuration.");
   }
   if (GetBF(this->hwconfig,4:4) != 0) {
-    ErrorF ("Warning: 14.318 MHz crystal detected; timings will be wrong.\n");
+    RAISE (MSG_ERROR, "14.318 MHz crystal detected; timings will be wrong.");
   }
 }
 
@@ -129,13 +130,12 @@ void TVCxCreate (TVEncoderObj *this, TVChip chip, I2CDevPtr dev)
 
 void TVCxSetPort (TVEncoderObj *this, int port)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
 
-  DPRINTF ("tv port cx %08X\n", port);
+  RAISE (MSG_DEBUG, "tv port cx %08X", port);
 
   if ((port & PORT_PCLK_MODE) != PORT_PCLK_MASTER) {
-    ERROR ("TVCxSetPort: Only clock master mode supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVCxSetPort: Only clock master mode supported.");
   }
   if ((port & PORT_SYNC_DIR) == PORT_SYNC_SLAVE) {
     this->hwstate |= SetBit (5);
@@ -160,10 +160,10 @@ void TVCxSetPort (TVEncoderObj *this, int port)
 
 void TVCxGetPort (TVEncoderObj *this, int *port)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte res;
 
-  DPRINTF ("tv get port cx\n");
+  RAISE (MSG_DEBUG, "tv get port cx");
   *port = PORT_PCLK_MASTER | PORT_PCLK_HIGH;
   TVReadBus (dev, 0xba, &res);
   *port |= (GetBit(res,5) ? PORT_SYNC_SLAVE : PORT_SYNC_MASTER);
@@ -182,9 +182,9 @@ void TVCxGetPort (TVEncoderObj *this, int *port)
 
 void TVCxInitRegs (TVEncoderObj *this, int port)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
 
-  DPRINTF ("tv init cx\n");
+  RAISE (MSG_DEBUG, "tv init cx");
   TVCxSetPort (this, port);
   TVWriteBus (dev, 0xba, 0x17);
   TVWriteBus (dev, 0xc4, 0x00);
@@ -199,9 +199,9 @@ void TVCxInitRegs (TVEncoderObj *this, int port)
 
 void TVCxSetRegs (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
 
-  DPRINTF ("tv set cx\n");
+  RAISE (MSG_DEBUG, "tv set cx");
   TVBtSetRegs (this, r, state);
   TVWriteBus (dev, 0x34, SetBitFlag(r->cx.flags5,CX_FLAG5_ADPT_FF,7)
                        | SetBitField(r->cx.c_altff,1:0,4:3)
@@ -256,11 +256,11 @@ void TVCxSetRegs (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 
 void TVCxGetRegs (TVEncoderObj *this, TVEncoderRegs *r)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte regs[0x100];
   int i;
 
-  DPRINTF ("tv get cx\n");
+  RAISE (MSG_DEBUG, "tv get cx");
   for (i = 0x2e; i < 0x100; i += 2) TVReadBus (dev, i, &regs[i]);
   r->cx.bt.h_active     = Set8Bits(regs[0x78])
                         | SetBitField(regs[0x86],6:4,10:8); /* BT: 5:4, 9:8 */
@@ -381,29 +381,30 @@ void TVCxGetRegs (TVEncoderObj *this, TVEncoderRegs *r)
 
 void TVCxSetState (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 {
-  DPRINTF ("tv state cx\n");
+  RAISE (MSG_DEBUG, "tv state cx");
   TVBtSetState (this, r, state);
   /* FIXME: 6c en_reg_rd=1 6c-5 ffcbar */
 }
 
 long TVCxGetStatus (TVEncoderObj *this, int index)
 { 
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte result;
 
-  TVReadBus (this->dev, 0x28, &result); 
+  TVReadBus (dev, 0x28, &result); 
   return result | ((tvState == TV_ON) ? 0x100 : 0);
 }
 
 TVConnect TVCxGetConnect (TVEncoderObj *this)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte result;
  
   tvBusOk = TRUE;  
   TVWriteBus (dev, 0xba, 0x40); 
   xf86usleep(50000); /* BT871 doc says: 20ms - 50ms */
   TVReadBus (dev, 0x06, &result); 
-  DPRINTF ("cx conn %02x (%s)\n", result, tvBusOk ? "ok" : "err");
+  RAISE (MSG_DEBUG, "cx conn %02x (%s)", result, tvBusOk ? "ok" : "err");
   TVWriteBus (dev, 0xba, 0x00); 
   /* FIXME. (But next SetTVState will write correct values.) */
   if (tvBusOk) {
@@ -413,7 +414,8 @@ TVConnect TVCxGetConnect (TVEncoderObj *this)
       case 0x20 : return CONNECT_CONVERT; break;
       case 0x40 : return CONNECT_CONVERT; break;
       default: 
-	ErrorF("Strange Conexant connection status %02x\n", result & 0xe0); 
+	RAISE (MSG_WARNING, "Strange Conexant connection status %02x", 
+	       result & 0xe0); 
       case 0xe0 : return CONNECT_BOTH; break;
     }
   } else {
@@ -446,16 +448,8 @@ char *TVDetectConexant (I2CDevPtr dev, TVChip *encoder)
   tvBusOk = TRUE;  
   TVWriteBus (dev, 0x6c, 0x46); /* enable reg rd */
 
-#if 0 //// FIXME remove
-  TVWriteBus (dev, 0x28, 0x55); /* FIXME this will change CNTRL2 in TW99 */
-  TVReadBus (dev, 0x28, &result); 
-  DPRINTF ("cx check (%s)\n", tvBusOk ? "ok" : "err");
-  if (!tvBusOk || result != 0x55) return NULL;
-  TVWriteBus (dev, 0x28, 0x00); 
-#endif
-
   TVReadBus (dev, 0x00, &result);
-  DPRINTF ("cx check %02x (%s)\n", result, tvBusOk ? "ok" : "err");
+  RAISE (MSG_DEBUG, "cx check %02x (%s)", result, tvBusOk ? "ok" : "err");
   if (!tvBusOk) return NULL;
   *encoder = TV_CONEXANT;
   switch (result & 0xe0) {
@@ -480,7 +474,7 @@ char *TVDetectConexant (I2CDevPtr dev, TVChip *encoder)
 /* Apparently, in 3:2 mode it can be larger. Maybe in async mode, too? */
 
 TVEncoderObj tvCxTemplate = {
-  chip: TV_CONEXANT, dev: NULL, minClock: 0, maxClock: 53333, 
+  type: TV_CONEXANT, ctrl: NULL, minClock: 0, maxClock: 53333, 
   Create:     TVCxCreate,
   InitRegs:   TVCxInitRegs, 
   SetRegs:    TVCxSetRegs, 
