@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id$
+ * $Id: tv_i810.c,v 1.24 2004/01/28 17:01:01 dthierbach Exp $
  *
  * Contents:
  *
@@ -43,6 +43,8 @@
 #define I810_GPIOA	0x05010
 #define I810_GPIOB	0x05014
 #define I830_GPIOC	0x05018  /* M_I2C */
+#define I845_GPIOD	0x0501c  /* Not sure if it is used */
+#define I845_GPIOE	0x05020  /*  */
 
 #define I810_DCLK(i)	(0x06000 + (i)*4)
 #define I810_DCLK_0D	0x06000
@@ -60,6 +62,39 @@
 /*  This is an attempt to explain the i830 registers as far as I can 
  *  guess them. Some of it may be severely wrong.
  *
+ *  The i845 seems to be similar enough. DVO port A is gone, there is
+ *  only one pipe, and plane B seems to be gone as well. Available planes
+ *  are primary, cursor, and overlay. There is now a video engine which
+ *  extends/replaces the motion compensation block.
+ *
+ *  The i845 can support synchronous display on two devices, unless one of
+ *  them is a TV (because of the low clock?), or both are digital devices.
+ *  The overlay engine can also use an external capture device as source.
+ *
+ *  The i865 also has only one pipe, one plane, and no DVO port A.
+ *  DVO ports are each capable of driving a 165 MHz pixel clock. Each
+ *  port is capable of driving a digital display up to 1600x1200 @
+ *  60Hz. When in dual-channel mode, the GMCH can drive a flat panel
+ *  up to 2048x1536 @ 60Hz or dCRT/HDTV up to 1920x1080 @ 85Hz.
+ *
+ *  The multiplexed digital display interface uses the MDVI_CLK and
+ *  MDVI_DATA signals to interrogate the panel. The i865 supports the
+ *  DDC2B protocol to initiate the transfer of EDID data. The
+ *  multiplexed digital display interface uses the M_I2C bus to
+ *  interrogate the external transmitter. A third set of signals
+ *  (MDDC_CLK and MDDC_DATA) is available for a variety of
+ *  purposes. They can be used as a second DDC pair when two TMDS
+ *  transmitters are used, or as a second I2C pair if there are
+ *  multiple devices (e.g., PROM and DVO device) that need I2C and
+ *  there is a speed or addressing conflict.  
+ *
+ *  Since the i865 has several display ports available for its single
+ *  pipe, it can support a synchronous display on 2 or 3 displays,
+ *  unless one of the displays is a TV. No synchronous display is
+ *  available when a TV is in use. The i865 cannot drive multiple
+ *  displays concurrently (different data or timings). In addition,
+ *  the i865 cannot operate in parallel with an external AGP device.
+ *
  *  1. Architecture overview
  *
  *  The i830 emulates two PCI cards. The register of the PCI cards seem
@@ -73,17 +108,40 @@
  *  Some registers may be double buffered, and may need a write to another
  *  register to commit.
  *
+ *  Here are some definitions from the i845G data sheet:
+ * 
+ *    A PLANE consists of a rectangular shaped image that has
+ *    characteristics such as source, size, position, method, and
+ *    format. These planes get attached to source surfaces that are
+ *    rectangular memory surfaces with a similar set of characteristics. 
+ *    They are also associated with a destination pipe.
+ *
+ *    A PIPE consists of a set of combined planes and a timing generator. 
+ *
+ *
  *  2. I2C control
  *
- *  There are 5 I2C ports: DDC1, I2C, DDC2, M_DDC, M_I2C. The DDC ports
- *  are used for EDID display information transfer from the monitor
- *  (typically at address 0xA0). On the I2C port (nvtv bus 0) there is a 
- *  device at 0x04, maybe a flat panel controller. The TV encoder chip
- *  is on M_I2C (bus 1). On the i810, the TV encoder is on bus 0.
+ *  Chipset  I2C lines                       Known TV
+ *  i810     DDC; LTV                        GPIOB ('TV0') = LTV
+ *  i830     DDC1, I2C, DDC2; M_DDC, M_I2C   GPIOC ('TV1') = M_I2C
+ *  i845     DDC_A; MI2C, MDVI, MDDC         GPIOE ('TV2')
+ *  i865     DDC_A; MI2C, MDVI, MDDC         GPIOE ('TV2') = M_I2C 
+ *
+ *  On the i830, DVO port A uses DDC2 or I2C, DVO ports B/C use M_I2C
+ *  or M_DDC. The DDC ports are used for EDID display information
+ *  transfer from the monitor (typically at address 0xA0) or flat
+ *  panel. On the I2C port (nvtv bus 0) there is a device at 0x04,
+ *  probably a flat panel controller. 
+ *
+ *  The i865 implements a hardware GMBus controller that can be used
+ *  to control these signals.  This allows higher speed transactions
+ *  (up to 400 kHz) on theses lines than was allowed with previous
+ *  software centric `bit-bashing' techniques.
  *
  *  0x05010 GPIOA General purpose I/O A, one of the DDC ports.
  *  0x05014 GPIOB General purpose I/O B, I2C port.
  *  0x05018 GPIOC General purpose I/O C, M_I2C port.
+ *  0x05020 GPIOE General purpose I/O E (for i845)
  *    31:13 reserved
  *       12 GPIO1 data in
  *       11 GPIO1 data value
@@ -107,6 +165,14 @@
  *  A primary display plane
  *  B 'sprite' plane (for specific laptop display information?)
  *  C overlay plane (can also be directly sent in YUV format to DVO)
+ *
+ *  The datasheet diagram shows overlay, sprite, two cursor, and primary and
+ *  secondary display units. Maybe those correspond to the planes. The
+ *  datasheet also meantions a popup plane for mobile applications.
+ *  The single overlay plane can only displayed on "single-pipe simultanious 
+ *  displays" for multi-monitors. (For the i810, 0x70008 PIXCONF
+ *  has some of the functions used here.) For the i865, these registers
+ *  seem to be somewhere else.
  * 
  *  0x70180 DSPACTRL Display plane A control
  *  0x71180 DSPBCTRL Display plane B control
@@ -163,7 +229,7 @@
  *
  *  0x60018 LCDTV_C LCD/TV-Out control register
  *
- *  0x70008 PIPEACONF
+ *  0x70008 PIPEACONF (PIXCONF for i810)
  *       31 enable (1=enabled)
  *       30 width (1=double, 0=single)
  *    29:27 reserved
@@ -200,18 +266,18 @@
  *
  *  0x06014 DPLL_A          
  *  0x06018 DPLL_B    
- *    31:28 UNKNOWN (M or N ?)
+ *    31:28 UNKNOWN (M or N ?) (=0xd for TFT and TV; 8,9 for mon)
  *    27:24 reserved
  *       23 post divisor (loop divide flag?)
  *       22 reserved
  *    21:16 post divisor (P ?)
  *       15 reserved
- *    14:13 UNKNOWN (=0 for LCD, =2 for TV ?)
+ *    14:13 UNKNOWN (=0 for LCD/mon, =2 for TV ?)
  *    12: 0 reserved
  *
  *  0x61100 ADPA 
  *       31 DAC enable (1=enabled) (=monitor enabled)
- *       30 Pipe select (0=A, 1=B)
+ *       30 Pipe select (0=A, 1=B) [maybe doesn't work]
  *    29:16 reserved
  *       15 Use_VGA_HVPolarity (0=Sets HVPolarity)
  *    14:12 UNKNOWN
@@ -236,10 +302,13 @@
  *       14 UNKNOWN, set for LFP1
  *        8 UNKNOWN, clear LFP, set for TV
  *        7 UNKNOWN, set for LFP1 and TV
+ *        6 UNKNOWN, clear for LFP, set for TV (external clock?)
  *        4 UNKNOWN, set for LFP12 (VSync active high?) clear for TV
  *        3 UNKNOWN, set for LFP12 (HSync active high?) clear for TV
  *        2 UNKNOWN, set for LFP12 TV
  *          			
+ *  0x61180 LVDS meaning unknown (not needed ?) (used in i855 instead of DVO)
+ *
  * Here LFPx refer to flat panel dumps. [?] Means a guess.
  *    
  *  0x61124 DVOA_SRCDIM
@@ -247,9 +316,10 @@
  *  0x61164 DVOC_SRCDIM
  *    23:12 (not needed ?)
  *    11: 0 (not needed ?)
- *          			    
- *  0x61180 LVDS meaning unknown (not needed ?)
  *
+ *  FIXME: What about 0x61104 ADPA_SRCDIM? and 0x61184 LVDS_SRCDIM?
+ *  Maybe used for centering?
+ *          			    
  *  6. Other registers
  *
  *  0x020cc UNKNOWN = 000c000c / 0000004c
@@ -267,7 +337,18 @@
  *    11: 8 Burst A
  *     7: 6 reserved
  *     5: 0 Watermark A
- *   */
+ *
+ *  Software Flags (= scratch registers):
+ *
+ *  0x71410 SWF0 
+ *  0x71414 SWF1 (amount of video memory available)
+ *  0x71418 SWF2 
+ *  0x7141c SWF3 
+ *  0x71420 SWF4 
+ *  0x71424 SWF5 
+ *  0x71428 SWF6 
+ * 
+ */
 
 /* -------- I2C Driver Routines -------- */
 
@@ -276,53 +357,65 @@
 #define I2C_SCL_IN    (1 << 4)
 #define I2C_SDA_IN    (1 << 12)
 
-static void I810TvI2CGetBits(I2CBusPtr b, int *clock, int *data)
+typedef struct {
+  I810Ptr pI810;
+  CARD32 port;
+} I810TvI2CRec, *I810TvI2CPtr;
+
+static void 
+I810TvI2CGetBits(I2CBusPtr b, int *clock, int *data)
 {
-  I810Ptr pI810 = I810PTR(xf86Screens[b->scrnIndex]);
+  I810TvI2CPtr priv = (I810TvI2CPtr) b->DriverPrivate.ptr;
+  I810Ptr pI810 = priv->pI810;
   register CARD32 val;
 
-  I810_WR32(pI810, b->DriverPrivate.val, 0x0);
-  val = I810_RD32(pI810, b->DriverPrivate.val);
+  I810_WR32 (pI810, priv->port, 0x0);
+  val = I810_RD32 (pI810, priv->port);
 
   *clock = (val & I2C_SCL_IN) != 0;
   *data  = (val & I2C_SDA_IN) != 0;
-
-  DEBUG(ErrorF("I810TvI2CGetBits(%p) val=0x%x, returns clock %d, data %d\n",
-	       b, val, *clock, *data));
 }
 
-static void I810TvI2CPutBits(I2CBusPtr b, int clock, int data)
+static void 
+I810TvI2CPutBits(I2CBusPtr b, int clock, int data)
 {
-  I810Ptr pI810 = I810PTR(xf86Screens[b->scrnIndex]);
+  I810TvI2CPtr priv = (I810TvI2CPtr) b->DriverPrivate.ptr;
+  I810Ptr pI810 = priv->pI810;
 
   if (clock) {
-    I810_WR32(pI810, b->DriverPrivate.val, 0x5);
-    I810_WR32(pI810, b->DriverPrivate.val, 0x4);
+    I810_WR32 (pI810, priv->port, 0x5);
+    I810_WR32 (pI810, priv->port, 0x4);
   } else { 
-    I810_WR32(pI810, b->DriverPrivate.val, 0x7);
-    I810_WR32(pI810, b->DriverPrivate.val, 0x6);
+    I810_WR32 (pI810, priv->port, 0x7);
+    I810_WR32 (pI810, priv->port, 0x6);
   }
   
   if (data) {
-    I810_WR32(pI810, b->DriverPrivate.val, 0x500);
-    I810_WR32(pI810, b->DriverPrivate.val, 0x400);
+    I810_WR32 (pI810, priv->port, 0x500);
+    I810_WR32 (pI810, priv->port, 0x400);
   } else { 
-    I810_WR32(pI810, b->DriverPrivate.val, 0x700);
-    I810_WR32(pI810, b->DriverPrivate.val, 0x600);
+    I810_WR32 (pI810, priv->port, 0x700);
+    I810_WR32 (pI810, priv->port, 0x600);
   }
-  
-  DEBUG(ErrorF("I810TvI2CPutBits(%p, %d, %d) val=0x%x\n", 
-	       b, clock, data, val));
 }
 
-I2CBusPtr I810TvI2CInit (ScrnInfoPtr pScrn, char *name, int private)
+I2CBusPtr 
+I810TvBusCreate (I810Ptr pI810, char *name, CARD32 port)
 {
   I2CBusPtr I2CPtr;
+  I810TvI2CPtr priv;
 
   I2CPtr = xf86CreateI2CBusRec();
+  priv = xcalloc (1, sizeof (I810TvI2CRec));
+  if (priv) {
+  }
+  if (priv) {
+    priv->pI810 = pI810;
+    priv->port = port;
+  }
   if (I2CPtr) {
     I2CPtr->BusName    = name;
-    I2CPtr->scrnIndex  = pScrn->scrnIndex;
+    I2CPtr->scrnIndex  = -1;
     I2CPtr->I2CPutBits = I810TvI2CPutBits;
     I2CPtr->I2CGetBits = I810TvI2CGetBits;
     /* Not sure about the delays. Matthew's code has HOLD_TIME = 100, and
@@ -337,73 +430,94 @@ I2CBusPtr I810TvI2CInit (ScrnInfoPtr pScrn, char *name, int private)
     I2CPtr->AcknTimeout  = 500; /* usec, default: 5 */
     I2CPtr->StartTimeout = 500; /* usec, default: 5 */
     I2CPtr->RiseFallTime = 100; /* usec, default: 2 */
-    I2CPtr->DriverPrivate.val = private; /* port addr */
+    I2CPtr->DriverPrivate.ptr = priv; 
   }
-  return I2CPtr;
+  if (I2CPtr && priv && xf86I2CBusInit (I2CPtr)) {
+    return I2CPtr;
+  }
+  if (I2CPtr && priv) {
+    RAISE (MSG_ERROR, "Bus %s is not unique.", name);
+  } else {
+    RAISE (MSG_ERROR, "Out of memory.");
+  }
+  TVDestroyBus (I2CPtr);
+  return NULL;
 }
 
-Bool I810TvBusInit (ScrnInfoPtr pScrn)
+Bool 
+I810TvBusInit (I810Ptr pI810)
 {
-  Bool ok;
-  I810Ptr pI810 = I810PTR(pScrn);
-  int bus;
-
   pI810->TvChain = NULL;
   pI810->TvDev = NULL;
   /* Must be named TV0, TV1, etc.; otherwise I2C_ID won't work */
-  if (pI810->arch.major == 0x30) {
-    pI810->TvMaxBus = 2;
-    pI810->TvBusses[0] = I810TvI2CInit (pScrn, "TV0", I810_GPIOB);
-    pI810->TvBusses[1] = I810TvI2CInit (pScrn, "TV1", I830_GPIOC);
-  } else {
-    pI810->TvMaxBus = 1;
-    pI810->TvBusses[0] = I810TvI2CInit (pScrn, "TV0", I810_GPIOB);
+  switch (pI810->arch.major) {
+    case 0x65: /* i865 */
+    case 0x45: /* i845 FIXME include GPIOD ??? */
+      pI810->TvMaxBus = 3;
+      pI810->TvBusses[0] = I810TvBusCreate (pI810, "TV0", I810_GPIOB);
+      pI810->TvBusses[1] = I810TvBusCreate (pI810, "TV1", I830_GPIOC);
+      pI810->TvBusses[2] = I810TvBusCreate (pI810, "TV2", I845_GPIOE);
+      return (pI810->TvBusses[0] && pI810->TvBusses[1] && pI810->TvBusses[2]);
+    case 0x30: /* i830 */
+      pI810->TvMaxBus = 2;
+      pI810->TvBusses[0] = I810TvBusCreate (pI810, "TV0", I810_GPIOB);
+      pI810->TvBusses[1] = I810TvBusCreate (pI810, "TV1", I830_GPIOC);
+      return (pI810->TvBusses[0] && pI810->TvBusses[1]);
+    case 0x10:
+    default:
+      pI810->TvMaxBus = 1;
+      pI810->TvBusses[0] = I810TvBusCreate (pI810, "TV0", I810_GPIOB);
+      return (pI810->TvBusses[0] != NULL);
   }
-  ok = TRUE;
-  for (bus = 0; bus < pI810->TvMaxBus; bus++) {
-    if (!xf86I2CBusInit(pI810->TvBusses[bus])) {
-      ErrorF ("Cannot init Bus %i\n", bus);
-      ok = FALSE;
-    }
-  }
-  return ok;
 }
 
 /* -------- CRT -------- */
 
-void I810SetTvRegs (I810Ptr pI810, int head, TVI810Regs *r)
+void 
+I810SetTvRegs (I810Ptr pI810, int head, TVI810Regs *r)
 {
-  I810_H_WR32 (pI810, head, 0x60000, SetBitField (r->tvHDisplay,10:0,10:0) 
-                                   | SetBitField (r->tvHTotal,11:0,27:16));
-  I810_H_WR32 (pI810, head, 0x60004, SetBitField (r->tvHBlankStart,11:0,11:0) 
-                                   | SetBitField (r->tvHBlankEnd,11:0,27:16));
-  I810_H_WR32 (pI810, head, 0x60008, SetBitField (r->tvHSyncStart,11:0,11:0)
-                                   | SetBitField (r->tvHSyncEnd,11:0,27:16));
-  I810_H_WR32 (pI810, head, 0x6000C, SetBitField (r->tvVDisplay,10:0,10:0)
-                                   | SetBitField (r->tvVTotal,11:0,27:16));
-  I810_H_WR32 (pI810, head, 0x60010, SetBitField (r->tvVBlankStart,11:0,11:0)
-                                   | SetBitField (r->tvVBlankEnd,11:0,27:16));
-  I810_H_WR32 (pI810, head, 0x60014, SetBitField (r->tvVSyncStart,11:0,11:0)
-                                   | SetBitField (r->tvVSyncEnd,11:0,27:16));
+  I810_H_WR32 (pI810,head,0x60000, SetBitField (r->tvHDisplay-1,10:0,10:0) 
+                                 | SetBitField (r->tvHTotal-1,11:0,27:16));
+  I810_H_WR32 (pI810,head,0x60004, SetBitField (r->tvHBlankStart-1,11:0,11:0) 
+                                 | SetBitField (r->tvHBlankEnd-1,11:0,27:16));
+  I810_H_WR32 (pI810,head,0x60008, SetBitField (r->tvHSyncStart-1,11:0,11:0)
+                                 | SetBitField (r->tvHSyncEnd-1,11:0,27:16));
+  I810_H_WR32 (pI810,head,0x6000C, SetBitField (r->tvVDisplay-1,10:0,10:0)
+                                 | SetBitField (r->tvVTotal-1,11:0,27:16));
+  I810_H_WR32 (pI810,head,0x60010, SetBitField (r->tvVBlankStart-1,11:0,11:0)
+                                 | SetBitField (r->tvVBlankEnd-1,11:0,27:16));
+  I810_H_WR32 (pI810,head,0x60014, SetBitField (r->tvVSyncStart-1,11:0,11:0)
+                                 | SetBitField (r->tvVSyncEnd-1,11:0,27:16));
 }
 
-void I810GetTvRegs (I810Ptr pI810, int head, TVI810Regs *r)
+void 
+I830SetTvRegs (I810Ptr pI810, int head, TVI810Regs *r)
 {
-  r->tvHDisplay    = SetBitField (I810_H_RD32 (pI810,head,0x60000),10: 0,10:0);
-  r->tvHTotal      = SetBitField (I810_H_RD32 (pI810,head,0x60000),27:16,11:0);
-  r->tvHBlankStart = SetBitField (I810_H_RD32 (pI810,head,0x60004),11: 0,11:0);
-  r->tvHBlankEnd   = SetBitField (I810_H_RD32 (pI810,head,0x60004),27:16,11:0);
-  r->tvHSyncStart  = SetBitField (I810_H_RD32 (pI810,head,0x60008),11: 0,11:0);
-  r->tvHSyncEnd    = SetBitField (I810_H_RD32 (pI810,head,0x60008),27:16,11:0);
-  r->tvVDisplay    = SetBitField (I810_H_RD32 (pI810,head,0x6000C),10: 0,10:0);
-  r->tvVTotal      = SetBitField (I810_H_RD32 (pI810,head,0x6000C),27:16,11:0);
-  r->tvVBlankStart = SetBitField (I810_H_RD32 (pI810,head,0x60010),11: 0,11:0);
-  r->tvVBlankEnd   = SetBitField (I810_H_RD32 (pI810,head,0x60010),27:16,11:0);
-  r->tvVSyncStart  = SetBitField (I810_H_RD32 (pI810,head,0x60014),11: 0,11:0);
-  r->tvVSyncEnd    = SetBitField (I810_H_RD32 (pI810,head,0x60014),27:16,11:0);
+  I810SetTvRegs (pI810, head, r);
+  I810_H_WR32 (pI810, head, 0x6001c, SetBitField (r->tvVDisplay-1,10:0,10:0) 
+                                   | SetBitField (r->tvHDisplay-1,10:0,26:16));
+
 }
 
-void I810SetClock (I810Ptr pI810, int index, int m, int n, int p, int vco)
+void 
+I810GetTvRegs (I810Ptr pI810, int head, TVI810Regs *r)
+{
+  r->tvHDisplay    = SetBitField(I810_H_RD32(pI810,head,0x60000),10: 0,10:0)+1;
+  r->tvHTotal      = SetBitField(I810_H_RD32(pI810,head,0x60000),27:16,11:0)+1;
+  r->tvHBlankStart = SetBitField(I810_H_RD32(pI810,head,0x60004),11: 0,11:0)+1;
+  r->tvHBlankEnd   = SetBitField(I810_H_RD32(pI810,head,0x60004),27:16,11:0)+1;
+  r->tvHSyncStart  = SetBitField(I810_H_RD32(pI810,head,0x60008),11: 0,11:0)+1;
+  r->tvHSyncEnd    = SetBitField(I810_H_RD32(pI810,head,0x60008),27:16,11:0)+1;
+  r->tvVDisplay    = SetBitField(I810_H_RD32(pI810,head,0x6000C),10: 0,10:0)+1;
+  r->tvVTotal      = SetBitField(I810_H_RD32(pI810,head,0x6000C),27:16,11:0)+1;
+  r->tvVBlankStart = SetBitField(I810_H_RD32(pI810,head,0x60010),11: 0,11:0)+1;
+  r->tvVBlankEnd   = SetBitField(I810_H_RD32(pI810,head,0x60010),27:16,11:0)+1;
+  r->tvVSyncStart  = SetBitField(I810_H_RD32(pI810,head,0x60014),11: 0,11:0)+1;
+  r->tvVSyncEnd    = SetBitField(I810_H_RD32(pI810,head,0x60014),27:16,11:0)+1;
+}
+
+void 
+I810SetClock (I810Ptr pI810, int index, int m, int n, int p, int vco)
 {
   register CARD32 val;
 
@@ -418,7 +532,8 @@ void I810SetClock (I810Ptr pI810, int index, int m, int n, int p, int vco)
 
 /* -------- Device enable/disable -------- */
 
-int I810GetDevFlags (I810Ptr pI810)
+int 
+I810GetDevFlags (I810Ptr pI810)
 {
   if (GetBit (I810_RD32(pI810, I810_LCDTV_C), 31)) {
     /* FIXME: may also be flat panel */
@@ -428,53 +543,70 @@ int I810GetDevFlags (I810Ptr pI810)
   }
 }
 
+int
+I830GetDevFlags (I810Ptr pI810, int head)
+{
+  static int dev[4] = {DEV_MONITOR, DEV_FLATPANEL, 0, DEV_TELEVISION};
+  int res = 0;
+  int i;
+
+  /* Check ADPA, DVOA and DVOC for Monitor, Flatpanel, and TV */
+  /* FIXME: This assumes a FP on DVOA and a TV on DVOC. Should
+     check clock slave mode or something like this ... */
+  for (i = 0; i < 4; i++) 
+  {
+    register CARD32 tmp = I810_H_RD32 (pI810,head,0x61100 + 0x20 * i);
+
+    if (GetBit (tmp, 31) && GetBit (tmp, 30) == head) res |= dev[i];
+  }
+  return res;
+}
+
+int
+IntelGetDevFlags (I810Ptr pI810, int head)
+{
+  switch (pI810->arch.major) 
+  {
+    case 0x10:
+      return I810GetDevFlags (pI810);
+    case 0x30:
+      return I830GetDevFlags (pI810, head);
+    case 0x45:
+    case 0x65:
+      return I830GetDevFlags (pI810, 0);
+    default:
+      return 0;
+  }
+}
+
 /* -------- Device routines -------- */
 
-void I810SetTvDevice (I810Ptr pI810, I2CChainPtr chain, Bool init)
+void 
+I810SetTvDevice (I810Ptr pI810, I2CChainPtr chain, Bool init)
 {
-  DPRINTF ("i810 set dev %p %i\n", chain, init);
-  TVSetTvEncoder (&pI810->tvEncoder, chain);
+  RAISE (MSG_DEBUG, "i810 set dev %p %i", chain, init);
+  TVSetTvEncoder (&pI810->tvEncoder, chain, NULL);
   tvBusOk = TRUE;
   pI810->tvEncoder.InitRegs (&pI810->tvEncoder, PORT_I810);
 }
 
-/*
- * Free chain and device records of all tv chips
- */
-
-void I810DestroyDevices (I810Ptr pI810)
-{
-  if (!pI810) return;
-  if (pI810->TvChain) TVDestroyChain (pI810->TvChain);
-  pI810->TvChain = NULL;
-  pI810->TvDev = NULL;
-  TVDestroyDevices (pI810->TvBusses, pI810->TvMaxBus);
-}
-
-/*
- * Free chain, device records, and all busses
- */
-
-void I810DestroyBusses (I810Ptr pI810)
-{
-  if (!pI810) return;
-  I810DestroyDevices (pI810);
-  TVDestroyBusses (pI810->TvBusses, pI810->TvMaxBus);
-}
-
-void I810ProbeTvDevices (I810Ptr pI810)
+void 
+I810ProbeTvDevices (I810Ptr pI810)
 {
   if (!pI810) return;
   I810UpdateTvState (pI810);
   tvState = TV_OFF;
-  TVProbeCreateKnown (pI810->TvBusses, pI810->TvMaxBus, &pI810->TvChain); 
+  TVCheckChain (pI810->TvChain);
+  pI810->TvChain = 
+    TVProbeCreateKnown (pI810->TvBusses, pI810->TvMaxBus, NULL); 
   /* FIXME: Is below necessary? */
   pI810->TvDev = pI810->TvChain ? pI810->TvChain->dev : NULL;
 }
 
-/* -------- -------- */
+/* -------- I810 -------- */
 
-void I810UpdateTvState (I810Ptr pI810)
+void 
+I810UpdateTvState (I810Ptr pI810)
 {
 #ifndef FAKE_MMIO
   if (GetBit (I810_RD32(pI810, I810_LCDTV_C), 31)) {
@@ -485,7 +617,8 @@ void I810UpdateTvState (I810Ptr pI810)
 #endif
 }
 
-void I810SetTestImage (I810Ptr pI810, TVEncoderRegs *r)
+void 
+I810SetTestImage (I810Ptr pI810, TVEncoderRegs *r)
 {
   /* FIXME chip registers ... */
   I810UpdateTvState (pI810);
@@ -495,14 +628,16 @@ void I810SetTestImage (I810Ptr pI810, TVEncoderRegs *r)
   }
 }
 
-void I810SetTvMode (I810Ptr pI810, TVRegs *r)
+void 
+I810SetTvMode (I810Ptr pI810, TVRegs *r)
 {
   TVState oldState, newState; /* hack for now, FIXME */
 
+  RAISE (MSG_DEBUG, "i810 set tvmode\n");
   I810UpdateTvState (pI810);
   oldState = tvState;
   newState = (r->devFlags & DEV_TELEVISION) ? TV_ON : TV_OFF;
-  DPRINTF ("i810 set tv mode %i -> %i\n", oldState, newState);
+  RAISE (MSG_DEBUG, "i810 set tv mode %i -> %i", oldState, newState);
 
   if (oldState == TV_ON && newState != TV_ON) 
   {
@@ -527,7 +662,8 @@ void I810SetTvMode (I810Ptr pI810, TVRegs *r)
   tvState = newState;
 }
 
-long I810GetTvStatus (I810Ptr pI810, int index)
+long 
+I810GetTvStatus (I810Ptr pI810, int index)
 {
   long result; 
 
@@ -537,7 +673,8 @@ long I810GetTvStatus (I810Ptr pI810, int index)
   return result;
 }
 
-TVConnect I810GetTvConnect (I810Ptr pI810)
+TVConnect 
+I810GetTvConnect (I810Ptr pI810)
 {
   TVConnect result;
 
@@ -549,6 +686,8 @@ TVConnect I810GetTvConnect (I810Ptr pI810)
 
 #if 0
 
+/* i810 stuff... */
+
 {
   register long l;
 
@@ -577,6 +716,56 @@ TVConnect I810GetTvConnect (I810Ptr pI810)
 
 #endif
 
+/* -------- I830/I845 -------- */
+
+void 
+I830SetTvMode (I810Ptr pI810, TVRegs *r)
+{
+  RAISE (MSG_DEBUG, "i830 set tvmode dev=%i\n", r->devFlags);
+  if (!(r->devFlags & DEV_TELEVISION)) return;
+  
+  /* TODO: fill in slowly, and test */
+
+  pI810->tvEncoder.SetRegs (&pI810->tvEncoder, &r->enc, TV_ON);
+  I830SetTvRegs (pI810, 0, &r->crtc.i810);
+
+  /* Need global var for pipe/plane to use */
+#if 0
+
+  if (newstate != oldstate) {
+  * Deactivate Monitor/FP/TV (old state):
+  *  - disable ADPA/DVOA/DVOC
+  * Setup pipe, including SRC, with I830SetTvRegs
+  * Set encoder if newstate has TV
+  * Activate Monitor/FP/TV (new state): 
+  *  - enable ADPA/DVOA/DVOC, switch to current pipe
+  *  - setup DVO_SRCDIM ???
+  *  - setup DPLL (of pipe)
+  *
+  * Watermarks??
+  }
+
+#endif
+}
+
+void 
+IntelSetTvMode (I810Ptr pI810, TVRegs *r)
+{
+  switch (pI810->arch.major) 
+  {
+    case 0x10:
+      I810SetTvMode (pI810, r); return;
+    case 0x30:
+    case 0x45:
+    case 0x65:
+      I830SetTvMode (pI810, r); return;
+    default:
+      return;
+  }
+}
+
+/* --------  -------- */
+
 /*
 
 06=C0/3F cfrb=1 m/s*=1
@@ -613,8 +802,8 @@ DVOB/C use  M_DDC, M_I2C
 00006040= 00021207 FPA0
 00006044= 00021207 FPA1
 00006014= 808b0000 DPLL_A
-00006048= 00021207 ? FPA2
-0000604c= 00021207 ? FPA3 
+00006048= 00021207 ? FPA2/FPA0_B (for second pipe/plane/whatever)
+0000604c= 00021207 ? FPA3/FPA1_B (for second pipe/plane/whatever)
 00006018= 808b0000 DPLL_B
 00070080= 00000000  
 00070180= 00000000 DSPACTRL (= pipe A)
