@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tv_ph.c,v 1.12 2002/04/18 06:02:42 dthierbach Exp $
+ * $Id: tv_ph1_saa7102.c,v 1.6 2004/01/27 17:05:26 dthierbach Exp $
  *
  * Contents:
  *
@@ -113,7 +113,7 @@ TVPhMacro tvPhPalMacro1= {
 
 TVPhMacro *TVMacroPhRegs (TVEncoderRegs *r)
 {
-  DPRINTF ("tv macro ph %i\n", r->ph1.macro); 
+  RAISE (MSG_DEBUG, "tv macro ph %i", r->ph1.macro); 
   switch (r->ph1.macro) {
     case 0: /* NTSC APS 0 */
       return &tvPhNtscMacro0;
@@ -139,11 +139,11 @@ TVPhMacro *TVMacroPhRegs (TVEncoderRegs *r)
   return NULL;
 }
 
-void TVPhCreate (TVEncoderObj *this, TVChip chip, I2CDevPtr dev)
+void TVPhCreate (TVEncoderObj *this, TVChip chip_type, void *ctrl)
 {
-  DPRINTF ("ph create\n");
-  this->chip = chip;
-  this->dev = dev;
+  RAISE (MSG_DEBUG, "ph create");
+  this->type = chip_type;
+  this->ctrl = ctrl;
   this->hwconfig = 0; /* No hardware config for PH */
   this->hwstate = 0;  /* pcbn=0, slave=0 */
 }
@@ -201,25 +201,23 @@ void TVPhCreate (TVEncoderObj *this, TVChip chip, I2CDevPtr dev)
 
 void TVPhSetPort (TVEncoderObj *this, int port)
 {
-  DPRINTF ("ph port %08X\n", port);
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
 
+  RAISE (MSG_DEBUG, "ph port %08X", port);
   if ((port & PORT_FORMAT_MASK) != PORT_FORMAT_RGB) {
-    ERROR ("TVPh1SetPort: Only RGB multiplex format supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVPh1SetPort: Only RGB multiplex format supported.");
   }
   if ((port & PORT_PCLK_MODE) != PORT_PCLK_MASTER) {
-    ERROR ("TVPh1SetPort: Only clock master mode supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVPh1SetPort: Only clock master mode supported.");
   }
   if ((port & PORT_BLANK_DIR) != PORT_BLANK_OUT) {
-    ERROR ("TVPh1SetPort: Only output blank supported.\n");
-    exit (1);
+    RAISE (MSG_ABORT, "TVPh1SetPort: Only output blank supported.");
   }
   this->hwstate = ((port & PORT_BLANK_POLARITY) ? SetBit(6) : 0)
                 | ((port & PORT_SYNC_DIR) ? SetBit(5) : 0);
-  TVWriteBus (this->dev, 0x96, SetBitField(this->hwstate,6:5,6:5));
+  TVWriteBus (dev, 0x96, SetBitField(this->hwstate,6:5,6:5));
   /* FIXME Test sync polarity - this varies wildly in the observed modes. */
-  TVWriteBus (this->dev, 0x97, 0x20 /* ofs=1, hfs=vfs=0 */
+  TVWriteBus (dev, 0x97, 0x20 /* ofs=1, hfs=vfs=0 */
 	      | ((port & PORT_SYNC_DIR) ? 0 : SetBit(1) | SetBit(3))
 	      | ((port & PORT_VSYNC_POLARITY) ? 0 : SetBit(2))
 	      | ((port & PORT_HSYNC_POLARITY) ? 0 : SetBit(0)));
@@ -227,25 +225,28 @@ void TVPhSetPort (TVEncoderObj *this, int port)
 
 void TVPh1SetPort (TVEncoderObj *this, int port)
 {
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
+
   TVPhSetPort (this, port);
   if ((port & PORT_PCLK_POLARITY) == PORT_PCLK_HIGH) {
-    TVWriteBus (this->dev, 0x54, 0x01); /* edge1: falling, edge2: rising */
+    TVWriteBus (dev, 0x54, 0x01); /* edge1: falling, edge2: rising */
   } else {
-    TVWriteBus (this->dev, 0x54, 0x02); /* edge1: raising, edge2: falling */
+    TVWriteBus (dev, 0x54, 0x02); /* edge1: raising, edge2: falling */
   }
 }
 
 void TVPhGetPort (TVEncoderObj *this, int *port)
 {
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte res;
 
-  DPRINTF ("ph get port\n");
+  RAISE (MSG_DEBUG, "ph get port");
   *port = PORT_FORMAT_RGB | PORT_PCLK_MASTER | PORT_PCLK_HIGH | 
           PORT_BLANK_OUT;
-  TVReadBus (this->dev, 0x96, &res);
+  TVReadBus (dev, 0x96, &res);
   *port |= (GetBit(res,6) ? PORT_BLANK_HIGH : PORT_BLANK_LOW) 
         |  (GetBit(res,5) ? PORT_SYNC_SLAVE : PORT_SYNC_MASTER);
-  TVReadBus (this->dev, 0x97, &res); 
+  TVReadBus (dev, 0x97, &res); 
   *port |= ((GetBit(res,1)|GetBit(res,3)) ? PORT_SYNC_OUT : PORT_SYNC_IN)
         |  (GetBit(res,2) ? PORT_VSYNC_LOW : PORT_VSYNC_HIGH) 
         |  (GetBit(res,0) ? PORT_HSYNC_LOW : PORT_HSYNC_HIGH);
@@ -253,20 +254,21 @@ void TVPhGetPort (TVEncoderObj *this, int *port)
 
 void TVPh1GetPort (TVEncoderObj *this, int *port)
 {
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte res;
 
-  DPRINTF ("ph1 get port\n");
+  RAISE (MSG_DEBUG, "ph1 get port");
   TVPhGetPort (this, port);
-  TVReadBus (this->dev, 0x54, &res); 
+  TVReadBus (dev, 0x54, &res); 
   *port |= ((res & 0x3) == 0x02 ? PORT_PCLK_LOW : PORT_PCLK_HIGH);
 }
 
 void TVPhInitRegs (TVEncoderObj *this)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   int i;
 
-  DPRINTF ("ph init %04X\n", this->chip);
+  RAISE (MSG_DEBUG, "ph init %04X", this->type);
   for (i = 0x01; i <= 0x16; i++) {
     TVWriteBus (dev, i, 0x00);  /* NV: 0x01 - 0x16 */ 
   }
@@ -308,9 +310,9 @@ void TVPhInitRegs (TVEncoderObj *this)
 
 void TVPh1InitRegs (TVEncoderObj *this, int port)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
 
-  DPRINTF ("ph1 init %04X\n", this->chip);
+  RAISE (MSG_DEBUG, "ph1 init %04X", this->type);
   TVPh1SetPort (this, port);
   TVPhInitRegs (this);
   TVWriteBus (dev, 0x37, 0x00); /* NV */ 
@@ -371,11 +373,11 @@ void TVPh1InitRegs (TVEncoderObj *this, int port)
 
 void TVPh1SetRegs (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   TVPhMacro *m;
   int blckl, blnnl;
 
-  DPRINTF ("nv set ph %04X\n", this->chip);
+  RAISE (MSG_DEBUG, "nv set ph %04X", this->type);
   m = TVMacroPhRegs (r);
   blckl = r->ph1.blckl;
   blnnl = r->ph1.blnnl;
@@ -487,7 +489,7 @@ void TVPh1SetRegs (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 
 void TVPh1GetRegs (TVEncoderObj *this, TVEncoderRegs *r)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte regs[0x100];
   int i;
 
@@ -556,7 +558,7 @@ void TVPh1GetRegs (TVEncoderObj *this, TVEncoderRegs *r)
 
 void TVPh1SetState (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 {
-  register I2CDevPtr dev = this->dev;
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
 
   /* FIXME */
   /* 0x6e blckon, ... */
@@ -584,10 +586,11 @@ void TVPh1SetState (TVEncoderObj *this, TVEncoderRegs *r, TVState state)
 
 long TVPh1GetStatus (TVEncoderObj *this, int index)
 { 
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   I2CByte fifo, status;
 
-  TVReadBus (this->dev, 0x00, &status); /* FIFO status */
-  TVReadBus (this->dev, 0x80, &fifo); /* FIFO status */
+  TVReadBus (dev, 0x00, &status); /* FIFO status */
+  TVReadBus (dev, 0x80, &fifo); /* FIFO status */
   
   return SetBitField (status,1:0,1:0) | SetBitField (fifo,1:0,3:2) |
     ((tvState == TV_ON) ? 0x100 : 0);
@@ -595,17 +598,18 @@ long TVPh1GetStatus (TVEncoderObj *this, int index)
 
 TVConnect TVPh1GetConnect (TVEncoderObj *this)
 {
+  register I2CDevPtr dev = (I2CDevPtr) this->ctrl;
   TVConnect status = CONNECT_BOTH;
   I2CByte msense;
 
   /* Enable monitor sensing temporarily */
-  TVWriteBus (this->dev, 0x1a, 0x2c); /* NV */
-  TVWriteBus (this->dev, 0x1b, 0x80);
+  TVWriteBus (dev, 0x1a, 0x2c); /* NV */
+  TVWriteBus (dev, 0x1b, 0x80);
 
   /* Read monitor sense data */
-  TVReadBus (this->dev, 0x1b, &msense); /* Monitor sense */
+  TVReadBus (dev, 0x1b, &msense); /* Monitor sense */
 
-  DPRINTF("TVPh1GetConnect %02X\n", msense);
+  RAISE (MSG_DEBUG, "TVPh1GetConnect %02X", msense);
 
   switch (msense & 0x7) { /* 0=active, 1=inactive */
     case 0x06: 
@@ -618,7 +622,7 @@ TVConnect TVPh1GetConnect (TVEncoderObj *this)
       status = CONNECT_CONVERT;
       break;
     case 0x07:
-      ERROR("Warning: No TV seems to be connected\n");
+      RAISE (MSG_WARNING, "No TV seems to be connected.");
       /* no break */
     default:
       status = CONNECT_BOTH;
@@ -626,7 +630,7 @@ TVConnect TVPh1GetConnect (TVEncoderObj *this)
   }
 
   /* Disable monitor sensing */
-  TVWriteBus (this->dev, 0x1b, 0x00 );
+  TVWriteBus (dev, 0x1b, 0x00 );
 
   return status;
 }
@@ -656,32 +660,12 @@ char *TVDetectPhilips (I2CDevPtr dev, TVChip *encoder)
 #endif
 #endif
 
-#if 0 //// FIXME REMOVE
-  tvBusOk = TRUE;  
-  TVWriteBus (dev, 0xff, 0x00);
-  DPRINTF ("ph check LUT (%s)\n", tvBusOk ? "ok" : "err");
-  if (!tvBusOk) return NULL;
-  /* If we're here, it's not a BT, but may be an CX, in either read mode. */
-  tvBusOk = TRUE;  
-  TVWriteBus (dev, 0x01, 0x00); /* should be zero according to docs, anyway */
-  DPRINTF ("ph check 0x01 (%s)\n", tvBusOk ? "ok" : "err");
-  if (!tvBusOk) return NULL;
-  TVReadBus (dev, 0x00, &ver); 
-  TVReadBus (dev, 0x01, &res); 
-  DPRINTF ("ph check %02X %02X (%s)\n", ver, res, tvBusOk ? "ok" : "err");
-  if (res != 0x00 || ver == res) return NULL;
-  /* Do it again, in case there were just changes it status */
-  TVReadBus (dev, 0x00, &ver); 
-  TVReadBus (dev, 0x01, &res); 
-  DPRINTF ("ph check %02X %02X (%s)\n", ver, res, tvBusOk ? "ok" : "err");
-  if (res != 0x00 || ver == res) return NULL;
-#endif
-
   tvBusOk = TRUE;  
   TVReadBus (dev, 0x00, &ver); 
   TVReadBus (dev, 0x1c, &cid); 
   ver = (ver & 0xe0 ) >> 5; /* 3 highest bits are version info */
-  DPRINTF ("ph check ver=%01x cid=%02x (%s)\n", ver, cid, tvBusOk?"ok":"err");
+  RAISE (MSG_DEBUG, "ph check ver=%01x cid=%02x (%s)", ver, cid, 
+	 tvBusOk?"ok":"err");
   *encoder = TV_PHILIPS;
   if (ver != 0x2 && ver != 0x5) {
     snprintf (version, 50, "Unknown Philips #%i (%1s:%02X)", ver, I2C_ID(dev));
@@ -689,8 +673,8 @@ char *TVDetectPhilips (I2CDevPtr dev, TVChip *encoder)
   }
   if (!tvBusOk) return NULL;
   /* Probe addr 0x40 and 0x42 to identify SAA7108, SAA7109 */
-  if (xf86I2CProbeAddress(dev->pI2CBus, 0x40) || 
-      xf86I2CProbeAddress(dev->pI2CBus, 0x42)) {
+  if (TVProbeBus (dev->pI2CBus, 0x40) || 
+      TVProbeBus (dev->pI2CBus, 0x42)) {
     switch (cid) {
       case 0x02: name = "7108";  *encoder = TV_PHILIPS_7108; break;
       case 0x03: name = "7109";  *encoder = TV_PHILIPS_7109; break;
@@ -719,50 +703,8 @@ char *TVDetectPhilips (I2CDevPtr dev, TVChip *encoder)
   }
 }
 
-#ifdef NVTV_DEBUG
-int PhilipsRegs[] =
-{
-  0x00,
-  0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c,
-  0x22,
-  0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b ,0x2c, 0x2d,
-  0x38, 0x39, 0x3a,
-  0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x5b, 0x5c, 0x5d, 0x5e, 0x5f,
-  0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e, 0x6f,
-  0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x7b, 0x7c, 0x7d, 0x7e, 0x7f,
-  0x80, 0x81, 0x82, 0x83, 0x84,
-  0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
-  0xa0, 0xa1, 0xa2, 0xa3, 0xa4,
-  0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff,
-};
-
-void PhilipsRegDump (I2CDevPtr dev)
-{
-  I2CByte value;
-  int i,j;
-  char regstring[50];
-  char binary[32];
-  
-  DPRINTF("Philips register dump:\n" );
-  DPRINTF("[Reg] [dec] [bin(7-0)]\n" );
-  for( i = 0; i < ( sizeof(PhilipsRegs) / sizeof(PhilipsRegs[0])); i++ ) {
-    TVReadBus (dev, PhilipsRegs[i], &value);
-    
-    /* Convert to binary */
-    for(j=0;j < 8;j++) {
-      if( ( value & ( 0x01 << j ) ) != 0 ) binary[7 - j] = '1';
-      else binary[7 - j] = '0';
-    }
-    binary[8] = '\0';
-    
-    snprintf( regstring, 50, "0x%02x:  %03d   %s\n", PhilipsRegs[i], value, binary );
-    DPRINTF( regstring );
-  }
-}
-#endif /* NVTV_DEBUG */
-
 TVEncoderObj tvPh1Template = {
-  chip: TV_PHILIPS_MODEL1, dev: NULL, minClock: 10000, maxClock: 44000, 
+  type: TV_PHILIPS_MODEL1, ctrl: NULL, minClock: 10000, maxClock: 44000, 
   Create:     TVPhCreate,
   InitRegs:   TVPh1InitRegs, 
   SetRegs:    TVPh1SetRegs, 
