@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tv_tdfx.c,v 1.6 2002/03/24 17:42:48 dthierbach Exp $
+ * $Id: tv_tdfx.c,v 1.22 2004/01/27 17:05:26 dthierbach Exp $
  *
  * Contents:
  *
@@ -230,50 +230,17 @@ VSyncEnd     ---   ---   ---   ---   ---   ---   ---  11[3] 11[2] 11[1] 11[0]
 #define TDFX_WR8(p,i,d)  MMIO_OUT8(((p)->MMIOBase[0]), (i), (d))
 #define TDFX_RD8(p,i)    MMIO_IN8(((p)->MMIOBase[0]), (i))
 
-/* FIXME: Put that somewhere else */
-/* from xfree common/compiler.h */
-
-#ifdef linux 
-#include <sys/io.h>
-
-#define OUTB(p,v) outb(v,p)
-#define INB(p) inb(p)
-
-#else /* FIXME: we won't need that for now under Windows */
-
-#define OUTB(p,v)
-#define INB(p) 0
-
-#endif 
-
-#if 0 /* Linux def, maybe reusable for Cygwin ... */
-
-static __inline void
-outb (unsigned char value, unsigned short int port)
-{
-  __asm__ __volatile__ ("outb %b0,%w1": :"a" (value), "Nd" (port));
-}
-
-static __inline unsigned char
-inb (unsigned short int port)
-{
-  unsigned char _v;
-
-  __asm__ __volatile__ ("inb %w1,%0":"=a" (_v):"Nd" (port));
-  return _v;
-}
-
-#endif
-
 /* FIXME: variable port addr */
 
-CARD8 readCrtTdfx (TDFXPtr hwp, CARD8 index)
+CARD8 
+readCrtTdfx (TDFXPtr hwp, CARD8 index)
 {
   OUTB (hwp->PIOBase[0] + hwp->IOBase + VGA_CRTC_INDEX_OFFSET, index);
   return INB (hwp->PIOBase[0] + hwp->IOBase + VGA_CRTC_DATA_OFFSET);
 }
 
-void writeCrtTdfx (TDFXPtr hwp, CARD8 index, CARD8 value)
+void 
+writeCrtTdfx (TDFXPtr hwp, CARD8 index, CARD8 value)
 {
   OUTB (hwp->PIOBase[0] + hwp->IOBase + VGA_CRTC_INDEX_OFFSET, index);
   OUTB (hwp->PIOBase[0] + hwp->IOBase + VGA_CRTC_DATA_OFFSET, value);
@@ -316,26 +283,34 @@ TDFXReadSeq (TDFXPtr hwp, CARD8 index)
 #define CYCLE_DELAY	10
 #define TIMEOUT		50
 
-static void TDFXTvI2CGetBits(I2CBusPtr b, int *clock, int *data)
+/* Bus private structure. Necessary, even though it has only one element,
+   because unified destruction won't work otherwise. */
+
+typedef struct {
+  TDFXPtr pTdfx;
+} TDFXTvI2CRec, *TDFXTvI2CPtr;
+
+static void 
+TDFXTvI2CGetBits(I2CBusPtr b, int *clock, int *data)
 {
-  TDFXPtr pTDFX = TDFXPTR(xf86Screens[b->scrnIndex]);
+  TDFXTvI2CPtr priv = (TDFXTvI2CPtr) b->DriverPrivate.ptr;
+  TDFXPtr pTdfx = priv->pTdfx;
   register CARD32 val;
 
-  val = TDFX_RD32(pTDFX, VIDSERIALPARALLELPORT);
+  val = TDFX_RD32(pTdfx, VIDSERIALPARALLELPORT);
 
   *clock = (val & I2C_SCL_IN) != 0;
   *data  = (val & I2C_SDA_IN) != 0;
-
-  DEBUG(ErrorF("TDFXTvI2CGetBits(%p) val=0x%x, returns clock %d, data %d\n",
-	       b, val, *clock, *data));
 }
 
-static void TDFXTvI2CPutBits(I2CBusPtr b, int clock, int data)
+static void 
+TDFXTvI2CPutBits(I2CBusPtr b, int clock, int data)
 {
-  TDFXPtr pTDFX = TDFXPTR(xf86Screens[b->scrnIndex]);
+  TDFXTvI2CPtr priv = (TDFXTvI2CPtr) b->DriverPrivate.ptr;
+  TDFXPtr pTdfx = priv->pTdfx;
   register CARD32 val;
 
-  val = TDFX_RD32(pTDFX, VIDSERIALPARALLELPORT);
+  val = TDFX_RD32(pTdfx, VIDSERIALPARALLELPORT);
   if (clock)
     val |= I2C_SCL_OUT;
   else
@@ -346,20 +321,23 @@ static void TDFXTvI2CPutBits(I2CBusPtr b, int clock, int data)
   else
     val &= ~I2C_SDA_OUT;
   
-  TDFX_WR32(pTDFX, VIDSERIALPARALLELPORT, val);
-  
-  DEBUG(ErrorF("TDFXTvI2CPutBits(%p, %d, %d) val=0x%x\n", 
-	       b, clock, data, val));
+  TDFX_WR32(pTdfx, VIDSERIALPARALLELPORT, val);
 }
 
-I2CBusPtr TDFXTvI2CInit (ScrnInfoPtr pScrn, char *name, int private)
+I2CBusPtr 
+TDFXTvBusCreate (TDFXPtr pTdfx, char *name)
 {
   I2CBusPtr I2CPtr;
+  TDFXTvI2CPtr priv;
 
   I2CPtr = xf86CreateI2CBusRec();
+  priv = xcalloc (1, sizeof (TDFXTvI2CRec));
+  if (priv) {
+    priv->pTdfx = pTdfx;
+  }
   if (I2CPtr) {
     I2CPtr->BusName    = name;
-    I2CPtr->scrnIndex  = pScrn->scrnIndex;
+    I2CPtr->scrnIndex  = -1;
     I2CPtr->I2CPutBits = TDFXTvI2CPutBits;
     I2CPtr->I2CGetBits = TDFXTvI2CGetBits;
     /* Not sure about the delays. lm_sensors has udelay = mdelay = CYCLE = 10,
@@ -376,33 +354,35 @@ I2CBusPtr TDFXTvI2CInit (ScrnInfoPtr pScrn, char *name, int private)
     I2CPtr->AcknTimeout  = 50; /* usec, default: 5 */
     I2CPtr->StartTimeout = 50; /* usec, default: 5 */
     I2CPtr->RiseFallTime = 10; /* usec, default: 2 */
-    I2CPtr->DriverPrivate.val = private; 
+    I2CPtr->DriverPrivate.ptr = priv; 
   }
-  return I2CPtr;
+  if (I2CPtr && priv && xf86I2CBusInit (I2CPtr)) {
+    return I2CPtr;
+  }
+  if (I2CPtr && priv) {
+    RAISE (MSG_ERROR, "Bus %s is not unique", name);
+  } else {
+    RAISE (MSG_ERROR, "Out of memory");
+  }
+  TVDestroyBus (I2CPtr);
+  return NULL;
 }
 
-Bool TDFXTvBusInit (ScrnInfoPtr pScrn)
+Bool 
+TDFXTvBusInit (TDFXPtr pTdfx)
 {
-  Bool ok;
-  TDFXPtr pTdfx = TDFXPTR(pScrn);
-
   pTdfx->TvChain = NULL;
   /* Must be named TV0, TV1, etc.; otherwise I2C_ID won't work */
-  pTdfx->TvBus = TDFXTvI2CInit (pScrn, "TV0", 0);
-  
-  ok = TRUE;
-  if (!xf86I2CBusInit(pTdfx->TvBus)) {
-    ErrorF ("Cannot init bus\n");
-    ok = FALSE;
-  }
-  return ok;
+  pTdfx->TvBus = TDFXTvBusCreate (pTdfx, "TV0");
+  return (pTdfx->TvBus != NULL);
 }
 
-void TDFXTvBusConfig (TDFXPtr pTdfx)
+void 
+TDFXTvBusConfig (TDFXPtr pTdfx)
 {
   register CARD32 val;
 
-  DPRINTF ("tdfx bus config\n");
+  RAISE (MSG_DEBUG, "tdfx bus config");
   val = TDFX_RD32(pTdfx, VIDINFORMAT);
   val &= ~SetBit(12)  /* TV vsync active high (as for nv) */
       &  ~SetBit(13)  /* TV hsync active high (as for nv) */
@@ -424,7 +404,8 @@ void TDFXTvBusConfig (TDFXPtr pTdfx)
 
 /* -------- CRT -------- */
 
-void TDFXSetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
+void 
+TDFXSetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
 {
   int horizDisplay    = (r->HDisplay/8)    - 1;
   int horizBlankStart = (r->HBlankStart/8) - 1;
@@ -441,7 +422,7 @@ void TDFXSetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
 
   /* horizSkew = (r->HSkew/8) + 1; */
 
-  DPRINTF ("TDFXSetCrtRegs %i %i\n", r->HDisplay, r->VDisplay);
+  RAISE (MSG_DEBUG, "TDFXSetCrtRegs %i %i", r->HDisplay, r->VDisplay);
 #ifdef FAKE_CRTC
   FPRINTF ("\ncrt[");
 #endif
@@ -505,7 +486,8 @@ void TDFXSetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
 #endif
 }
 
-void TDFXGetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
+void 
+TDFXGetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
 {
   int horizDisplay;
   int horizBlankStart;
@@ -520,7 +502,7 @@ void TDFXGetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
   int vertBlankEnd;
   int vertTotal;
 
-  DPRINTF ("TDFXGetCrtRegs [");
+  RAISE (MSG_DEBUG, "TDFXGetCrtRegs [");
   horizTotal      = Set8Bits(readCrtTdfx(pTdfx, 0x00)) 
                   | SetBitField(readCrtTdfx(pTdfx, 0x1a),0:0,8:8);
   horizDisplay    = Set8Bits(readCrtTdfx(pTdfx, 0x01)) 
@@ -587,16 +569,17 @@ void TDFXGetCrtRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
 
   r->clock = 0;  
 
-  DPRINTF ("%i %i %i %i, %i %i %i %i]\n", 
+  RAISE (MSG_DEBUG, "%i %i %i %i, %i %i %i %i]", 
 	   r->HDisplay, r->HSyncStart, r->HSyncEnd, r->HTotal,
 	   r->VDisplay, r->VSyncStart, r->VSyncEnd, r->VTotal);
 }
 
-void TDFXSetTvRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
+void 
+TDFXSetTvRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
 {
   register CARD32 val;
 
-  DPRINTF ("tdfx set extra v=%i/%i h=%i/%i\n",
+  RAISE (MSG_DEBUG, "tdfx set extra v=%i/%i h=%i/%i",
 	   r->tvVBlankStart, r->tvVBlankEnd, 
 	   r->tvHBlankStart, r->tvHBlankEnd);
   TDFX_WR32(pTdfx, 0x3c, SetBitField (r->tvVBlankStart,10:0,10:0)
@@ -616,9 +599,10 @@ void TDFXSetTvRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
   TDFX_WR32(pTdfx, 0x24, val);
 }
 
-void TDFXGetTvRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
+void 
+TDFXGetTvRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
 {
-  DPRINTF ("tdfx get extra\n");
+  RAISE (MSG_DEBUG, "tdfx get extra");
   r->tvVBlankStart = SetBitField (TDFX_RD32 (pTdfx, 0x3c),10:0,10:0);
   r->tvVBlankEnd   = SetBitField (TDFX_RD32 (pTdfx, 0x3c),26:16,10:0);
   r->tvHBlankStart = SetBitField (TDFX_RD32 (pTdfx, 0x74),10:0,10:0);
@@ -628,7 +612,8 @@ void TDFXGetTvRegs (TDFXPtr pTdfx, TVTdfxRegs *r)
   r->tvLatency     = SetBitField (TDFX_RD32 (pTdfx, 0x24),19:16,3:0);
 }
 
-void TDFXSetAllRegs (TDFXPtr pTdfx, TVRegs *r)
+void 
+TDFXSetAllRegs (TDFXPtr pTdfx, TVRegs *r)
 {
   /* Maybe CRT regs are not needed for TV ? */
   if ((r->devFlags & DEV_MONITOR) || 
@@ -681,7 +666,8 @@ void TDFXGetAllRegs (TDFXPtr pTdfx, TVRegs *r)
 
 /* -------- Device enable/disable -------- */
 
-int TDFXGetDevFlags (TDFXPtr pTdfx)
+int 
+TDFXGetDevFlags (TDFXPtr pTdfx)
 {
   if (GetBit (readCrtTdfx (pTdfx, 0x70), 15)) {
     return DEV_TELEVISION;
@@ -692,48 +678,29 @@ int TDFXGetDevFlags (TDFXPtr pTdfx)
 
 /* -------- Device routines -------- */
 
-void TDFXSetTvDevice (TDFXPtr pTdfx, I2CChainPtr chain, Bool init)
+void 
+TDFXSetTvDevice (TDFXPtr pTdfx, I2CChainPtr chain, Bool init)
 {
-  DPRINTF ("tdfx set dev %p %i\n", chain, init);
-  TVSetTvEncoder (&pTdfx->tvEncoder, chain);
+  RAISE (MSG_DEBUG, "tdfx set dev %p %i", chain, init);
+  TVSetTvEncoder (&pTdfx->tvEncoder, chain, NULL);
   tvBusOk = TRUE;
   pTdfx->tvEncoder.InitRegs (&pTdfx->tvEncoder, PORT_TDFX);
 }
 
-/*
- * Free chain and device records of all tv chips
- */
-
-void TDFXDestroyDevices (TDFXPtr pTdfx)
-{
-  if (!pTdfx) return;
-  if (pTdfx->TvChain) TVDestroyChain (pTdfx->TvChain);
-  pTdfx->TvChain = NULL;
-  TVDestroyDevices (&pTdfx->TvBus, 1);
-}
-
-/*
- * Free chain, device records, and all busses
- */
-
-void TDFXDestroyBusses (TDFXPtr pTdfx)
-{
-  if (!pTdfx) return;
-  TDFXDestroyDevices (pTdfx);
-  TVDestroyBusses (&pTdfx->TvBus, 1);
-}
-
-void TDFXProbeTvDevices (TDFXPtr pTdfx)
+void 
+TDFXProbeTvDevices (TDFXPtr pTdfx)
 {
   if (!pTdfx) return;
   /* TDFXUpdateTvState (pTdfx); */
   tvState = TV_OFF;
-  TVProbeCreateKnown (&pTdfx->TvBus, 1, &pTdfx->TvChain); 
+  TVCheckChain (pTdfx->TvChain);
+  pTdfx->TvChain = TVProbeCreateKnown (&pTdfx->TvBus, 1, NULL); 
 }
 
 /* -------- -------- */
 
-void TDFXUpdateTvState (TDFXPtr pTdfx)
+void 
+TDFXUpdateTvState (TDFXPtr pTdfx)
 {
 #ifndef FAKE_MMIO
   if (GetBit (TDFX_RD32(pTdfx, VIDINFORMAT), 15)) {
@@ -744,10 +711,11 @@ void TDFXUpdateTvState (TDFXPtr pTdfx)
 #endif
 }
 
-void TDFXSetTestImage (TDFXPtr pTdfx, TVEncoderRegs *r)
+void 
+TDFXSetTestImage (TDFXPtr pTdfx, TVEncoderRegs *r)
 {
   /* FIXME chip registers ... */
-  DPRINTF ("tdfx tv testimage\n");
+  RAISE (MSG_DEBUG, "tdfx tv testimage");
   TDFXUpdateTvState (pTdfx);
   if (tvState != TV_ON) {
     /* Only do bars if not in tv-mode */
@@ -755,7 +723,8 @@ void TDFXSetTestImage (TDFXPtr pTdfx, TVEncoderRegs *r)
   }
 }
 
-void TDFXSetTvMode (TDFXPtr pTdfx, TVRegs *r)
+void 
+TDFXSetTvMode (TDFXPtr pTdfx, TVRegs *r)
 {
   register CARD32 val;
   TVState oldState, newState; /* hack for now, FIXME */
@@ -763,10 +732,10 @@ void TDFXSetTvMode (TDFXPtr pTdfx, TVRegs *r)
   TDFXUpdateTvState (pTdfx);
   oldState = tvState;
   newState = (r->devFlags & DEV_TELEVISION) ? TV_ON : TV_OFF;
-  DPRINTF ("tdfx set tv mode %i -> %i\n", oldState, newState);
+  RAISE (MSG_DEBUG, "tdfx set tv mode %i -> %i", oldState, newState);
   if (oldState == TV_ON && newState != TV_ON) 
   {
-    DPRINTF ("tdfx set tv off\n");
+    RAISE (MSG_DEBUG, "tdfx set tv off");
     val = TDFX_RD32(pTdfx, VIDINFORMAT);
     val &= ~SetBit(15)   /* TV interface enable off */
 	&  ~SetBit(16)   /* Genlock enable off */
@@ -784,7 +753,7 @@ void TDFXSetTvMode (TDFXPtr pTdfx, TVRegs *r)
     pTdfx->tvEncoder.SetRegs (&pTdfx->tvEncoder, &r->enc, newState);
   }
   if (oldState != TV_ON && newState == TV_ON) {
-    DPRINTF ("tdfx set tv on %08lX\n", val);
+    RAISE (MSG_DEBUG, "tdfx set tv on %08lX", val);
     val = TDFX_RD32(pTdfx, VIDINFORMAT);
     val |= SetBit(15)   /* TV interface enable on */
 	|  SetBit(16)   /* Genlock enable on */
@@ -812,7 +781,8 @@ void TDFXSetTvMode (TDFXPtr pTdfx, TVRegs *r)
   TDFX_OR32 (pTdfx, 0x5c, SetBit(0));
 }
 
-long TDFXGetTvStatus (TDFXPtr pTdfx, int index)
+long 
+TDFXGetTvStatus (TDFXPtr pTdfx, int index)
 
 {
   long result; 
@@ -823,7 +793,8 @@ long TDFXGetTvStatus (TDFXPtr pTdfx, int index)
   return result;
 }
 
-TVConnect TDFXGetTvConnect (TDFXPtr pTdfx)
+TVConnect 
+TDFXGetTvConnect (TDFXPtr pTdfx)
 {
   TVConnect result;
 
@@ -832,67 +803,3 @@ TVConnect TDFXGetTvConnect (TDFXPtr pTdfx)
   result = pTdfx->tvEncoder.GetConnect (&pTdfx->tvEncoder);
   return result;
 }
-
-/*
-
-lm_sensors:
-status,ntsc,res,half,depth
-
-detect:
-c4 = 0
-
-init:
-ba = 80
-// d6 = 00
-a0 = 80  // slave clock
-ba = 20
-c6 = 01 // depth = 16bpp
-c4 = 01
-c8 = 00
-// 640x480, ntsc=1, half=0, depth=16
-
-update: 640x480
-b8 = (!data->ntsc)
-a0 = 8c
-
-update: 800x600
-b8 = 2 + (!data->ntsc)
-a0 = 91
-
-update: depth=16
-c6=1
-
-update: depth=24
-c6=0
-
-d4 = half << 6
-ba = 20
-c6 = 01
-c4 = 01
-c4 = 01 | bars << 2
-c4 = 41 | bars << 2
-c4 = 81 | bars << 2
-c4 = c1 | bars << 2
-
-
- *  A0-7   en_xclk = 1 (for slave)
- *  C4-0   en_out = 1 (!) (in slave!)
- *  C6-7   en_blanko = 0 (blank in/not out)
- *  C6-6   en_dot = 0 (!) (default) (internal blanking)
- *  C6-5   fieldi = 0
- *  C6-4   vsynci = 0 (active low) (default 0) 
- *  C6-3   hsynci = 0 (active low) (default 0) 
- *  C6-2:0 in_mode = 1 (16-bit RGB multiplexed) (should be 0!)
-
-VIDINFORMAT = 0x00008160
-
- 5 = 1 (vmi_vsync active low)
- 6 = 1 (vmi_hsync active low)
- 8 = 1 (posegde BT)
-12 = 0 (tv_vsync active high)
-13 = 0 (tv_hsync active high)
-15 = 1 TV interface enable
-16 = 0 Genlock disable (slave)
-17 = 0 Timing from VGA
-
-*/
