@@ -476,17 +476,18 @@
 #define PCI_CHIP_BT848		0x0350
 #define PCI_CHIP_BT849		0x0351
 
-/* NVIDIA */
+/* NVIDIA */ 
 #define PCI_CHIP_NV1		0x0008
 #define PCI_CHIP_DAC64		0x0009
 #define PCI_CHIP_TNT		0x0020
 #define PCI_CHIP_TNT2		0x0028
 #define PCI_CHIP_UTNT2		0x0029
-#define PCI_CHIP_VTNT2		0x002C
+/* Unknown TNT2 = 0x002A */
+#define PCI_CHIP_VTNT2		0x002C 
 #define PCI_CHIP_UVTNT2		0x002D
 #define PCI_CHIP_TNT2_A		0x002E
 #define PCI_CHIP_TNT2_B		0x002F
-#define PCI_CHIP_ITNT2		0x00A0
+#define PCI_CHIP_ITNT2		0x00A0 /* Aladdin TNT2 */
 #define PCI_CHIP_GEFORCE256     0x0100
 #define PCI_CHIP_GEFORCEDDR     0x0101
 #define PCI_CHIP_QUADRO         0x0103
@@ -507,7 +508,8 @@
 #define PCI_CHIP_GEFORCE4GO_420_32M     0x0176 
 #define PCI_CHIP_GEFORCE4GO_460         0x0177 
 #define PCI_CHIP_QUADRO4XGL_500         0x0178 
-#define PCI_CHIP_GEFORCE4GO_440_64M     0x0179 
+#define PCI_CHIP_GEFORCE4GO_440_64M     0x0179 /* Mac */
+#define PCI_CHIP_GEFORCE4MX_MAC		0x0179
 #define PCI_CHIP_QUADRO4NVS             0x017A 
 #define PCI_CHIP_QUADRO4XGL_550         0x017B 
 #define PCI_CHIP_QUADRO4GOGL_500        0x017C 
@@ -515,7 +517,10 @@
 #define	PCI_CHIP_GEFORCE4MX_440_AGP8X   0x0181
 #define	PCI_CHIP_GEFORCE4MX_440SE_AGP8X 0x0182
 #define	PCI_CHIP_GEFORCE4MX_420_AGP8X   0x0183
+#define PCI_CHIP_GEFORCE4GO_448         0x0186
+#define PCI_CHIP_GEFORCE4GO_488         0x0187
 #define	PCI_CHIP_QUADRO4_XGL_580        0x0188
+#define PCI_CHIP_GEFORCE4MX_MAC_AGP8X   0x0189 /* Mac */
 #define	PCI_CHIP_QUADRO4_NVS_280        0x018A
 #define	PCI_CHIP_QUADRO4_XGL_380        0x018B
 #define PCI_CHIP_GEFORCE2GPU            0x01A0 
@@ -525,7 +530,8 @@
 #define PCI_CHIP_GEFORCE3TI_500         0x0202
 #define PCI_CHIP_QUADRO_DDC             0x0203
 #define PCI_CHIP_GEFORCE4TI_4600        0x0250 
-#define PCI_CHIP_GEFORCE4TI_4400        0x0251 
+#define PCI_CHIP_GEFORCE4TI_4400        0x0251
+/* 0x0252 */
 #define PCI_CHIP_GEFORCE4TI_4200        0x0253 
 #define PCI_CHIP_QUADRO4XGL_900         0x0258 
 #define PCI_CHIP_QUADRO4XGL_750         0x0259 
@@ -538,7 +544,54 @@
 #define PCI_CHIP_QUADRO4XGL_780         0x0289
 #define PCI_CHIP_QUADRO4GOGL_700        0x028C
 #define PCI_CHIP_GEFORCE3_MCPX	        0x02A0 /* XBox */
-
+#define PCI_CHIP_GEFORCEFX_5800ULTRA	0x0301
+#define PCI_CHIP_GEFORCEFX_5800		0x0302
+#define PCI_CHIP_QUADROFX_2000		0x0308
+#define PCI_CHIP_QUADROFX_1000		0x0309
+#define PCI_CHIP_GEFORCEFX_5600ULTRA	0x0311
+#define PCI_CHIP_GEFORCEFX_5600		0x0312
+/* 0x0313 */
+#define PCI_CHIP_GEFORCEFX_5600SE	0x0314
+/* 0x0316 */
+/* 0x0317 */
+#define PCI_CHIP_GEFORCEFXGO_5600	0x031A
+#define PCI_CHIP_GEFORCEFXGO_5650	0x031B
+#define PCI_CHIP_QUADROFXGO_700		0x031C
+/* 0x031D */
+/* 0x031E */
+/* 0x031F */
+#define PCI_CHIP_GEFORCEFX_5200_X	0x0320 /* ??? */
+#define PCI_CHIP_GEFORCEFX_5200ULTRA	0x0321
+#define PCI_CHIP_GEFORCEFX_5200		0x0322 /* ??? */
+#define PCI_CHIP_GEFORCEFX_5200SE	0x0323
+#define PCI_CHIP_GEFORCEFXGO_5200	0x0324
+#define PCI_CHIP_GEFORCEFXGO_5250	0x0325
+#define PCI_CHIP_GEFORCEFXGO_5200_32M	0x0328
+#define PCI_CHIP_GEFORCEFX_5200_MAC	0x0329
+/* 0x0329 */
+#define PCI_CHIP_QUADRO_NVS_280		0x032A
+#define PCI_CHIP_QUADROFX_500		0x032B 
+#define PCI_CHIP_GEFORCEFXGO_5300	0x032C
+#define PCI_CHIP_GEFORCEFXGO_5100	0x032D
+/* 0x032F */
+#define PCI_CHIP_GEFORCEFX_5900ULTRA	0x0330
+#define PCI_CHIP_GEFORCEFX_5900		0x0331
+#define PCI_CHIP_GEFORCEFX_5900XT	0x0332
+#define PCI_CHIP_GEFORCEFX_5950ULTRA	0x0333
+/* 0x0334 */
+#define PCI_CHIP_QUADROFX_3000		0x0338
+#define PCI_CHIP_GEFORCEFX_5700ULTRA	0x0341
+#define PCI_CHIP_GEFORCEFX_5700		0x0342
+#define PCI_CHIP_GEFORCEFX_5700LE	0x0343
+#define PCI_CHIP_GEFORCEFX_5700VE	0x0344
+/* 0x0345 */
+#define PCI_CHIP_GEFORCEFXGO_5700	0x0347 /* ??? */
+#define PCI_CHIP_GEFORCEFXGO_5700_X	0x0348 /* ??? */
+/* 0x0349 */
+/* 0x034B */
+#define PCI_CHIP_QUADROFXGO_1000	0x034C
+#define PCI_CHIP_QUADROFX_1100		0x034E
+/* 0x034F */
 
 /* NVIDIA & SGS */
 #define PCI_CHIP_RIVA128	0x0018
@@ -640,14 +693,20 @@
 #define PCI_CHIP_I815_BRIDGE		0x1130
 #define PCI_CHIP_I815			0x1132
 #define PCI_CHIP_I810_BRIDGE		0x7120
-#define PCI_CHIP_I810			0x7121
+#define PCI_CHIP_I810			0x7121 /* graphics */
 #define PCI_CHIP_I810_DC100_BRIDGE	0x7122
-#define PCI_CHIP_I810_DC100		0x7123
+#define PCI_CHIP_I810_DC100		0x7123 /* graphics */
 #define PCI_CHIP_I810_E_BRIDGE		0x7124
-#define PCI_CHIP_I810_E			0x7125
+#define PCI_CHIP_I810_E			0x7125 /* graphics */
 #define PCI_CHIP_I740_AGP		0x7800
 
-#define PCI_CHIP_I830		0x3577
+#define PCI_CHIP_I810_IG	0x1112 /* graphics */
+#define PCI_CHIP_I810_CFC	0x1132 /* graphics */ /* = I815?? */
+
+#define PCI_CHIP_I830		0x3577 /* graphics */
+#define PCI_CHIP_I845		0x2562 /* graphics */
+#define PCI_CHIP_I855		0x3582 /* graphics */
+#define PCI_CHIP_I865		0x2572 /* graphics */
 
 /* Silicon Motion Inc. */
 #define PCI_CHIP_SMI910		0x910
@@ -787,7 +846,7 @@ static SymTabRec xf86PCIVendorNameInfoData[] = {
 #endif
 
 /* Increase this as required */
-#define MAX_DEV_PER_VENDOR 64
+#define MAX_DEV_PER_VENDOR 100
 
 typedef struct {
     unsigned short VendorID;
@@ -1315,6 +1374,8 @@ static pciVendorDeviceInfo xf86PCIVendorInfoData[] = {
 	{PCI_CHIP_GEFORCE4MX_440_AGP8X,	"GeForce4 MX 440 with AGP8X",0},
 	{PCI_CHIP_GEFORCE4MX_440SE_AGP8X,"GeForce4 MX 440SE with AGP8X",0},
 	{PCI_CHIP_GEFORCE4MX_420_AGP8X,	"GeForce4 MX 420 with AGP8X",0},
+	{PCI_CHIP_GEFORCE4GO_448,	"GeForce4 448 Go",0},
+	{PCI_CHIP_GEFORCE4GO_488,	"GeForce4 488 Go",0},
 	{PCI_CHIP_QUADRO4_XGL_580,	"Quadro4 580 XGL",0},
 	{PCI_CHIP_QUADRO4_NVS_280,	"Quadro4 280 NVS",0},
 	{PCI_CHIP_QUADRO4_XGL_380,	"Quadro4 380 XGL",0},
@@ -1338,6 +1399,52 @@ static pciVendorDeviceInfo xf86PCIVendorInfoData[] = {
 	{PCI_CHIP_QUADRO4XGL_780,	"Quadro4 780 XGL",0},
 	{PCI_CHIP_QUADRO4GOGL_700,	"Quadro4 700 GoGL",0},
 	{PCI_CHIP_GEFORCE3_MCPX,	"GeForce3 Integrated MCPX GPU",0},
+	{PCI_CHIP_GEFORCEFX_5800ULTRA,	"GeForce FX 5800 Ultra",0}, 
+	{PCI_CHIP_GEFORCEFX_5800,	"GeForce FX 5800",0}, 
+	{PCI_CHIP_QUADROFX_2000,	"Quadro FX 2000",0}, 
+	{PCI_CHIP_QUADROFX_1000,	"Quadro FX 1000",0}, 
+	{PCI_CHIP_GEFORCEFX_5600ULTRA,	"GeForce FX 5600 Ultra",0}, 
+	{PCI_CHIP_GEFORCEFX_5600,	"GeForce FX 5600",0}, 
+/* 0x0313 */
+	{PCI_CHIP_GEFORCEFX_5600SE,	"GeForce FX 5600SE",0},
+/* 0x0316 */
+/* 0x0317 */
+	{PCI_CHIP_GEFORCEFXGO_5600,	"GeForce FX Go5600",0},
+	{PCI_CHIP_GEFORCEFXGO_5650,     "GeForce FX Go5650",0},
+	{PCI_CHIP_QUADROFXGO_700,	"Quadro FX Go700",0},
+/* 0x031D */
+/* 0x031E */
+/* 0x031F */
+	{PCI_CHIP_GEFORCEFX_5200_X,	"GeForce FX 5200",0}, /* ??? */
+	{PCI_CHIP_GEFORCEFX_5200ULTRA,  "GeForce FX 5200 Ultra",0}, 
+	{PCI_CHIP_GEFORCEFX_5200,	"GeForce FX 5200",0}, 
+	{PCI_CHIP_GEFORCEFX_5200SE,	"GeForce FX 5200SE",0},
+	{PCI_CHIP_GEFORCEFXGO_5200,	"GeForce FX Go5200",0},
+	{PCI_CHIP_GEFORCEFXGO_5250,	"GeForce FX Go5250",0},
+	{PCI_CHIP_GEFORCEFXGO_5200_32M,	"GeForce FX Go5200 32M/64M",0},
+/* 0x0329 */
+	{PCI_CHIP_QUADRO_NVS_280,	"Quadro NVS 280 PCI",0},
+	{PCI_CHIP_QUADROFX_500,	        "Quadro FX 500",0}, 
+	{PCI_CHIP_GEFORCEFXGO_5300,	"GeForce FX Go5300",0},
+	{PCI_CHIP_GEFORCEFXGO_5100,	"GeForce FX Go5100",0},
+/* 0x032F */
+	{PCI_CHIP_GEFORCEFX_5900ULTRA,	"GeForce FX 5900 Ultra",0},
+	{PCI_CHIP_GEFORCEFX_5900,	"GeForce FX 5900",0},
+	{PCI_CHIP_GEFORCEFX_5900XT,	"GeForce FX 5900XT",0},
+	{PCI_CHIP_GEFORCEFX_5950ULTRA,	"GeForce FX 5950 Ultra",0},
+/* 0x0334 */
+	{PCI_CHIP_QUADROFX_3000,	"Quadro FX 3000",0},
+	{PCI_CHIP_GEFORCEFX_5700ULTRA,	"GeForce FX 5700 Ultra",0},
+	{PCI_CHIP_GEFORCEFX_5700,	"GeForce FX 5700",0},
+	{PCI_CHIP_GEFORCEFX_5700LE,	"GeForce FX 5700LE",0},
+	{PCI_CHIP_GEFORCEFX_5700VE,	"GeForce FX 5700VE",0},
+/* 0x0345 */
+	{PCI_CHIP_GEFORCEFXGO_5700,	"GeForce FX Go5700",0}, /* ??? */
+	{PCI_CHIP_GEFORCEFXGO_5700_X,	"GeForce FX Go5700",0}, /* ??? */
+/* 0x0349 */
+/* 0x034B */
+	{PCI_CHIP_QUADROFXGO_1000,	"Quadro FX Go1000",0},
+	{PCI_CHIP_QUADROFX_1100,	"Quadro FX 1100",0},
 	{0x0000,			NULL,0}}},
     {PCI_VENDOR_IMS, {
 				{PCI_CHIP_IMSTT128,	"TwinTurbo 128", 0},
@@ -1598,8 +1705,13 @@ static pciVendorDeviceInfo xf86PCIVendorInfoData[] = {
 				{PCI_CHIP_I810,		"i810",0},
 				{PCI_CHIP_I810_DC100,	"i810-dc100",0},
 				{PCI_CHIP_I810_E,	"i810e",0},
+				{PCI_CHIP_I810_IG,	"i810-ig",0},
+/*				{PCI_CHIP_I810_CFC,	"i810-cfc",0}, */
 				{PCI_CHIP_I815,		"i815",0},
 				{PCI_CHIP_I830,		"i830",0},
+				{PCI_CHIP_I845,		"i845",0},
+				{PCI_CHIP_I855, 	"i855",0},
+				{PCI_CHIP_I865, 	"i865",0},
 				{0x0000,		NULL,0}}},
     {PCI_VENDOR_ADAPTEC, {
 				{0x0010, "2940U2",0 },
