@@ -8,11 +8,14 @@
 
 /* $XFree86: xc/programs/Xserver/hw/xfree86/i2c/xf86i2c.c,v 1.6 1999/06/12 15:37:08 dawes Exp $ */
 
+#include "local.h"
 #include "xfree.h"
+#include "xf86_ansic.h"
 #include <string.h>
-#define NULL ((void *)0)
 
 #if 0
+#define NULL ((void *)0)
+
 #include "misc.h"
 #include "xf86.h"
 #include "xf86_ansic.h"
@@ -31,7 +34,6 @@
 /* #else */
 typedef int Bool;
 typedef void *Pointer;
-#define X_DEFAULT 0
 #define TRUE  1
 #define FALSE 0
 #endif
